/*
 *
 *	File: CmnError.h 
 *  Desc: Our standard error codes
 *  Authors: MrAndrew, Swap
 *  Date: 23.12.2002
 *  Update: 31.12.2002
 *
 */

#ifndef _CMNERROR_H_
  #define _CMNERROR_H_

  //   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
  //   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
  //  +---+-+-+-----------------------+-------------------------------+
  //  |Sev|C|R|     Facility          |               Code            |
  //  +---+-+-+-----------------------+-------------------------------+
  //
  //  Sev - is the severity code
  //    00 - Success
  //    01 - Informational
  //    10 - Warning
  //    11 - Error
  //
  //  C - is the Customer code flag for application defined error codes
  //
  //  R - is a reserved bit
  //
  //  Facility - is the facility code
  //
  //  Code - is the facility's status code
  
  //  E_CMN_BASE has Sev = 11 stating error and C = 1 standing for it is app defined error
  #define E_CMN_BASE					0xE0000000
  #define E_CMN_LOG_NOT_INIT			E_CMN_BASE + 1
  #define E_CMN_LOG_FILE_CREATE_FAIL	E_CMN_BASE + 2
  #define E_CMN_REG_CREATE_FAIL			E_CMN_BASE + 3
  #define E_CMN_REG_OPEN_FAIL			E_CMN_BASE + 4
  #define E_CMN_REG_WRITE_FAIL			E_CMN_BASE + 5
  #define E_CMN_REG_READ_FAIL			E_CMN_BASE + 6
  #define E_CMN_REG_TOO_LARGE_VALUE		E_CMN_BASE + 7
  #define E_CMN_REG_INVALID_TYPE		E_CMN_BASE + 8
  #define E_CMN_REG_INVALID_BUFFER		E_CMN_BASE + 9
  #define E_CMN_REG_INVALID_HKEY		E_CMN_BASE + 10
  #define E_CMN_REG_KEY_NOT_FOUND		E_CMN_BASE + 11
  #define E_CMN_REG_INVALID_RIGHTS		E_CMN_BASE + 12

  #define E_CMN_INVALID_PARAMETER		E_CMN_BASE + 15

  #define E_CMN_UNKNOWN			E_CMN_BASE + 0xFF

#endif