/*
 *
 * CmnFile.cpp - implementation
 * Date:	24.12.2002
 * Update:	26.12.2002
 * Authors:	Ondrej Burisin & Jiri Formanek
 *
 */

#include "stdafx.h"

#define COMMON_API __declspec(dllexport)
#include "CmnLog.h"
#include "CmnFile.h"
#include "CmnStr.h"

#pragma COMMENT("Return error codes if we don't read the whole data")

// Loads the whole file to the specified (BYTE) buffer in memory using filename in UNICODE char coding
COMMON_API DWORD cmnLoadFileFromPathW(LPCWSTR _lpcwszFile, LPBYTE _lpBuffer, LPDWORD _lpdwSize)
{
	DWORD dwRet = ERROR_INVALID_PARAMETER;

	if(_lpcwszFile)
	{
		HANDLE hFile = NULL;

		hFile = ::CreateFileW(_lpcwszFile, GENERIC_READ, FILE_SHARE_READ|FILE_SHARE_WRITE,
								 NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		
		if(INVALID_HANDLE_VALUE == hFile) { return ERROR_FILE_NOT_FOUND; }

		DWORD dwRealSize = ::GetFileSize(hFile, NULL);	// Get the size

		if(_lpBuffer && _lpdwSize)
		{
			if(*_lpdwSize < dwRealSize)
			{
				return ERROR_MORE_DATA;	// We simply don't have so much memory
			}

			DWORD dwBytesRead = 0;
			// We don't work on the file async
			BOOL bOK = ::ReadFile(hFile, (LPVOID)_lpBuffer, dwRealSize, &dwBytesRead, NULL);

			if(bOK && (dwRealSize == dwBytesRead))
			{
				dwRet = ERROR_SUCCESS;	// Set this func OK
			}
		}
		else
		{
			// The user wants just to know, how big the file is ...
			*_lpdwSize = dwRealSize;	// Set the size
			dwRet = ERROR_SUCCESS;	// Success
		}

		::CloseHandle(hFile);
	}

	return dwRet;
}

// Use the ANSI version of CreateFile to work with ANSI strings, otherwise if using the convention of
// converting the ANSI string to UNICODE and calling the LoadFileToBufferW won't work under Windows 95
// because the CreateFileW isn't implemented there
COMMON_API DWORD cmnLoadFileFromPathA(LPCSTR _lpcszFile, LPBYTE _lpBuffer, LPDWORD _lpdwSize)
{
	DWORD dwRet = ERROR_INVALID_PARAMETER;

	if(_lpcszFile)
	{
		HANDLE hFile = NULL;

		hFile = ::CreateFileA(_lpcszFile, GENERIC_READ, FILE_SHARE_READ|FILE_SHARE_WRITE,
								 NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		
		if(INVALID_HANDLE_VALUE == hFile) { return ERROR_FILE_NOT_FOUND; }

		DWORD dwRealSize = ::GetFileSize(hFile, NULL);	// Get the size

		if(_lpBuffer && _lpdwSize)
		{
			if(*_lpdwSize < dwRealSize)
			{
				return ERROR_MORE_DATA;	// We simply don't have so much memory
			}

			DWORD dwBytesRead = 0;
			// We don't work on the file async
			BOOL bOK = ::ReadFile(hFile, (LPVOID)_lpBuffer, dwRealSize, &dwBytesRead, NULL);

			if(bOK && (dwRealSize == dwBytesRead))
			{
				dwRet = ERROR_SUCCESS;	// Set this func OK
			}
		}
		else
		{
			// The user wants just to know, how big the file is ...
			*_lpdwSize = dwRealSize;	// Set the size
			dwRet = ERROR_SUCCESS;	// Success
		}

		::CloseHandle(hFile);
	}

	return dwRet;
}

// Loads the file identified by the HANDLE, if _lpBuffer == NULL then just returns the size of the buffer needed to
// store the file in _lpdwSize
// Removed bug with not moving the file pointer to the beginning of the file
COMMON_API DWORD cmnLoadFileFromHandle(HANDLE _hFile, LPBYTE _lpBuffer, LPDWORD _lpdwSize)
{
	DWORD dwRet = ERROR_INVALID_PARAMETER;

	if(INVALID_HANDLE_VALUE != _hFile)
	{
		// Move the file pointer to the beginning of the file
		::SetFilePointer(_hFile, 0, NULL, FILE_BEGIN);
		DWORD dwRealSize = ::GetFileSize(_hFile, NULL);	// Get the size

		if(_lpBuffer && _lpdwSize)
		{
			if(*_lpdwSize < dwRealSize)
			{
				return ERROR_MORE_DATA;	// We simply don't have so much memory
			}

			DWORD dwBytesRead = 0;
			// We don't work on the file async
			BOOL bOK = ::ReadFile(_hFile, (LPVOID)_lpBuffer, dwRealSize, &dwBytesRead, NULL);

			if(bOK && (dwRealSize == dwBytesRead))
			{
				dwRet = ERROR_SUCCESS;	// Set this func OK
			}
		}
		else
		{
			// The user wants just to know, how big the file is ...
			*_lpdwSize = dwRealSize;	// Set the size
			dwRet = ERROR_SUCCESS;	// Success
		}
	}

	return dwRet;
}

COMMON_API HANDLE cmnOpenExistingFileW(LPCWSTR _lpcwszFile, DWORD _dwDesiredAccess, DWORD _dwShareMode)
{
	HANDLE hRet = INVALID_HANDLE_VALUE;

	if(_lpcwszFile)
	{
		hRet = ::CreateFileW(_lpcwszFile, _dwDesiredAccess, _dwShareMode, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL,
							  NULL);
	}

	return hRet;
}

COMMON_API HANDLE cmnOpenExistingFileA(LPCSTR _lpcszFile, DWORD _dwDesiredAccess, DWORD _dwShareMode)
{
	
	HANDLE hRet = INVALID_HANDLE_VALUE;

	if(_lpcszFile)
	{
		hRet = ::CreateFileA(_lpcszFile, _dwDesiredAccess, _dwShareMode, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL,
							  NULL);
	}

	return hRet;
}

// Get the last file write time in local time (including GMT and DST) in the form of 64bit integer
COMMON_API DWORD cmnGetLastFileWriteTime(HANDLE _hFile, PULARGE_INTEGER _lpdwdwTime)
{
	DWORD dwRet = ERROR_INVALID_PARAMETER;

	if((INVALID_HANDLE_VALUE != _hFile) && _lpdwdwTime)
	{
		FILETIME ftWrite;
		if(::GetFileTime(_hFile, NULL, NULL, &ftWrite))
		{
			FILETIME ftLWrite;	// local write time
			if(::FileTimeToLocalFileTime(&ftWrite, &ftLWrite))
			{
				// Now it is correct time including DST and GMT, if debugging write it
				#ifdef _DEBUG
				  SYSTEMTIME st;
				  			  
				  if(::FileTimeToSystemTime(&ftLWrite, &st))
				  {
					  //TRACEA("The last write file time is > %02d/%02d/%d  %02d:%02d\n", st.wDay, st.wMonth, st.wYear,
						//		st.wHour, st.wMinute);
				  }
				#endif
			
				// Now copy the data to the parameter specified location
				// Don't use reinterpret_cast because we need to really copy the data
				_lpdwdwTime->LowPart = ftLWrite.dwLowDateTime;
				_lpdwdwTime->HighPart = ftLWrite.dwHighDateTime;

				dwRet = ERROR_SUCCESS;
			}
		}
	}

	return dwRet;
}

COMMON_API BOOL cmnHasFileChanged(HANDLE _hFile, PULARGE_INTEGER _lpdwdwLastTime)
{
	BOOL bRet = FALSE;

	if(_hFile)
	{
		ULARGE_INTEGER dwdwCurrent;

		if(ERROR_SUCCESS == cmnGetLastFileWriteTime(_hFile, &dwdwCurrent))
		{
            // If the current time is bigger (i.e. the file is newer)
			bRet = ((_lpdwdwLastTime->QuadPart) < dwdwCurrent.QuadPart);
		}
	}

	return bRet;
}

COMMON_API BOOL cmnFileExistsW(LPCWSTR _lpcwszPath, LPCWSTR _lpcwszFile)
{
	BOOL bRet = FALSE;

	if(_lpcwszPath)
	{
		LPCWSTR lpcwszFullPath = NULL;
		WCHAR wszTempPath[MAX_PATH];
		DWORD dwSize = MAX_PATH;

		if(_lpcwszFile)
		{
			// If both strings exists, then create path from them ...
			cmnCreatePathW(wszTempPath, _lpcwszPath, _lpcwszFile, &dwSize);
			
			lpcwszFullPath = wszTempPath;
		}
		else
		{
			lpcwszFullPath = _lpcwszPath;
		}

		// Now check the existence by opening the file using CreateFile
		HANDLE hFile = INVALID_HANDLE_VALUE;

		hFile = cmnOpenExistingFileW(lpcwszFullPath);	// Open the file for reading

		if(INVALID_HANDLE_VALUE != hFile)	
		{
			cmnCloseFile(hFile);	// close the handle
			bRet = TRUE;	// The file exists, so the unit is complete
		}
		else
		{
			DWORD dwLastError = GetLastError();
			// If the last error isn't the system cannot find the specified file, then report it
			if(ERROR_FILE_NOT_FOUND != dwLastError)
			{
				//TRACEW(L"Error checking %s existence > %08X\n", lpcwszFullPath, dwLastError);
			}
			// The handle isn't valid for the file, so there is no need to close it ...
		}
	}

	return bRet;
}

COMMON_API BOOL cmnFileExistsA(LPCSTR _lpcszPath, LPCSTR _lpcszFile)
{
	BOOL bRet = FALSE;

	if(_lpcszPath)
	{
		CHAR szTempPath[MAX_PATH];	// Has to be created here, if placed in if, it would be out of the scope
		DWORD dwSize = MAX_PATH;

		LPCSTR lpcszFullPath = NULL;

		if(_lpcszFile)
		{
			// If both strings exists, then create path from them ...
			cmnCreatePathA(szTempPath, _lpcszPath, _lpcszFile, &dwSize);
			
			lpcszFullPath = szTempPath;
		}
		else
		{
			lpcszFullPath = _lpcszPath;
		}

		// Now check the existence by opening the file using CreateFile
		HANDLE hFile = INVALID_HANDLE_VALUE;

		hFile = cmnOpenExistingFileA(lpcszFullPath);	// Open the file for reading

		if(INVALID_HANDLE_VALUE != hFile)	
		{
			cmnCloseFile(hFile);	// close the handle
			bRet = TRUE;	// The file exists, so the unit is complete
		}
		else
		{
			DWORD dwLastError = GetLastError();
			// If the last error isn't the system cannot find the specified file, then report it
			if(ERROR_FILE_NOT_FOUND != dwLastError)
			{
				//TRACEA("Error checking %s existence > %08X\n", lpcszFullPath, dwLastError);
			}
			// The handle isn't valid for the file, so there is no need to close it ...
		}
	}

	return bRet;
}

COMMON_API HANDLE cmnCreateFileW(LPCWSTR _lpcwszFile, DWORD _dwDesiredAccess, DWORD _dwShareMode)
{
	HANDLE hRet = INVALID_HANDLE_VALUE;

	if(_lpcwszFile)
	{
		hRet = ::CreateFileW(_lpcwszFile, _dwDesiredAccess, _dwShareMode, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL,
							  NULL);
		
		if(INVALID_HANDLE_VALUE == hRet)
		{
			ERRW(L"The file couldn't be opened.", GetLastError());
		}
	}

	return hRet;
}

COMMON_API HANDLE cmnCreateFileA(LPCSTR _lpcszFile, DWORD _dwDesiredAccess, DWORD _dwShareMode)
{
	HANDLE hRet = INVALID_HANDLE_VALUE;

	if(_lpcszFile)
	{
		hRet = ::CreateFileA(_lpcszFile, _dwDesiredAccess, _dwShareMode, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL,
							  NULL);
		
		if(INVALID_HANDLE_VALUE == hRet)
		{
			ERRA("The file couldn't be opened.", GetLastError());
		}
	}

	return hRet;
}


