// CmnFile.h - functions for loading the file to the memory buffers
// Date :	28.08.2002
// Update : 25.12.2002
// Author :	Ondrej Burisin
// Updater:	Ondrej Burisin, Swap

#ifndef _CMNFILE_H_
	#define _CMNFILE_H_	
	
    // Loads the whole file to the specified file buffer in memory
	COMMON_API DWORD cmnLoadFileFromPathW(LPCWSTR _lpcwszFile, LPBYTE _lpBuffer, LPDWORD _lpdwSize);
	COMMON_API DWORD cmnLoadFileFromPathA(LPCSTR _lpcszFile, LPBYTE _lpBuffer, LPDWORD _lpdwSize);
	  
	COMMON_API DWORD cmnLoadFileFromHandle(HANDLE _hFile, LPBYTE _lpBuffer, LPDWORD _lpdwSize);
	  
	#ifdef UNICODE
		#define cmnLoadFileFromPath	cmnLoadFileFromPathW
	#else
		#define cmnLoadFileFromPath	cmnLoadFileFromPathA
	#endif

	// Opens the specified file and returns it's handle (the return value of CreateFile
	COMMON_API HANDLE cmnOpenExistingFileW(LPCWSTR _lpcwszFile, DWORD _dwDesiredAccess = GENERIC_READ, DWORD _dwShareMode = FILE_SHARE_READ|FILE_SHARE_WRITE);
	COMMON_API HANDLE cmnOpenExistingFileA(LPCSTR _lpcszFile, DWORD _dwDesiredAccess = GENERIC_READ, DWORD _dwShareMode = FILE_SHARE_READ|FILE_SHARE_WRITE);

	#ifdef UNICODE
		#define cmnOpenExistingFile	cmnOpenExistingFileW
	#else
		#define cmnOpenExistingFile	cmnOpenExistingFileA
	#endif

	inline COMMON_API DWORD cmnCloseFile(HANDLE _hFile)
	{
		return ((::CloseHandle(_hFile)) ? ERROR_SUCCESS : ERROR_INVALID_PARAMETER);
	}

	// Get the last file write time in local time (including GMT and DST) in the form of 64bit integer
	COMMON_API DWORD cmnGetLastFileWriteTime(HANDLE _hFile, PULARGE_INTEGER _lpdwdwTime);
	  
	// Returns TRUE if the last write time of the file has changed
	COMMON_API BOOL cmnHasFileChanged(HANDLE _hFile, PULARGE_INTEGER _lpdwdwLastTime);

	// Checks if the specified file exists, by opening and then closing it's handle (this way it works for UNC names
	// also). If the second parameter is NULL, then the first parameter is full path to the file, otherwise, the first
	// parameter is the path and the second is the file name
	COMMON_API BOOL cmnFileExistsW(LPCWSTR _lpcwszPath, LPCWSTR _lpcwszFile = NULL);
	COMMON_API BOOL cmnFileExistsA(LPCSTR _lpcszPath, LPCSTR _lpcszFile = NULL);

	#ifdef UNICODE
		#define cmnFileExists	cmnFileExistsW
	#else
		#define cmnFileExists	cmnFileExistsA
	#endif

	COMMON_API HANDLE cmnCreateFileW(LPCWSTR _lpcwszFile, DWORD _dwDesiredAccess = GENERIC_WRITE, DWORD _dwShareMode = FILE_SHARE_READ|FILE_SHARE_WRITE);
	COMMON_API HANDLE cmnCreateFileA(LPCSTR _lpcszFile, DWORD _dwDesiredAccess = GENERIC_WRITE, DWORD _dwShareMode = FILE_SHARE_READ|FILE_SHARE_WRITE);

	#ifdef UNICODE
		#define cmnCreateFile	cmnCreateFileW
	#else
		#define cmnCreateFile	cmnCreateFileA
	#endif

#endif