/*
 *
 * CmnInfo.cpp - implementation
 * Date:	24.12.2002
 * Update:	26.12.2002
 * Authors:	Ondrej Burisin & Jiri Formanek
 *
 */

#include "stdafx.h"

#include <stdlib.h>
#include <mmsystem.h>

#define COMMON_API __declspec(dllexport)
#include "CmnLog.h"
#include "CmnStr.h"
#include "CmnFile.h"
#include "CmnInfo.h"

LPSTR g_lpszSysDir = NULL;	// The path to the system directory
LPWSTR g_lpwszSysDir = NULL;
LPSTR g_lpszWinDir = NULL;	// Path to the windows directory
LPWSTR g_lpwszWinDir = NULL;

// Returns the string containing windows name, version, service pack if any ...
// If the function fails(FALSE) it sets the last error, that can be checked by GetLastError
// The function also returns if the windows are NT's or other
COMMON_API DWORD cmnGetWindowsInfoStringW(LPWSTR _lpwszInfo, DWORD _dwBufferSize, PBOOL pbIsWinNT)
{
	DWORD dwReturn = FALSE;

	// If the selected buffer isn't allocated
	if((!_lpwszInfo)||(_dwBufferSize <= 0))
	{
		SetLastError(ERROR_INVALID_PARAMETER);
		return dwReturn;
	}

	if(pbIsWinNT) { *pbIsWinNT = FALSE; }	// Init the BOOL, if it is valid
	
	DWORD dwVersion;
	dwVersion = GetVersion();

	OSVERSIONINFOEXW WinInfoW;
	ZeroMemory(&WinInfoW, sizeof(OSVERSIONINFOEXW));

	if(dwVersion < 0x80000000)              // Windows NT/2000/XP - we can use EX structure
	{
		WinInfoW.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEXW);
		//TRACE(" * NT based System detected - using OSVERSIONINFOEX struct\n");
	}
	else
	{
		WinInfoW.dwOSVersionInfoSize = sizeof(OSVERSIONINFOW);
		//TRACE(" * 98 based System detected - using OSVERSIONINFO struct\n");
	}

	if(FALSE == GetVersionExW((LPOSVERSIONINFOW)&WinInfoW))
	{	// If the GetVersionEx fails ...
		return dwReturn;	// We return an error
	}
 	// Else we continue with creating the Windows version string - we use only important info
	WCHAR VersionStr[255] = L"Windows ";
	
	switch(WinInfoW.dwMajorVersion) {
			case 5 : 
				// Should be better optimized ...
				if(0 == WinInfoW.dwMinorVersion) { wcscat(VersionStr, L"2000 "); }
				if(1 <= WinInfoW.dwMinorVersion) { wcscat(VersionStr, L"XP "); }
				break;
			case 4 :
				if(0 == WinInfoW.dwMinorVersion) { 
					if(VER_PLATFORM_WIN32_NT == WinInfoW.dwPlatformId) { wcscat(VersionStr, L"NT "); }
					else { wcscat(VersionStr, L"95 "); }
				}
				if(0 < WinInfoW.dwMinorVersion) { wcscat(VersionStr, L"98 "); }
				
				break;
			case 3 :
				if(51 == WinInfoW.dwMinorVersion) { wcscat(VersionStr, L"NT "); }
				break;
			default:
				break;
	}

	// We build the X.Y version info ...
	WCHAR Temp[135];
	wcscat(VersionStr, _itow(WinInfoW.dwMajorVersion, Temp, 10));
	wcscat(VersionStr, L".");	// Has to be a string for _tcscat
	wcscat(VersionStr, _itow(WinInfoW.dwMinorVersion, Temp, 10));

	wcscat(VersionStr, L" build ");

	switch(WinInfoW.dwPlatformId) {
			case VER_PLATFORM_WIN32_WINDOWS :
				// Windows 9x/ME
				wcscat(VersionStr, _itow(LOWORD(WinInfoW.dwBuildNumber), Temp, 10));
				break;
			case VER_PLATFORM_WIN32_NT :
				// Set the bool if the parameter is valid
				if(pbIsWinNT) { *pbIsWinNT = TRUE; }
				// Windows NT system
				wcscat(VersionStr, _itow(LOWORD(WinInfoW.dwBuildNumber), Temp, 10));
				// We add space only if service pack info is available ...
				if(WinInfoW.szCSDVersion[0]) { wcscat(VersionStr, L" "); }
				wcscat(VersionStr, WinInfoW.szCSDVersion);
				break;
			default :
				wcscat(VersionStr, L"3.1 Emulation");
	};

	// Now we check if the VersionStr length isn't bigger than the size of the supplied buffer
	if(wcslen(VersionStr) <= _dwBufferSize)
	{
		wcscpy(_lpwszInfo, VersionStr);
		dwReturn = TRUE;
	}
	else
	{
		// Returns the system error, should be rewritten for my own error codes ...
		SetLastError(ERROR_INSUFFICIENT_BUFFER);
	}

	return dwReturn;
}

COMMON_API DWORD cmnGetWindowsInfoStringA(LPSTR _lpszInfo, DWORD _dwBufferSize, PBOOL pbIsWinNT)
{
	DWORD dwReturn = FALSE;

	// If the selected buffer isn't allocated
	if((!_lpszInfo)||(_dwBufferSize <= 0))
	{
		SetLastError(ERROR_INVALID_PARAMETER);
		return dwReturn;
	}

	if(pbIsWinNT) { *pbIsWinNT = FALSE; }	// Init the BOOL, if it is valid
	
	DWORD dwVersion;
	dwVersion = GetVersion();

	OSVERSIONINFOEXA WinInfoA;
	ZeroMemory(&WinInfoA, sizeof(OSVERSIONINFOEXA));

	if(dwVersion < 0x80000000)              // Windows NT/2000/XP - we can use EX structure
	{
		WinInfoA.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEXA);
		//TRACE(" * NT based System detected - using OSVERSIONINFOEX struct\n");
	}
	else
	{
		WinInfoA.dwOSVersionInfoSize = sizeof(OSVERSIONINFOA);
		//TRACE(" * 98 based System detected - using OSVERSIONINFO struct\n");
	}

	if(FALSE == GetVersionExA((LPOSVERSIONINFOA)&WinInfoA))
	{	// If the GetVersionEx fails ...
		return dwReturn;	// We return an error
	}
 	// Else we continue with creating the Windows version string - we use only important info
	CHAR VersionStr[255] = "Windows ";
	
	switch(WinInfoA.dwMajorVersion) {
			case 5 : 
				// Should be better optimized ...
				if(0 == WinInfoA.dwMinorVersion) { strcat(VersionStr, "2000 "); }
				if(1 <= WinInfoA.dwMinorVersion) { strcat(VersionStr, "XP "); }
				break;
			case 4 :
				if(0 == WinInfoA.dwMinorVersion) { 
					if(VER_PLATFORM_WIN32_NT == WinInfoA.dwPlatformId) { strcat(VersionStr, "NT "); }
					else { strcat(VersionStr, "95 "); }
				}
				if(0 < WinInfoA.dwMinorVersion) { strcat(VersionStr, "98 "); }
				
				break;
			case 3 :
				if(51 == WinInfoA.dwMinorVersion) { strcat(VersionStr, "NT "); }
				break;
			default:
				break;
	}

	// We build the X.Y version info ...
	CHAR Temp[135];
	strcat(VersionStr, _itoa(WinInfoA.dwMajorVersion, Temp, 10));
	strcat(VersionStr, ".");	// Has to be a string for _tcscat
	strcat(VersionStr, _itoa(WinInfoA.dwMinorVersion, Temp, 10));

	strcat(VersionStr, " build ");

	switch(WinInfoA.dwPlatformId) {
			case VER_PLATFORM_WIN32_WINDOWS :
				// Windows 9x/ME
				strcat(VersionStr, _itoa(LOWORD(WinInfoA.dwBuildNumber), Temp, 10));
				break;
			case VER_PLATFORM_WIN32_NT :
				// Set the bool if the parameter is valid
				if(pbIsWinNT) { *pbIsWinNT = TRUE; }
				// Windows NT system
				strcat(VersionStr, _itoa(LOWORD(WinInfoA.dwBuildNumber), Temp, 10));
				// We add space only if service pack info is available ...
				if(WinInfoA.szCSDVersion[0]) { strcat(VersionStr, " "); }
				strcat(VersionStr, WinInfoA.szCSDVersion);
				break;
			default :
				strcat(VersionStr, "3.1 Emulation");
	};

	// Now we check if the VersionStr length isn't bigger than the size of the supplied buffer
	if(strlen(VersionStr) <= _dwBufferSize)
	{
		strcpy(_lpszInfo, VersionStr);
		dwReturn = TRUE;
	}
	else
	{
		// Returns the system error, should be rewritten for my own error codes ...
		SetLastError(ERROR_INSUFFICIENT_BUFFER);
	}

	return dwReturn;
}

#pragma COMMENT("Add support for detection of CPU information under Win9x/WinME systems")

COMMON_API DWORD cmnGetProcessorInfoW(LPPROCESSORINFOW _lpProcInfoW, DWORD _dwProcNum)
{
	BOOL dwReturn = FALSE;

	// The buffer isn't allocated, we exit ...
	if(!_lpProcInfoW)
	{
		SetLastError(ERROR_INVALID_PARAMETER);
		return dwReturn;
	}

	LONG Result;
	HKEY hKey;
	// Works only on Win NT class operating systems :(, that store info on registry
	WCHAR ProcNumString[255] = L"Hardware\\Description\\System\\CentralProcessor\\";
	WCHAR Temp[10];
	
	wcscat(ProcNumString, _itow(_dwProcNum, Temp, 10));
		
	Result = ::RegOpenKeyExW(HKEY_LOCAL_MACHINE, ProcNumString, 0, KEY_QUERY_VALUE, &hKey);

	if(ERROR_SUCCESS == Result)
	{	// If we succeded then get the info ...
		DWORD DataSize;

		DataSize = sizeof(_lpProcInfoW->m_Frequency);
		Result = ::RegQueryValueExW(hKey, L"~MHz", NULL, NULL, (LPBYTE)&_lpProcInfoW->m_Frequency, &DataSize);
		
		DataSize = sizeof(_lpProcInfoW->m_Identifier);
		Result = ::RegQueryValueExW(hKey, L"Identifier", NULL, NULL, (LPBYTE)_lpProcInfoW->m_Identifier,&DataSize);

		DataSize = sizeof(_lpProcInfoW->m_ProcessorName);
		Result = ::RegQueryValueExW(hKey, L"ProcessorNameString", NULL, NULL, (LPBYTE)_lpProcInfoW->m_ProcessorName,&DataSize);

		DataSize = sizeof(_lpProcInfoW->m_VendorIdentifier);
		Result = ::RegQueryValueExW(hKey, L"VendorIdentifier", NULL, NULL, (LPBYTE)_lpProcInfoW->m_VendorIdentifier, &DataSize);

		// Now close the key, we don't need it anymore
		if(ERROR_SUCCESS == RegCloseKey(hKey)) { dwReturn = TRUE; }
	}
	
	return dwReturn;
}

COMMON_API DWORD cmnGetProcessorInfoA(LPPROCESSORINFO _lpProcInfo, DWORD _dwProcNum)
{
	BOOL dwReturn = FALSE;

	// The buffer isn't allocated, we exit ...
	if(!_lpProcInfo)
	{
		SetLastError(ERROR_INVALID_PARAMETER);
		return dwReturn;
	}

	LONG Result;
	HKEY hKey;
	// Works only on Win NT class operating systems :(
	TCHAR ProcNumString[255] = _T("Hardware\\Description\\System\\CentralProcessor\\");
	TCHAR Temp[10];
	_tcscat(ProcNumString,_itot(_dwProcNum, Temp, 10));
		
	Result = ::RegOpenKeyEx(HKEY_LOCAL_MACHINE, ProcNumString, 0, KEY_QUERY_VALUE, &hKey);

	if(ERROR_SUCCESS == Result)
	{	// If we succeded then get the info ...
		DWORD DataSize;

		DataSize = sizeof(_lpProcInfo->m_Frequency);
		Result = ::RegQueryValueEx(hKey, _T("~MHz"), NULL, NULL, (LPBYTE)&_lpProcInfo->m_Frequency, &DataSize);
		
		DataSize = sizeof(_lpProcInfo->m_Identifier);
		Result = ::RegQueryValueEx(hKey,_T("Identifier"), NULL, NULL, (LPBYTE)_lpProcInfo->m_Identifier,&DataSize);

		DataSize = sizeof(_lpProcInfo->m_ProcessorName);
		Result = ::RegQueryValueEx(hKey,_T("ProcessorNameString"), NULL, NULL, (LPBYTE)_lpProcInfo->m_ProcessorName,&DataSize);

		DataSize = sizeof(_lpProcInfo->m_VendorIdentifier);
		Result = ::RegQueryValueEx (hKey, _T("VendorIdentifier"), NULL, NULL, (LPBYTE)_lpProcInfo->m_VendorIdentifier, &DataSize);

		// Now close the key, we don't need it anymore
		if(ERROR_SUCCESS == RegCloseKey(hKey)) { dwReturn = TRUE; }
	}
	
	return dwReturn;
}

COMMON_API BOOL cmnIsWin2000orXP()
{
	BOOL bRet = FALSE;	// Implicitly not, because 2000 and higher can do almost anything that 9x's can

	DWORD dwVersion;
	dwVersion = GetVersion();

	DWORD dwMajorVer =  (DWORD)(LOBYTE(LOWORD(dwVersion)));
	
	if(dwMajorVer >= 5)
	{
		// It is Win2000 or WinXP or even higher
		bRet = TRUE;
	}

	return bRet;
}

COMMON_API BOOL cmnIsWinNT()
{
	DWORD dwRet = FALSE;	// Implicitly not, because NT's offer more than 98's
	
	OSVERSIONINFO WinInfo;
	ZeroMemory(&WinInfo, sizeof(OSVERSIONINFO));
	
	WinInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

	if(FALSE == ::GetVersionEx(&WinInfo))
	{
		return dwRet;
	}
	
	if(VER_PLATFORM_WIN32_NT == WinInfo.dwPlatformId)
	{
		dwRet = TRUE;
	}

	return dwRet;
}

COMMON_API DWORD cmnGetCPUSpeed()
{
	int				timeStart	= 0;
	int				timeStop	= 0;
	unsigned long   StartTicks	= 0;
	unsigned long   EndTicks	= 0;
	unsigned long   TotalTicks	= 0;
	unsigned long   cpuSpeed	= 0;

	timeStart = timeGetTime();		// Get tick edge
	for( ; ; )
	{
		timeStop = timeGetTime();
		if ((timeStop-timeStart) > 1)		// rollover past 1
		{
			__asm {
				xor    eax, eax
				xor    ebx, ebx
				xor    ecx, ecx
				xor    edx, edx
				cpuid
				rdtsc
				mov    [StartTicks], eax	// Tick counter starts here
			}
			break;
		}
	}

	timeStart = timeStop;	

	for( ; ; )
	{
		timeStop = timeGetTime();
		if ((timeStop-timeStart) > 1000)	// one second
		{
			__asm {
				xor    eax, eax
				xor    ebx, ebx
				xor    ecx, ecx
				xor    edx, edx
				cpuid
				rdtsc
				mov    [EndTicks], eax
			}
			break;	
		}
	}

	TotalTicks = EndTicks-StartTicks;

	cpuSpeed = TotalTicks / 1000000;	// speed in MHz

	return (DWORD)cpuSpeed;
}

COMMON_API DWORD cmnGetDXVersionW(LPWSTR _lpwszDXVers, DWORD _dwBufferSize)
{
	DWORD dwDirectXVersion = 0;
	
	// Init values to unknown
    DWORD dwDirectXVersionMajor = 0;
    DWORD dwDirectXVersionMinor = 0;
	wchar_t wcDXVersLetter = ' ';	// The version letter
        
    // Getting the DirectX version info from DxDiag failed, so most likely we are on DX8.x or earlier
    if(SUCCEEDED(GetDirectXVerionViaFileVersionsW(&dwDirectXVersionMajor, &dwDirectXVersionMinor, &wcDXVersLetter)))
	{         
		// Set the output values to what we got and return       
		wcDXVersLetter = (wchar_t)towlower(wcDXVersLetter);
	    
		dwDirectXVersion = dwDirectXVersionMajor;
		dwDirectXVersion <<= 8;
		dwDirectXVersion += dwDirectXVersionMinor;
		dwDirectXVersion <<= 8;
	    
		if((wcDXVersLetter >= L'a') && (wcDXVersLetter <= L'z'))
				dwDirectXVersion += (wcDXVersLetter - L'a') + 1;

		if((_lpwszDXVers) && (_dwBufferSize >= 5))
		{
			if(wcDXVersLetter == ' ')
				_snwprintf(_lpwszDXVers, _dwBufferSize, L"%d.%d", dwDirectXVersionMajor, dwDirectXVersionMinor );
			else
				_snwprintf(_lpwszDXVers, _dwBufferSize, L"%d.%d%c", dwDirectXVersionMajor, dwDirectXVersionMinor, wcDXVersLetter);
	        
			_lpwszDXVers[_dwBufferSize-1] = '\0';
		}
	}
	       
    return dwDirectXVersion;
}

COMMON_API DWORD cmnGetDXVersionA(LPSTR _lpszDXVers, DWORD _dwBufferSize)
{
	DWORD dwDirectXVersion = 0;
	
	// Init values to unknown
    DWORD dwDirectXVersionMajor = 0;
    DWORD dwDirectXVersionMinor = 0;
	char cDXVersLetter = ' ';	// The version letter
        
    // Getting the DirectX version info from DxDiag failed, so most likely we are on DX8.x or earlier
    if(SUCCEEDED(GetDirectXVerionViaFileVersionsA(&dwDirectXVersionMajor, &dwDirectXVersionMinor, &cDXVersLetter)))
	{         
		// Set the output values to what we got and return       
		cDXVersLetter = (char)tolower(cDXVersLetter);
	    
		dwDirectXVersion = dwDirectXVersionMajor;
		dwDirectXVersion <<= 8;
		dwDirectXVersion += dwDirectXVersionMinor;
		dwDirectXVersion <<= 8;
	    
		if((cDXVersLetter >= 'a') && (cDXVersLetter <= 'z'))
				dwDirectXVersion += (cDXVersLetter - 'a') + 1;

		if((_lpszDXVers) && (_dwBufferSize >= 5))
		{
			if(cDXVersLetter == ' ')
				_snprintf(_lpszDXVers, _dwBufferSize, "%d.%d", dwDirectXVersionMajor, dwDirectXVersionMinor );
			else
				_snprintf(_lpszDXVers, _dwBufferSize, "%d.%d%c", dwDirectXVersionMajor, dwDirectXVersionMinor, cDXVersLetter);
	        
			_lpszDXVers[_dwBufferSize-1] = '\0';
		}
	}
	       
    return dwDirectXVersion;
}

COMMON_API HRESULT cmnGetFileVersionW(LPCWSTR _lpcwszPath, LPCWSTR _lpcwszFile, ULARGE_INTEGER* _lpllFileVersion)
{
	if((_lpcwszPath == NULL) || (_lpcwszFile == NULL) || (_lpllFileVersion == NULL))
        return E_INVALIDARG;

	if(cmnFileExistsW(_lpcwszPath, _lpcwszFile))
	{
		LPWSTR lpwszFullPath = NULL;
		DWORD dwSize = 0;
		
		cmnCreatePathW(NULL, _lpcwszPath, _lpcwszFile, &dwSize);

		if(dwSize)
		{
			lpwszFullPath = new wchar_t[dwSize];

			if(lpwszFullPath)
			{
				cmnCreatePathW(lpwszFullPath, _lpcwszPath, _lpcwszFile, &dwSize);

				DWORD dwHandle;
				UINT  cb;
				cb = GetFileVersionInfoSizeW(lpwszFullPath, &dwHandle);
				if(cb > 0)
				{
					BYTE* lpFileVersionBuffer = new BYTE[cb];
					if(lpFileVersionBuffer == NULL)
						return E_OUTOFMEMORY;

					if(GetFileVersionInfoW(lpwszFullPath, 0, cb, lpFileVersionBuffer))
					{
						VS_FIXEDFILEINFO* lpVersion = NULL;
						if(VerQueryValueW(lpFileVersionBuffer, L"\\", (VOID**)&lpVersion, &cb) && (lpVersion != NULL)) 
						{
							_lpllFileVersion->HighPart = lpVersion->dwFileVersionMS;
							_lpllFileVersion->LowPart  = lpVersion->dwFileVersionLS;
							delete[] lpFileVersionBuffer;
							return S_OK;
						}
					}

					delete[] lpFileVersionBuffer;
				}
				delete[] lpwszFullPath;
			}
		}
	}

    return E_FAIL;
}

COMMON_API HRESULT cmnGetFileVersionA(LPCSTR _lpcszPath, LPCSTR _lpcszFile, ULARGE_INTEGER* _lpllFileVersion)
{
	if((_lpcszPath == NULL) || (_lpcszFile == NULL) || (_lpllFileVersion == NULL))
        return E_INVALIDARG;

	if(cmnFileExistsA(_lpcszPath, _lpcszFile))
	{
		LPSTR lpszFullPath = NULL;
		DWORD dwSize = 0;
		
		cmnCreatePathA(NULL, _lpcszPath, _lpcszFile, &dwSize);

		if(dwSize)
		{
			lpszFullPath = new char[dwSize];

			if(lpszFullPath)
			{
				cmnCreatePathA(lpszFullPath, _lpcszPath, _lpcszFile, &dwSize);

				DWORD dwHandle;
				UINT  cb;
				cb = GetFileVersionInfoSizeA(lpszFullPath, &dwHandle);
				if(cb > 0)
				{
					BYTE* lpFileVersionBuffer = new BYTE[cb];
					if(lpFileVersionBuffer == NULL)
						return E_OUTOFMEMORY;

					if(GetFileVersionInfoA(lpszFullPath, 0, cb, lpFileVersionBuffer))
					{
						VS_FIXEDFILEINFO* lpVersion = NULL;
						if(VerQueryValueA(lpFileVersionBuffer, "\\", (VOID**)&lpVersion, &cb) && (lpVersion != NULL)) 
						{
							_lpllFileVersion->HighPart = lpVersion->dwFileVersionMS;
							_lpllFileVersion->LowPart  = lpVersion->dwFileVersionLS;
							delete[] lpFileVersionBuffer;
							return S_OK;
						}
					}

					delete[] lpFileVersionBuffer;
				}
				delete[] lpszFullPath;
			}
		}
	}

    return E_FAIL;
}

// Try to get the DirectX version by looking at DirectX file versions (UNICODE version)
HRESULT GetDirectXVerionViaFileVersionsW(LPDWORD _lpdwDXVersMajor, LPDWORD _lpdwDXVersMinor, LPWSTR _lpwcDXVersChar)
{
    ULARGE_INTEGER llFileVersion;
    BOOL bFound = FALSE;
	LPCWSTR lpcwszSysDir = cmnGetSysDirW();
	
    if(lpcwszSysDir)
	{
        // Switch off the ddraw version
		if(SUCCEEDED(cmnGetFileVersionW(lpcwszSysDir, L"ddraw.dll", &llFileVersion)))
		{
			if(CompareLargeInts(llFileVersion, MakeInt64(4, 2, 0, 95)) >= 0) // Win9x version
			{
				// flle is >= DX1.0 version, so we must be at least DX1.0
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 1; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
				if(_lpwcDXVersChar) { *_lpwcDXVersChar = L' '; }
				bFound = TRUE;
			}

			if(CompareLargeInts(llFileVersion, MakeInt64(4, 3, 0, 1096)) >= 0) // Win9x version
			{
				// flle is is >= DX2.0 version, so we must DX2.0 or DX2.0a (no redist change)
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 2; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
				if(_lpwcDXVersChar) { *_lpwcDXVersChar = L' '; }
				bFound = TRUE;
			}

			if(CompareLargeInts(llFileVersion, MakeInt64(4, 4, 0, 68)) >= 0) // Win9x version
			{
				// flle is is >= DX3.0 version, so we must be at least DX3.0
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 3; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
				if(_lpwcDXVersChar) { *_lpwcDXVersChar = L' '; }
				bFound = TRUE;
			}
		}
		
        // Switch off the d3drg8x.dll version
		if(SUCCEEDED(cmnGetFileVersionW(lpcwszSysDir, L"d3drg8x.dll", &llFileVersion)))
		{
			if(CompareLargeInts(llFileVersion, MakeInt64(4, 4, 0, 70)) >= 0) // Win9x version
			{
				// d3drg8x.dll is the DX3.0a version, so we must be DX3.0a or DX3.0b  (no redist change)
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 3; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
				if(_lpwcDXVersChar) { *_lpwcDXVersChar = L'a'; }
				bFound = TRUE;
			}
		}

        // Switch off the ddraw version
		if(SUCCEEDED(cmnGetFileVersionW(lpcwszSysDir, L"ddraw.dll", &llFileVersion)))
		{
			if(CompareLargeInts(llFileVersion, MakeInt64(4, 5, 0, 155)) >= 0) // Win9x version
			{
				// ddraw.dll is the DX5.0 version, so we must be DX5.0 or DX5.2 (no redist change)
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 5; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
				if(_lpwcDXVersChar) { *_lpwcDXVersChar = L' '; }
				bFound = TRUE;
			}

			if(CompareLargeInts(llFileVersion, MakeInt64(4, 6, 0, 318)) >= 0) // Win9x version
			{
				// ddraw.dll is the DX6.0 version, so we must be at least DX6.0
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 6; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
				if(_lpwcDXVersChar) { *_lpwcDXVersChar = L' '; }
				bFound = TRUE;
			}

			if(CompareLargeInts(llFileVersion, MakeInt64(4, 6, 0, 436)) >= 0) // Win9x version
			{
				// ddraw.dll is the DX6.1 version, so we must be at least DX6.1
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 6; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 1; }
				if(_lpwcDXVersChar) { *_lpwcDXVersChar = L' '; }
				bFound = TRUE;
			}
		}

        // Switch off the dplayx.dll version
        if(SUCCEEDED(cmnGetFileVersionW(lpcwszSysDir, L"dplayx.dll", &llFileVersion)))
        {
            if(CompareLargeInts(llFileVersion, MakeInt64(4, 6, 3, 518)) >= 0) // Win9x version
            {
                // ddraw.dll is the DX6.1 version, so we must be at least DX6.1a
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 6; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 1; }
				if(_lpwcDXVersChar) { *_lpwcDXVersChar = L'a'; }
                bFound = TRUE;
            }
        }

        // Switch off the ddraw version
		if(SUCCEEDED(cmnGetFileVersionW(lpcwszSysDir, L"ddraw.dll", &llFileVersion)))
        {
            if(CompareLargeInts(llFileVersion, MakeInt64(4, 7, 0, 700)) >= 0) // Win9x version
            {
                // TODO: find win2k version
                
                // ddraw.dll is the DX7.0 version, so we must be at least DX7.0
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 7; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
				if(_lpwcDXVersChar) { *_lpwcDXVersChar = L' '; }
                bFound = TRUE;
            }
        }

        // Switch off the dinput version
   		if(SUCCEEDED(cmnGetFileVersionW(lpcwszSysDir, L"dinput.dll", &llFileVersion)))
        {
            if(CompareLargeInts(llFileVersion, MakeInt64(4, 7, 0, 716)) >= 0) // Win9x version
            {
				// ddraw.dll is the DX7.0 version, so we must be at least DX7.0a
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 7; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
				if(_lpwcDXVersChar) { *_lpwcDXVersChar = L'a'; }
                bFound = TRUE;
            }
        }

        // Switch off the ddraw version      
		if(SUCCEEDED(cmnGetFileVersionW(lpcwszSysDir, L"ddraw.dll", &llFileVersion)))
        {
            if(((HIWORD(llFileVersion.HighPart) == 4) && CompareLargeInts(llFileVersion, MakeInt64(4, 8, 0, 400)) >= 0) || // Win9x version
               ((HIWORD(llFileVersion.HighPart) == 5) && CompareLargeInts(llFileVersion, MakeInt64(5, 1, 2258, 400)) >= 0)) // Win2k/WinXP version
            {
                // ddraw.dll is the DX8.0 version, so we must be at least DX8.0 or DX8.0a (no redist change)
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 8; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
				if(_lpwcDXVersChar) { *_lpwcDXVersChar = L' '; }
                bFound = TRUE;
            }
        }

        if(SUCCEEDED(cmnGetFileVersionW(lpcwszSysDir, L"d3d8.dll", &llFileVersion)))
        {
            if(((HIWORD(llFileVersion.HighPart) == 4) && CompareLargeInts(llFileVersion, MakeInt64(4, 8, 1, 881)) >= 0) || // Win9x version
               ((HIWORD(llFileVersion.HighPart) == 5) && CompareLargeInts(llFileVersion, MakeInt64(5, 1, 2600, 881)) >= 0)) // Win2k/WinXP version
            {
                // d3d8.dll is the DX8.1 version, so we must be at least DX8.1
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 8; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 1; }
				if(_lpwcDXVersChar) { *_lpwcDXVersChar = L' '; }
                bFound = TRUE;
            }

            if(((HIWORD(llFileVersion.HighPart) == 4) && CompareLargeInts(llFileVersion, MakeInt64(4, 8, 1, 901)) >= 0) || // Win9x version
               ((HIWORD(llFileVersion.HighPart) == 5) && CompareLargeInts(llFileVersion, MakeInt64(5, 1, 2600, 901)) >= 0)) // Win2k/WinXP version
            {                    
                // d3d8.dll is the DX8.1a version, so we must be at least DX8.1a
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 8; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 1; }
                if(_lpwcDXVersChar) *_lpwcDXVersChar = L'a';
                bFound = TRUE;
            }
        }

        if(SUCCEEDED(cmnGetFileVersionW(lpcwszSysDir, L"mpg2splt.ax", &llFileVersion)))
        {
            if(CompareLargeInts(llFileVersion, MakeInt64(6, 3, 1, 885)) >= 0) // Win9x/Win2k/WinXP version
            {
                // quartz.dll is the DX8.1b version, so we must be at least DX8.1b
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 8; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 1; }
				if(_lpwcDXVersChar) { *_lpwcDXVersChar = L'b'; }
                bFound = TRUE;
            }
        }
         
		if(SUCCEEDED(cmnGetFileVersionW(lpcwszSysDir, L"dpnet.dll", &llFileVersion)))
        {
            if(((HIWORD(llFileVersion.HighPart) == 4) && CompareLargeInts(llFileVersion, MakeInt64(4, 9, 0, 134)) >= 0) || // Win9x version
               ((HIWORD(llFileVersion.HighPart) == 5) && CompareLargeInts(llFileVersion, MakeInt64(5, 2, 3677, 134)) >= 0)) // Win2k/WinXP version
            {                    
                // dpnet.dll is the DX8.2 version, so we must be at least DX8.2
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 8; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 2; }
				if(_lpwcDXVersChar) { *_lpwcDXVersChar = L' '; }
                bFound = TRUE;
            }
        }
             
		if(SUCCEEDED(cmnGetFileVersionW(lpcwszSysDir, L"d3d9.dll", &llFileVersion)))
        {
            // File exists, but be at least DX9
			if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 9; }
			if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
			if(_lpwcDXVersChar) { *_lpwcDXVersChar = L' '; }
            bFound = TRUE;
        }
    }

    if(!bFound)
    {
        // No DirectX installed
		if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 0; }
		if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
		if(_lpwcDXVersChar) { *_lpwcDXVersChar = ' '; }
    }

    return S_OK;
}

// Try to get the DirectX version by looking at DirectX file versions (ANSI version)
HRESULT GetDirectXVerionViaFileVersionsA(LPDWORD _lpdwDXVersMajor, LPDWORD _lpdwDXVersMinor, LPSTR _lpcDXVersChar)
{
    ULARGE_INTEGER llFileVersion;
    BOOL bFound = FALSE;
	LPCSTR lpcszSysDir = cmnGetSysDirA();
	
    if(lpcszSysDir)
	{
        // Switch off the ddraw version
		if(SUCCEEDED(cmnGetFileVersionA(lpcszSysDir, "ddraw.dll", &llFileVersion)))
		{
			if(CompareLargeInts(llFileVersion, MakeInt64(4, 2, 0, 95)) >= 0) // Win9x version
			{
				// flle is >= DX1.0 version, so we must be at least DX1.0
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 1; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
				if(_lpcDXVersChar) { *_lpcDXVersChar = ' '; }
				bFound = TRUE;
			}

			if(CompareLargeInts(llFileVersion, MakeInt64(4, 3, 0, 1096)) >= 0) // Win9x version
			{
				// flle is is >= DX2.0 version, so we must DX2.0 or DX2.0a (no redist change)
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 2; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
				if(_lpcDXVersChar) { *_lpcDXVersChar = ' '; }
				bFound = TRUE;
			}

			if(CompareLargeInts(llFileVersion, MakeInt64(4, 4, 0, 68)) >= 0) // Win9x version
			{
				// flle is is >= DX3.0 version, so we must be at least DX3.0
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 3; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
				if(_lpcDXVersChar) { *_lpcDXVersChar = ' '; }
				bFound = TRUE;
			}
		}
		
        // Switch off the d3drg8x.dll version
		if(SUCCEEDED(cmnGetFileVersionA(lpcszSysDir, "d3drg8x.dll", &llFileVersion)))
		{
			if(CompareLargeInts(llFileVersion, MakeInt64(4, 4, 0, 70)) >= 0) // Win9x version
			{
				// d3drg8x.dll is the DX3.0a version, so we must be DX3.0a or DX3.0b  (no redist change)
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 3; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
				if(_lpcDXVersChar) { *_lpcDXVersChar = 'a'; }
				bFound = TRUE;
			}
		}

        // Switch off the ddraw version
		if(SUCCEEDED(cmnGetFileVersionA(lpcszSysDir, "ddraw.dll", &llFileVersion)))
		{
			if(CompareLargeInts(llFileVersion, MakeInt64(4, 5, 0, 155)) >= 0) // Win9x version
			{
				// ddraw.dll is the DX5.0 version, so we must be DX5.0 or DX5.2 (no redist change)
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 5; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
				if(_lpcDXVersChar) { *_lpcDXVersChar = ' '; }
				bFound = TRUE;
			}

			if(CompareLargeInts(llFileVersion, MakeInt64(4, 6, 0, 318)) >= 0) // Win9x version
			{
				// ddraw.dll is the DX6.0 version, so we must be at least DX6.0
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 6; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
				if(_lpcDXVersChar) { *_lpcDXVersChar = ' '; }
				bFound = TRUE;
			}

			if(CompareLargeInts(llFileVersion, MakeInt64(4, 6, 0, 436)) >= 0) // Win9x version
			{
				// ddraw.dll is the DX6.1 version, so we must be at least DX6.1
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 6; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 1; }
				if(_lpcDXVersChar) { *_lpcDXVersChar = ' '; }
				bFound = TRUE;
			}
		}

        // Switch off the dplayx.dll version
        if(SUCCEEDED(cmnGetFileVersionA(lpcszSysDir, "dplayx.dll", &llFileVersion)))
        {
            if(CompareLargeInts(llFileVersion, MakeInt64(4, 6, 3, 518)) >= 0) // Win9x version
            {
                // ddraw.dll is the DX6.1 version, so we must be at least DX6.1a
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 6; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 1; }
				if(_lpcDXVersChar) { *_lpcDXVersChar = 'a'; }
                bFound = TRUE;
            }
        }

        // Switch off the ddraw version
		if(SUCCEEDED(cmnGetFileVersionA(lpcszSysDir, "ddraw.dll", &llFileVersion)))
        {
            if(CompareLargeInts(llFileVersion, MakeInt64(4, 7, 0, 700)) >= 0) // Win9x version
            {
                // TODO: find win2k version
                
                // ddraw.dll is the DX7.0 version, so we must be at least DX7.0
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 7; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
				if(_lpcDXVersChar) { *_lpcDXVersChar = ' '; }
                bFound = TRUE;
            }
        }

        // Switch off the dinput version
   		if(SUCCEEDED(cmnGetFileVersionA(lpcszSysDir, "dinput.dll", &llFileVersion)))
        {
            if(CompareLargeInts(llFileVersion, MakeInt64(4, 7, 0, 716)) >= 0) // Win9x version
            {
				// ddraw.dll is the DX7.0 version, so we must be at least DX7.0a
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 7; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
				if(_lpcDXVersChar) { *_lpcDXVersChar = 'a'; }
                bFound = TRUE;
            }
        }

        // Switch off the ddraw version      
		if(SUCCEEDED(cmnGetFileVersionA(lpcszSysDir, "ddraw.dll", &llFileVersion)))
        {
            if(((HIWORD(llFileVersion.HighPart) == 4) && CompareLargeInts(llFileVersion, MakeInt64(4, 8, 0, 400)) >= 0) || // Win9x version
               ((HIWORD(llFileVersion.HighPart) == 5) && CompareLargeInts(llFileVersion, MakeInt64(5, 1, 2258, 400)) >= 0)) // Win2k/WinXP version
            {
                // ddraw.dll is the DX8.0 version, so we must be at least DX8.0 or DX8.0a (no redist change)
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 8; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
				if(_lpcDXVersChar) { *_lpcDXVersChar = ' '; }
                bFound = TRUE;
            }
        }

        if(SUCCEEDED(cmnGetFileVersionA(lpcszSysDir, "d3d8.dll", &llFileVersion)))
        {
            if(((HIWORD(llFileVersion.HighPart) == 4) && CompareLargeInts(llFileVersion, MakeInt64(4, 8, 1, 881)) >= 0) || // Win9x version
               ((HIWORD(llFileVersion.HighPart) == 5) && CompareLargeInts(llFileVersion, MakeInt64(5, 1, 2600, 881)) >= 0)) // Win2k/WinXP version
            {
                // d3d8.dll is the DX8.1 version, so we must be at least DX8.1
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 8; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 1; }
				if(_lpcDXVersChar) { *_lpcDXVersChar = ' '; }
                bFound = TRUE;
            }

            if(((HIWORD(llFileVersion.HighPart) == 4) && CompareLargeInts(llFileVersion, MakeInt64(4, 8, 1, 901)) >= 0) || // Win9x version
               ((HIWORD(llFileVersion.HighPart) == 5) && CompareLargeInts(llFileVersion, MakeInt64(5, 1, 2600, 901)) >= 0)) // Win2k/WinXP version
            {                    
                // d3d8.dll is the DX8.1a version, so we must be at least DX8.1a
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 8; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 1; }
                if(_lpcDXVersChar) *_lpcDXVersChar = 'a';
                bFound = TRUE;
            }
        }

        if(SUCCEEDED(cmnGetFileVersionA(lpcszSysDir, "mpg2splt.ax", &llFileVersion)))
        {
            if(CompareLargeInts(llFileVersion, MakeInt64(6, 3, 1, 885)) >= 0) // Win9x/Win2k/WinXP version
            {
                // quartz.dll is the DX8.1b version, so we must be at least DX8.1b
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 8; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 1; }
				if(_lpcDXVersChar) { *_lpcDXVersChar = 'b'; }
                bFound = TRUE;
            }
        }
         
		if(SUCCEEDED(cmnGetFileVersionA(lpcszSysDir, "dpnet.dll", &llFileVersion)))
        {
            if(((HIWORD(llFileVersion.HighPart) == 4) && CompareLargeInts(llFileVersion, MakeInt64(4, 9, 0, 134)) >= 0) || // Win9x version
               ((HIWORD(llFileVersion.HighPart) == 5) && CompareLargeInts(llFileVersion, MakeInt64(5, 2, 3677, 134)) >= 0)) // Win2k/WinXP version
            {                    
                // dpnet.dll is the DX8.2 version, so we must be at least DX8.2
				if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 8; }
				if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 2; }
				if(_lpcDXVersChar) { *_lpcDXVersChar = ' '; }
                bFound = TRUE;
            }
        }
             
		if(SUCCEEDED(cmnGetFileVersionA(lpcszSysDir, "d3d9.dll", &llFileVersion)))
        {
            // File exists, but be at least DX9
			if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 9; }
			if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
			if(_lpcDXVersChar) { *_lpcDXVersChar = ' '; }
            bFound = TRUE;
        }
    }

    if(!bFound)
    {
        // No DirectX installed
		if(_lpdwDXVersMajor) { *_lpdwDXVersMajor = 0; }
		if(_lpdwDXVersMinor) { *_lpdwDXVersMinor = 0; }
		if(_lpcDXVersChar) { *_lpcDXVersChar = ' '; }
    }

    return S_OK;
}

ULARGE_INTEGER MakeInt64(WORD _a, WORD _b, WORD _c, WORD _d)
{
    ULARGE_INTEGER ull;
    ull.HighPart = MAKELONG(_b,_a);
    ull.LowPart = MAKELONG(_d,_c);
    return ull;
}

int CompareLargeInts(ULARGE_INTEGER _ullParam1, ULARGE_INTEGER _ullParam2)
{
    if(_ullParam1.HighPart > _ullParam2.HighPart)
        return 1;
    if(_ullParam1.HighPart < _ullParam2.HighPart)
        return -1;

    if(_ullParam1.LowPart > _ullParam2.LowPart)
        return 1;
    if(_ullParam1.LowPart < _ullParam2.LowPart)
        return -1;

    return 0;
}