/*
 *
 * CmnInfo.h - informations about processor, operating system ...
 * Date:	24.12.2002
 * Update:	26.12.2002
 * Author:	Ondrej Burisin & Jiri Formanek
 *
 */

#ifndef _CMNINFO_H_
  #define _CMNINFO_H_

  extern LPSTR g_lpszSysDir;	// The path to the system directory
  extern LPWSTR g_lpwszSysDir;
  extern LPSTR g_lpszWinDir;	// Path to the windows directory
  extern LPWSTR g_lpwszWinDir;

  COMMON_API typedef struct tagPROCESSORINFOW {
	  DWORD m_Frequency;
	  WCHAR m_Identifier[64];
	  WCHAR m_ProcessorName[64];
	  WCHAR m_VendorIdentifier[64];
  } PROCESSORINFOW, *LPPROCESSORINFOW;

  COMMON_API typedef struct tagPROCESSORINFOA {
	  DWORD m_Frequency;
	  CHAR m_Identifier[64];
	  CHAR m_ProcessorName[64];
	  CHAR m_VendorIdentifier[64];
  } PROCESSORINFOA, *LPPROCESSORINFOA;

  #ifdef UNICODE
    typedef PROCESSORINFOW PROCESSORINFO;
    typedef LPPROCESSORINFOW LPPROCESSORINFO;
  #else
    typedef PROCESSORINFOA PROCESSORINFO;
    typedef LPPROCESSORINFOA LPPROCESSORINFO;
  #endif

  // Returns the string containing windows name, version, service pack if any ...
  // If the function fails(FALSE) it sets the last error, that can be checked by GetLastError
  COMMON_API DWORD cmnGetWindowsInfoStringW(LPWSTR _lpwszInfo, DWORD _dwBufferSize, PBOOL _pbIsWinNT = NULL);
  COMMON_API DWORD cmnGetWindowsInfoStringA(LPSTR _lpszInfo, DWORD _dwBufferSize, PBOOL _pbIsWinNT = NULL);

  #ifdef UNICODE
    #define cmnGetWindowsInfoString	cmnGetWindowsInfoStringW
  #else
    #define cmnGetWindowsInfoString	cmnGetWindowsInfoStringA
  #endif
  
  // Fills the SProcessorInfo structure, using reads from registry
  // dwProcNum specifies the number of the processor to get the info about
  // If the function fails(FALSE) it sets the last error, that can be checked by GetLastError
  COMMON_API DWORD cmnGetProcessorInfoW(LPPROCESSORINFOW _lpProcInfoW, DWORD _dwProcNum);
  COMMON_API DWORD cmnGetProcessorInfoA(LPPROCESSORINFOA _lpProcInfo, DWORD _dwProcNum);

  #ifdef UNICODE
    #define cmnGetProcessorInfo	cmnGetProcessorInfoW
  #else
    #define cmnGetProcessorInfo	cmnGetProcessorInfoA
  #endif

  // Returns TRUE if the installed operating system is Windows 2000 or Windows XP or higher
  COMMON_API BOOL cmnIsWin2000orXP();

  // Returns TRUE if the operating system is based on NT kernel.
  COMMON_API BOOL cmnIsWinNT();
  
  // Get the speed of the CPU by RDTSC and CPUID instructions
  COMMON_API DWORD cmnGetCPUSpeed();	// timeGetTime

  inline COMMON_API LPCWSTR cmnGetSysDirW() { return g_lpwszSysDir; }
  inline COMMON_API LPCSTR cmnGetSysDirA() { return g_lpszSysDir; }
  inline COMMON_API LPCWSTR cmnGetWinDirW() { return g_lpwszWinDir; }
  inline COMMON_API LPCSTR cmnGetWinDirA() { return g_lpszWinDir; }

  #ifdef UNICODE
    #define cmnGetSysDir	cmnGetSysDirW
    #define cmnGetWinDir	cmnGetWinDirW
  #else
    #define cmnGetSysDir	cmnGetSysDirA
    #define cmnGetWinDir	cmnGetWinDirA
  #endif

  // Helper functions for cmnGetDXVersion()
  
  // Returns a ULARGE_INTEGER where a<<48|b<<32|c<<16|d<<0
  ULARGE_INTEGER MakeInt64(WORD _a, WORD _b, WORD _c, WORD _d);
  
  // Standard compare function
  // Returns 1 if ullParam1 > ullParam2
  // Returns 0 if ullParam1 = ullParam2
  // Returns -1 if ullParam1 < ullParam2
  int CompareLargeInts(ULARGE_INTEGER _ullParam1, ULARGE_INTEGER _ullParam2);

  // Checks the files for versions and determines the version of DX installed
  HRESULT GetDirectXVerionViaFileVersionsW(LPDWORD _lpdwDXVersMajor, LPDWORD _lpdwDXVersMinor, LPWSTR _lpcDXVersChar);
  HRESULT GetDirectXVerionViaFileVersionsA(LPDWORD _lpdwDXVersMajor, LPDWORD _lpdwDXVersMinor, LPSTR _lpcDXVersChar);

  // Returns the version of the file
  COMMON_API HRESULT cmnGetFileVersionW(LPCWSTR _lpcwszPath, LPCWSTR _lpcwszFile,ULARGE_INTEGER* _pllFileVersion);
  COMMON_API HRESULT cmnGetFileVersionA(LPCSTR _lpcszPath, LPCSTR _lpcszFile, ULARGE_INTEGER* _lpllFileVersion);

  #ifdef UNICODE
    #define cmnGetFileVersion	cmnGetFileVersionW
  #else
    #define cmnGetFileVersion	cmnGetFileVersionA
  #endif

  // Returns the version of DirectX installed on the computer, if no DirectX is installed, then returns 0
  // the buffer should be at least able to store X.Xc, that is four chars. For dx10 XX.Xc ;)
  COMMON_API DWORD cmnGetDXVersionW(LPWSTR _lpszDXVers, DWORD _dwBufferSize);
  COMMON_API DWORD cmnGetDXVersionA(LPSTR _lpszDXVers, DWORD _dwBufferSize);

  #ifdef UNICODE
    #define cmnGetDXVersion	cmnGetDXVersionW
  #else
    #define cmnGetDXVersion	cmnGetDXVersionA
  #endif

#endif
  