/*
 *
 *	File: CmnLog.h 
 *  Desc: Functions and macros for diagnostic (TRACE, ...)
 *  Authors: MrAndrew, Swap
 *  Date:   23.12.2002
 *  Update: 19.9.2003
 *
 */

#ifndef _CMNLOG_H_
	#define _CMNLOG_H_	
	//	
	// Standard tracing with logging
	COMMON_API	void	cmnTraceW(LPCWSTR lpcwszFormat, ...);					
	COMMON_API	void	cmnTraceA(LPCSTR lpcszFormat, ...);						
	// Tracing with logging and decoding error code
	COMMON_API	void	cmnTraceErrW(LPCWSTR lpcwszMsg, HRESULT hr);			
	COMMON_API	void	cmnTraceErrA(LPCSTR lpcszMsg, HRESULT hr);			
	// Display message box (standard Windows message box)
	COMMON_API	void	cmnMessageBoxW(LPCWSTR lpcwszCaption, UINT uType);	
	COMMON_API	void	cmnMessageBoxA(LPCSTR lpcszCaption, UINT uType);		
	// Calls theLog.SavePosition()
	COMMON_API	void	cmnSavePosW(LPCWSTR	lpcwszFile, DWORD	dwLine);	
	COMMON_API	void	cmnSavePosA(LPCSTR	lpcszFile, DWORD	dwLine);			
	//
	// Macros
	// If TRACE macro is already defined, undef it	
	#ifdef TRACE
		#undef TRACE
	#endif // TRACE
	// If THROW macro is already defined, undef it	
	#ifdef THROW
		#undef THROW
	#endif // THROW	
	//
	// HUMUS
	#define WIDEN2(x) L ## x
	#define WIDEN(x) WIDEN2(x)
	#define __WFILE__ WIDEN(__FILE__)
	//
	// If the user wants to trace specific ANSI or UNICODE message
	// These macros are always defined
	#define TRACEA		cmnSavePosA(__FILE__, __LINE__); cmnTraceA			// ANSI verze
	#define TRACEW		cmnSavePosW(__WFILE__, __LINE__); cmnTraceW			// UNICODE verze
	#define ERRA		cmnSavePosA(__FILE__, __LINE__); cmnTraceErrA		// ANSI verze
	#define ERRW		cmnSavePosW(__WFILE__, __LINE__); cmnTraceErrW		// UNICODE verze
	#define THROWERRA	cmnSavePosA(__FILE__, __LINE__); cmnThrowErrA		// ANSI verze
	#define THROWERRW	cmnSavePosW(__WFILE__, __LINE__); cmnThrowErrW		// UNICODE verze

	// Define DBGTRACE, that writes only to the debugger window and only with DEBUG build
	#ifdef _DEBUG
      COMMON_API void dbgDebugTraceW(LPCWSTR lpszFormat, ...);	// UNICODE version
      COMMON_API void dbgDebugTraceA(LPCSTR lpszFormat, ...);	// ANSI version

	  // If the user wants to trace specific ANSI or UNICODE message
	  // These macros are always defined
	  #define DBGTRACEW	dbgDebugTraceW
	  #define DBGTRACEA	dbgDebugTraceA

      #ifdef UNICODE
        #define dbgDebugTrace	dbgDebugTraceW
      #else
        #define dbgDebugTrace	dbgDebugTraceA
      #endif

      #ifndef DBGTRACE
        #define DBGTRACE	dbgDebugTrace
      #endif
    #else	// _DEBUG isn't defined
      #ifndef DBGTRACE
	    #define DBGTRACE	__noop
      #endif
      #define DBGTRACEW	__noop
      #define DBGTRACEA	__noop
    #endif

    #ifdef UNICODE
		#define cmnTrace	TRACEW
		#define	cmnTraceErr	ERRW
	#else
		#define cmnTrace	TRACEA
		#define	cmnTraceErr	ERRA
    #endif
	//
	// Define own macros without MFC
	#define		TRACE						cmnTrace	
	#define		ERR(str, hr)				cmnTraceErr(str, hr);
	//
	// Message box values - second parameter uType
	#define	MSG_ERROR		MB_ICONERROR
	#define	MSG_WARNING		MB_ICONWARNING
	#define	MSG_INFO		MB_ICONINFORMATION
	//
	// Message box titles
	#define	 _S_WARNINGW		L"Warning"
	#define	 _S_WARNINGA		"Warning"
	#define	 _S_ERRORW			L"Error"
	#define	 _S_ERRORA			"Error"
	#define	 _S_INFOW			L"Information"
	#define	 _S_INFOA			"Information"
	#define	 _S_COMMONW			L"Common message"	
	#define	 _S_COMMONA			"Common message"	
	#define	 UNKNOWN_DX_ERRW	L"Unknown"
	#define	 UNKNOWN_DX_ERRA	"Unknown"
	#define	 _S_LOG_TXTW		L"log.txt"
	#define	 _S_LOG_TXTA		"log.txt"

	#ifdef UNICODE
		#define	 _S_WARNING			_S_WARNINGW
		#define	 _S_ERROR			_S_ERRORW
		#define	 _S_INFO			_S_INFOW
		#define	 _S_COMMON			_S_COMMONW		
		#define	 UNKNOWN_DX_ERR		UNKNOWN_DX_ERRW
		#define	_S_LOG_TXT			_S_LOG_TXTW
	#else
		#define	 _S_WARNING			_S_WARNINGA
		#define	 _S_ERROR			_S_ERRORA
		#define	 _S_INFO			_S_INFOA
		#define	 _S_COMMON			_S_COMMONA	
		#define	 UNKNOWN_DX_ERR		UNKNOWN_DX_ERRA
		#define	_S_LOG_TXT			_S_LOG_TXTA
	#endif
	//
	// Logging flags
	#define LOGFLG_HTML			1		// Generate also HTML log file
	#define LOGFLG_UNICODE		2		// Unicode log file
	// The structure holds settings and other variables of the log module. The structure is available
	// directly only in Common.dll. Other modules have to access it via supplied functions.
	// The string variables are stored in UNICODE coding to avoid unnecessary conversions on WinNT kernel based 
	// Windows
	// Structure is initialized via constructor and cleanup operations are done in destructor
	typedef struct tagLogSettings 
	{		
		DWORD dwFlags;
		BOOL bLogToFile;
		LPWSTR lpwszLogFile;

		HANDLE hLogFile;
		HANDLE hHTMLFile;

		// Now the variables that store the line and filename where the error occured
		WCHAR wszFailFileName[MAX_PATH];
		DWORD dwFailLine;

		tagLogSettings();
		~tagLogSettings();
	} LOGSETTINGS, *LPLOGSETTINGS;
	
	extern LOGSETTINGS g_sLogSettings;

	// The functions that makes the log settings structure accessible to the outer world
	inline COMMON_API BOOL cmnIsLogFileUnicode() { return (g_sLogSettings.dwFlags & LOGFLG_UNICODE); }
	inline COMMON_API BOOL cmnGenerateHTML() { return (g_sLogSettings.dwFlags & LOGFLG_HTML); }
	inline COMMON_API BOOL cmnIsLoggingToFile() { return g_sLogSettings.bLogToFile; }
	inline COMMON_API HANDLE cmnGetLogFileHandle() { return g_sLogSettings.hLogFile; }
	inline COMMON_API HANDLE cmnGetHTMLFileHandle() { return g_sLogSettings.hHTMLFile; }
	inline COMMON_API LPWSTR cmnGetFailFileName() { return g_sLogSettings.wszFailFileName; }
	inline COMMON_API DWORD cmnGetFailLineNumber() { return g_sLogSettings.dwFailLine; }

	COMMON_API DWORD cmnGetLogFileNameW(LPWSTR _lpwszBuffer, LPDWORD _lpdwSize);
	COMMON_API DWORD cmnGetLogFileNameA(LPSTR _lpszBuffer, LPDWORD _lpdwSize);

	#ifdef UNICODE
	  #define cmnGetLogFileName	cmnGetLogFileNameW
	#else
	  #define cmnGetLogFileName	cmnGetLogFileNameA
	#endif

	// Call to init the logger, otherwise you wan't be able to log to the file, if no file
	// is specified, then all logging goes to the debugger output only
	// The _bUnicodeLog value is used only when the filename is specified
	// Common.dll sets its default values as following (using constructor and destructor)
	//		bUnicodeLogFile = FALSE
	//		bLogToFile = FALSE
	//		lpwszLogFile = NULL
	//		hLogFile = NULL
	//		wszFileName[MAX_PATH] undefined
	//		dwLine = 0
	// Note: You cannot change the logfile name and its coding (UNICODE/ANSI) once the object is initialized
	COMMON_API DWORD cmnInitLogW(LPCWSTR _lpwszFile, DWORD dwFlags = LOGFLG_HTML);
	COMMON_API DWORD cmnInitLogA(LPCSTR _lpwszFile, DWORD dwFlags = LOGFLG_HTML);

	#ifdef UNICODE
	  #define cmnInitLog	cmnInitLogW
	#else
	  #define cmnInitLog	cmnInitLogA
	#endif

	/*
		Common exceptions (only ANSI version)
	*/
	// Error types
	enum ERROR_TYPE	{ET_FATAL = 0, ET_CRITICAL, ET_ALERT, ET_ERROR, ET_WARNING, ET_INFORMATION};
	// Flags for msgbox method
	#define MBFLG_MODULE		1
	#define MBFLG_CODE			2
	#define MBFLG_REASON		4
	#define MBFLG_LINE			8
	#define MBFLG_ALL			15
	// Flags for trace method
	#define	TRFLG_TIMESTAMP		1
	#define	TRFLG_MODULE		2
	#define	TRFLG_CODE			4
	#define	TRFLG_ALL			7

	class COMMON_API XException// : public exception
	{
	protected:
		DWORD			m_dwErrorCode;
		char			m_chErrorReason[512];
		ERROR_TYPE		m_dwErrorType;
		char			m_chErrorMsg[512];

		// Time and place from where the exception was throwen
		char			m_szFile[MAX_PATH];
		DWORD			m_dwLine;
		DWORD			m_dwTimeStamp;

	public:		
		XException() throw();		
		XException(char* msg, DWORD code = S_FALSE, ERROR_TYPE type = ET_ERROR) throw();
		// Internal function
		void	SetTimePlace(LPCSTR szFile, DWORD dwLine, DWORD dwTimeStamp) throw();
		virtual	~XException() throw();
		// Saves formatted string into error message
		virtual		void	format(LPCSTR	szFormat, ...) throw();
		// Returns error message
		virtual	const char*	what() const throw() {return m_chErrorMsg;}
		// Returns error code
		virtual DWORD				code() throw() {return m_dwErrorCode;}
		// Returns error type (critical, fatal, warning, alert, error)
		virtual ERROR_TYPE			type() throw() {return m_dwErrorType;}
		// Shows 'OK' message box with error code and error message
		virtual void				msgbox(LPCSTR szCaption = NULL, DWORD dwFlags = MBFLG_REASON|MBFLG_CODE) throw();
		// Returns formated string of the error message and code
		// Returns size of the returned string
		// If the first argument is NULL, method returns size of the message
		// int the dwSize parameter
		virtual void				msg(char * msg, DWORD &dwSize) throw();		
		// Trace error message into log file or output window
		virtual	void				trace(DWORD dwFlags = TRFLG_MODULE|TRFLG_CODE) throw();
	};
	// Throws XException
	COMMON_API	void	cmnThrow(LPCSTR szFile, DWORD dwLine, XException &pException);
	// Macro for XException throwing
	#define		THROWA(exp)	cmnThrow(__FILE__, __LINE__, exp);
	// Macro for XException throwing
	#define		THROW(exp)	THROWA(exp)	

	// HTML formats
	// header
	#define	HTML_HEADER "<html><head><title>%s</title></head><body><font face='Courier' size='2pt'>"
	#define	HTML_FOOTER "</font></body></html>"
	// information (generated by TRACE)
	#define	HTML_INFORMATION "<font color='green'>%s</font><br>"
	// warning (exception)
	#define	HTML_WARNING "<font color='orange'>%s</font><br>"
	// alert (exception)
	#define	HTML_ALERT "<font color='blue'>%s</font><br>"
	// error (exception)
	#define	HTML_ERROR "<font color='red'>%s</font><br>"
	// fatal or critical error (exception)
	#define	HTML_FATAL "<font color='black'>%s</font><br>"
	
#endif // _CMNLOG_H_