/*
 * CmnMisc.cpp - Set/Get common functions
 *
 * Date:	about 2000
 * Authors:	Jiri Formanek & Ondrej Burisin
 * Update: 12.1. 2003
 *
 */
#include "stdafx.h"

#define COMMON_API __declspec(dllexport)

#include "Symbols.h"
#include "CmnMisc.h"
#include "CmnLog.h"
#include "CmnStr.h"
#include "CmnError.h"
#include <math.h>

wchar_t		g_wszIniFilePath[MAX_PATH];

COMMON_API	DWORD cmnSetIniFileNameA(LPCSTR	szIniFileName)
{
	DWORD	dwRet, dwLen;
	wchar_t	wszFilename[MAX_PATH];
	// Convert input parameter to UNICODE
	dwLen = 0;
	// Get size of UNICODE string
	dwRet = cmnConvertANSIToUNICODE(NULL, &dwLen, (LPSTR)szIniFileName);
	if(dwRet != ERROR_SUCCESS) {
		return dwRet;
	}
	// Check space of our string to path storing
	if(dwLen > MAX_PATH) {
        return ERROR_INVALID_PARAMETER;
	}
	// Convert input string to our wchar string
	dwRet = cmnConvertANSIToUNICODE(wszFilename, &dwLen, (LPSTR)szIniFileName);
	if(dwRet != ERROR_SUCCESS) {
		return dwRet;
	}
	//
	// Add full path to .ini file
	cmnGetDataFilePathW(g_wszIniFilePath, wszFilename);
	return dwRet;
}

COMMON_API	DWORD cmnSetIniFileNameW(LPCWSTR	wszIniFileName)
{
	//
	// simply copy input string to our wstring and add full path
	cmnGetDataFilePathW(g_wszIniFilePath, wszIniFileName);
	return ERROR_SUCCESS;
}

COMMON_API	DWORD		cmnReadSetupStringW(LPCWSTR wszSection, LPCWSTR wszKey, LPWSTR wszOutput, DWORD nSize)
{
	return GetPrivateProfileStringW(wszSection, wszKey, _S_EMPTYW, wszOutput, nSize, g_wszIniFilePath);
}

COMMON_API	DWORD		cmnReadSetupStringA(LPCSTR szSection, LPCSTR szKey, LPSTR szOutput, DWORD nSize)
{
	char	szFilename[MAX_PATH];
	DWORD dwLen, dwRet;
	// Convert UNICODE path to ANSI
	dwLen = 0;
	// Get size of UNICODE string
	dwRet = cmnConvertUNICODEToANSI(NULL, &dwLen, (LPWSTR)g_wszIniFilePath);
	if(dwRet != ERROR_SUCCESS) {
		return dwRet;
	}
	// Check space of our string to path storing
	if(dwLen > MAX_PATH) {
        return ERROR_INVALID_PARAMETER;
	}
	// Convert UNICODE path to ANSI
	dwRet =  cmnConvertUNICODEToANSI(szFilename, &dwLen, (LPWSTR)g_wszIniFilePath);
	if(dwRet != ERROR_SUCCESS) {
		return dwRet;
	}
	return GetPrivateProfileStringA(szSection, szKey, _S_EMPTYA, szOutput, nSize, szFilename);
}

COMMON_API	int		cmnReadSetupIntW(LPCWSTR wszSection, LPCWSTR wszKey)
{
	return GetPrivateProfileIntW(wszSection, wszKey, 0, g_wszIniFilePath);
}

COMMON_API	int		cmnReadSetupIntA(LPCSTR szSection, LPCSTR szKey)
{
	char	szFilename[MAX_PATH];
	DWORD dwLen, dwRet;
	// Convert UNICODE path to ANSI
	dwLen = 0;
	// Get size of UNICODE string
	dwRet = cmnConvertUNICODEToANSI(NULL, &dwLen, (LPWSTR)g_wszIniFilePath);
	if(dwRet != ERROR_SUCCESS) {
		return dwRet;
	}
	// Check space of our string to path storing
	if(dwLen > MAX_PATH) {
        return ERROR_INVALID_PARAMETER;
	}
	// Convert UNICODE path to ANSI
	dwRet =  cmnConvertUNICODEToANSI(szFilename, &dwLen, (LPWSTR)g_wszIniFilePath);
	if(dwRet != ERROR_SUCCESS) {
		return dwRet;
	}
	return GetPrivateProfileIntA(szSection, szKey, 0, (LPSTR)szFilename);
}

COMMON_API	DWORD		cmnWriteSetupStringW(LPCWSTR wszSection, LPCWSTR wszKey, LPWSTR wszInput)
{
	return WritePrivateProfileStringW(wszSection, wszKey, wszInput, (LPWSTR)g_wszIniFilePath);
}

COMMON_API	DWORD		cmnWriteSetupStringA(LPCSTR szSection, LPCSTR szKey, LPSTR szInput)
{
	char	szFilename[MAX_PATH];
	DWORD dwLen, dwRet;
	// Convert UNICODE path to ANSI
	dwLen = 0;
	// Get size of UNICODE string
	dwRet = cmnConvertUNICODEToANSI(NULL, &dwLen, (LPWSTR)g_wszIniFilePath);
	if(dwRet != ERROR_SUCCESS) {
		return dwRet;
	}
	// Check space of our string to path storing
	if(dwLen > MAX_PATH) {
        return ERROR_INVALID_PARAMETER;
	}
	// Convert UNICODE path to ANSI
	dwRet =  cmnConvertUNICODEToANSI(szFilename, &dwLen, (LPWSTR)g_wszIniFilePath);
	if(dwRet != ERROR_SUCCESS) {
		return dwRet;
	}
	return WritePrivateProfileStringA(szSection, szKey, szInput, (LPSTR)szFilename);
}

COMMON_API	DWORD		cmnWriteSetupIntW(LPCWSTR wszSection, LPCWSTR wszKey, int iValue)
{
	wchar_t	wszValue[MAX_PATH];
	/* convert int value to string */
	swprintf(wszValue, L"%d", iValue);	
	/* write int as string */
	return cmnWriteSetupStringW(wszSection, wszKey, wszValue);
}

COMMON_API	DWORD		cmnWriteSetupIntA(LPCSTR szSection, LPCSTR szKey, int iValue)
{	
	char	szValue[MAX_PATH];	
	/* convert int value to string */
	sprintf(szValue, "%d", iValue);	
	/* write int as string */
	return cmnWriteSetupStringA(szSection, szKey, szValue);
}

COMMON_API	float		cmnGetTime(DWORD dwTimerCommand)
{
	static BOOL     m_bTimerInitialized = FALSE;
    static BOOL     m_bUsingQPF         = FALSE;
    static BOOL     m_bTimerStopped     = TRUE;
    static LONGLONG m_llQPFTicksPerSec  = 0;

    // Initialize the timer
    if( FALSE == m_bTimerInitialized )
    {
        m_bTimerInitialized = TRUE;

        // Use QueryPerformanceFrequency() to get frequency of timer.  If QPF is
        // not supported, we will timeGetTime() which returns milliseconds.
        LARGE_INTEGER qwTicksPerSec;
        m_bUsingQPF = QueryPerformanceFrequency( &qwTicksPerSec );
        if( m_bUsingQPF )
            m_llQPFTicksPerSec = qwTicksPerSec.QuadPart;
    }

    if(m_bUsingQPF)
    {
        static LONGLONG m_llStopTime        = 0;
        static LONGLONG m_llLastElapsedTime = 0;
		static LONGLONG m_llLastElapsedTime1 = 0;
		static LONGLONG m_llLastElapsedTime2 = 0;
        static LONGLONG m_llBaseTime        = 0;
        double fTime;
        double fElapsedTime;
        LARGE_INTEGER qwTime;
        
        // Get either the current time or the stop time, depending
        // on whether we're stopped and what command was sent
        if( m_llStopTime != 0 && dwTimerCommand != TIMER_START && dwTimerCommand != TIMER_GETABSOLUTETIME)
            qwTime.QuadPart = m_llStopTime;
        else
            QueryPerformanceCounter( &qwTime );

        // Return the elapsed time
        if( dwTimerCommand == TIMER_GETELAPSEDTIME )
        {
            fElapsedTime = (double) ( qwTime.QuadPart - m_llLastElapsedTime ) / (double) m_llQPFTicksPerSec;
            m_llLastElapsedTime = qwTime.QuadPart;
            return (FLOAT) fElapsedTime;
        }
		if( dwTimerCommand == TIMER_GETELAPSEDTIME1 )
        {
            fElapsedTime = (double) ( qwTime.QuadPart - m_llLastElapsedTime1 ) / (double) m_llQPFTicksPerSec;
            m_llLastElapsedTime1 = qwTime.QuadPart;
            return (FLOAT) fElapsedTime;
        }
		if( dwTimerCommand == TIMER_GETELAPSEDTIME2 )
        {
            fElapsedTime = (double) ( qwTime.QuadPart - m_llLastElapsedTime2 ) / (double) m_llQPFTicksPerSec;
            m_llLastElapsedTime2 = qwTime.QuadPart;
            return (FLOAT) fElapsedTime;
        }
    
        // Return the current time
        if( dwTimerCommand == TIMER_GETAPPTIME )
        {
            double fAppTime = (double) ( qwTime.QuadPart - m_llBaseTime ) / (double) m_llQPFTicksPerSec;
            return (FLOAT) fAppTime;
        }
    
        // Reset the timer
        if( dwTimerCommand == TIMER_RESET )
        {
            m_llBaseTime        = qwTime.QuadPart;
            m_llLastElapsedTime = qwTime.QuadPart;
            m_llStopTime        = 0;
            m_bTimerStopped     = FALSE;
            return 0.0f;
        }
    
        // Start the timer
        if( dwTimerCommand == TIMER_START )
        {
            if( m_bTimerStopped )
                m_llBaseTime += qwTime.QuadPart - m_llStopTime;
            m_llStopTime = 0;
            m_llLastElapsedTime = qwTime.QuadPart;
            m_bTimerStopped = FALSE;
            return 0.0f;
        }
    
        // Stop the timer
        if( dwTimerCommand == TIMER_STOP )
        {
            m_llStopTime = qwTime.QuadPart;
            m_llLastElapsedTime = qwTime.QuadPart;
            m_bTimerStopped = TRUE;
            return 0.0f;
        }
    
        // Advance the timer by 1/10th second
        if( dwTimerCommand == TIMER_ADVANCE )
        {
            m_llStopTime += m_llQPFTicksPerSec/10;
            return 0.0f;
        }

        if( dwTimerCommand == TIMER_GETABSOLUTETIME )
        {
            fTime = qwTime.QuadPart / (double) m_llQPFTicksPerSec;
            return (FLOAT) fTime;
        }

        return -1.0f; // Invalid command specified
    }
    else
    {
        // Get the time using timeGetTime()
        static double m_fLastElapsedTime  = 0.0;
        static double m_fBaseTime         = 0.0;
        static double m_fStopTime         = 0.0;
        double fTime;
        double fElapsedTime;
        
        // Get either the current time or the stop time, depending
        // on whether we're stopped and what command was sent
        if( m_fStopTime != 0.0 && dwTimerCommand != TIMER_START && dwTimerCommand != TIMER_GETABSOLUTETIME)
            fTime = m_fStopTime;
        else
            fTime = GetTickCount() * 0.001;
    
        // Return the elapsed time
        if( dwTimerCommand == TIMER_GETELAPSEDTIME )
        {   
            fElapsedTime = (double) (fTime - m_fLastElapsedTime);
            m_fLastElapsedTime = fTime;
            return (FLOAT) fElapsedTime;
        }
    
        // Return the current time
        if( dwTimerCommand == TIMER_GETAPPTIME )
        {
            return (FLOAT) (fTime - m_fBaseTime);
        }
    
        // Reset the timer
        if( dwTimerCommand == TIMER_RESET )
        {
            m_fBaseTime         = fTime;
            m_fLastElapsedTime  = fTime;
            m_fStopTime         = 0;
            m_bTimerStopped     = FALSE;
            return 0.0f;
        }
    
        // Start the timer
        if( dwTimerCommand == TIMER_START )
        {
            if( m_bTimerStopped )
                m_fBaseTime += fTime - m_fStopTime;
            m_fStopTime = 0.0f;
            m_fLastElapsedTime  = fTime;
            m_bTimerStopped = FALSE;
            return 0.0f;
        }
    
        // Stop the timer
        if( dwTimerCommand == TIMER_STOP )
        {
            m_fStopTime = fTime;
            m_fLastElapsedTime  = fTime;
            m_bTimerStopped = TRUE;
            return 0.0f;
        }
    
        // Advance the timer by 1/10th second
        if( dwTimerCommand == TIMER_ADVANCE )
        {
            m_fStopTime += 0.1f;
            return 0.0f;
        }

        if( dwTimerCommand == TIMER_GETABSOLUTETIME )
        {
            return (FLOAT) fTime;
        }

        return -1.0f; // Invalid command specified
    }	
}

/*
 *  Function: cmnWriteRegistryBinW (UNICODE version)
 *  Writes binary data to registry
 *
 *	hKey parameter specify root key:
 *	HKEY_CURRENT_USER - user specific data
 *	HKEY_LOCAL_MACHINE - computer data (administrator's rights needed)
 *
 *	Data is specified by last two parameters:
 *	lpData - pointer to byte array
 *	dwSize - size in bytes of the array (max. 32767 bytes)
 *
 *	Data is written to location specified by first parameter: 
 *	wszSubkeyName - string of subkey name that is created or opened if exist
 *
 *	Value name is specified by second parameter: (max. 256 wide-chars)
 *	wszValueName - string of value name
 *
 *	Data is written as:
 *	HKEY_CURRENT_USER\wszSubkeyName->wszValueName=lpData
 *
 *	Returns:
 *	ERROR_SUCCESS(0) - success.
 *	E_CMN_REG_CREATE_FAIL - cannot open or create sub key.
 *  E_CMN_REG_TOO_LARGE_VALUE - data is too large.
 *	E_CMN_REG_WRITE_FAIL - cannot write data.
 *	E_CMN_INVALID_PARAMETER - if one or more pamaters are invalid (NULL).
 *  E_CMN_REG_INVALID_HKEY - if hKey is not HKEY_CURRENT_USER or HKEY_LOCAL_MACHINE
 *	E_CMN_REG_INVALID_RIGHTS - returns if user pass HKEY_LOCAL_MACHINE as hKey and user is  not admin
 *  
 */
COMMON_API	DWORD	cmnWriteRegistryBinW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, LPBYTE lpData, DWORD dwSize)
{
	HKEY hLocalKey = NULL;
	DWORD dwRet;
	/* check input */
	if(!wszSubkeyName || !wszValueName || !lpData) {
		return E_CMN_INVALID_PARAMETER;
	}
	/* check hKey */
	if(!(hKey == HKEY_CURRENT_USER || hKey == HKEY_LOCAL_MACHINE)) {
		return E_CMN_REG_INVALID_HKEY;
	}
	/* check administrator's rights if user wants to write to local machine */
	if(hKey == HKEY_LOCAL_MACHINE) {
		#pragma COMMENT(TODO: Check admin to write to HKEY_LOCAL_MACHINE.)
		//return ERROR_CALL_NOT_IMPLEMENTED;
	}
	/*	Create or open desired subkey for writting */		
	dwRet = RegCreateKeyExW(hKey, wszSubkeyName, 0, 
							NULL, REG_OPTION_NON_VOLATILE, 
							KEY_WRITE, NULL, &hLocalKey, NULL);
	if(dwRet == ERROR_BAD_ENVIRONMENT) {
		return	E_CMN_REG_INVALID_RIGHTS;
	}
	if(ERROR_SUCCESS != dwRet) {		
		return	E_CMN_REG_CREATE_FAIL;
	}
	/* check if the data size is not too large */
	/* test ANSI because we want to write bytes */
	if(dwSize > MAX_VALUE_ANSI_SIZE) {
		return E_CMN_REG_TOO_LARGE_VALUE;
	}
	/* if dwSize > 2048, recommend external file */
	if(dwSize > MAX_RECOMMENDED_VALUE_SIZE) {
		TRACEW(L"The value size is too large. Windows recommended you to store data in a external file.");
	}
	/* set value in subkey */
	if(hLocalKey) {
		dwRet = RegSetValueExW(hLocalKey, wszValueName, 0, REG_BINARY, lpData, dwSize);		
		/* close registry */
		RegCloseKey(hLocalKey);
		hLocalKey = NULL;
		/* analyze return value */
		if(dwRet != ERROR_SUCCESS) {
			return	E_CMN_REG_WRITE_FAIL;	
		}	
	}
	return dwRet;
}
/* ANSI version converts input parameters to UNICODE and call UNICODE version */
COMMON_API	DWORD	cmnWriteRegistryBinA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, LPBYTE lpData, DWORD dwSize)
{
	HKEY hLocalKey = NULL;
	DWORD dwRet;
	/* check input */
	if(!szSubkeyName || !szValueName || !lpData) {
		return E_CMN_INVALID_PARAMETER;
	}
	/* check hKey */
	if(!(hKey == HKEY_CURRENT_USER || hKey == HKEY_LOCAL_MACHINE)) {
		return E_CMN_REG_INVALID_HKEY;
	}
	/* check administrator's rights if user wants to write to local machine */
	if(hKey == HKEY_LOCAL_MACHINE) {
		#pragma COMMENT(TODO: Check admin to write to HKEY_LOCAL_MACHINE.)
		//return ERROR_CALL_NOT_IMPLEMENTED;
	}
	/*	Create or open desired subkey for writting */		
	dwRet = RegCreateKeyExA(hKey, szSubkeyName, 0, 
							NULL, REG_OPTION_NON_VOLATILE, 
							KEY_WRITE, NULL, &hLocalKey, NULL);
	if(dwRet == ERROR_BAD_ENVIRONMENT) {
		return	E_CMN_REG_INVALID_RIGHTS;
	}
	if(ERROR_SUCCESS != dwRet) {
		return	E_CMN_REG_CREATE_FAIL;
	}
	/* check if the data size is not too large */
	/* test ANSI because we want to write bytes */
	if(dwSize > MAX_VALUE_ANSI_SIZE) {
		return E_CMN_REG_TOO_LARGE_VALUE;
	}
	/* if dwSize > 2048, recommend external file */
	if(dwSize > MAX_RECOMMENDED_VALUE_SIZE) {
		TRACEA("The value size is too large. Windows recommended you to store data in a external file.");
	}
	/* set value in subkey */
	if(hLocalKey) {
		dwRet = RegSetValueExA(hLocalKey, szValueName, 0, REG_BINARY, lpData, dwSize);
		/* close registry */
		RegCloseKey(hLocalKey);
		hLocalKey = NULL;
		/* analyze return value */
		if(dwRet != ERROR_SUCCESS) {
			return	E_CMN_REG_WRITE_FAIL;	
		}	
	}
	return dwRet;
}

/*
 *  Function: cmnReadRegistryBinW (UNICODE version)
 *	Reads binary data from registry
 * 
 *	hKey parameter specify root key:
 *	HKEY_CURRENT_USER - user specific data
 *	HKEY_LOCAL_MACHINE - computer data 	
 *
 *	Data is stored to third parameters:
 *	lpData - pointer to byte array, may be NULL, then function returns size of target 
 *			 value in the next parameter
 *	dwSize - size in bytes of the array (max. 32767 bytes)
 *
 *	Data is readen from location specified by first parameter: 
 *	wszSubkeyName - string of subkey name that is created or opened if exist
 *
 *	Value name is specified by second parameter: (max. 256 wide-chars)
 *	wszValueName - string of value name
 *
 *	Data is readen from:
 *	HKEY_CURRENT_USER\wszSubkeyName->wszValueName=lpData
 *
 *	Returns:
 *	ERROR_SUCCESS(0) - success.
 *	E_CMN_REG_INVALID_TYPE - type of target value is not binary
 *  E_CMN_REG_INVALID_HKEY - if hKey is not HKEY_CURRENT_USER or HKEY_LOCAL_MACHINE
 *	E_CMN_REG_INVALID_BUFFER - input buffer is too small.
 *	E_CMN_REG_OPEN_FAIL - cannot open sub key.
 *	E_CMN_REG_KEY_NOT_FOUND - if specified key not found in Windows registry
 *  E_CMN_REG_TOO_LARGE_VALUE - data is too large.
 *	E_CMN_REG_READ_FAIL - cannot read data. 
 *	E_CMN_INVALID_PARAMETER - if one or more pamaters are invalid (NULL).
 *  
 */
COMMON_API	DWORD	cmnReadRegistryBinW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, LPBYTE lpData, LPDWORD lpdwSize)
{
	HKEY hLocalKey = NULL;
	DWORD dwRet, dwRealSize, dwType;	
	/* check input */
	if(!lpdwSize || !wszSubkeyName || !wszValueName) {
		return E_CMN_INVALID_PARAMETER;
	}
	/* check hKey */
	if(!(hKey == HKEY_CURRENT_USER || hKey == HKEY_LOCAL_MACHINE)) {
		return E_CMN_REG_INVALID_HKEY;
	}
	/*	Create or open desired subkey for writting */		
	dwRet = RegOpenKeyExW(hKey, wszSubkeyName, 0, KEY_READ, &hLocalKey);
	if(ERROR_FILE_NOT_FOUND == dwRet) {
		return E_CMN_REG_KEY_NOT_FOUND;
	}
	if(ERROR_SUCCESS != dwRet) {
		return	E_CMN_REG_OPEN_FAIL;
	}	
	if(hLocalKey) {
		/* get size of real value in registry */
		dwRet = RegQueryValueExW(hLocalKey, wszValueName, NULL, &dwType, NULL, &dwRealSize);
		if(dwRet != ERROR_SUCCESS) {
			return	E_CMN_REG_READ_FAIL;	
		}
		/* check type of data */
		if(dwType != REG_BINARY) {
			return E_CMN_REG_INVALID_TYPE;
		}
		/* user wants only size of value */
		if(lpData == NULL) {
			*lpdwSize = dwRealSize;
			/* close registry */
			dwRet = RegCloseKey(hLocalKey);
			return dwRet;
		}
		/* check the size of input buffer */
		if(*lpdwSize < dwRealSize) {
			return E_CMN_REG_INVALID_BUFFER;
		}
		/* get data from registry */
		dwRet = RegQueryValueExW(hLocalKey, wszValueName, NULL, &dwType, lpData, lpdwSize);		
		/* analyze return value */
		if(dwRet != ERROR_SUCCESS) {
			return	E_CMN_REG_READ_FAIL;	
		}	
		/* close registry */
		dwRet = RegCloseKey(hLocalKey);
	}
	return dwRet;
}
/* ANSI version is same as UNICODE, only call ANSI API functions */
COMMON_API	DWORD	cmnReadRegistryBinA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, LPBYTE lpData, LPDWORD lpdwSize)
{
	HKEY hLocalKey = NULL;
	DWORD dwRet, dwRealSize, dwType;	
	/* check input */
	if(!lpdwSize || !szSubkeyName || !szValueName) {
		return E_CMN_INVALID_PARAMETER;
	}
	/* check hKey */
	if(!(hKey == HKEY_CURRENT_USER || hKey == HKEY_LOCAL_MACHINE)) {
		return E_CMN_REG_INVALID_HKEY;
	}
	/*	Create or open desired subkey for writting */		
	dwRet = RegOpenKeyExA(hKey, szSubkeyName, 0, KEY_READ, &hLocalKey);
	if(ERROR_FILE_NOT_FOUND == dwRet) {
		return E_CMN_REG_KEY_NOT_FOUND;
	}
	if(ERROR_SUCCESS != dwRet) {
		return	E_CMN_REG_OPEN_FAIL;
	}	
	if(hLocalKey) {
		/* get size of real value in registry */
		dwRet = RegQueryValueEx(hLocalKey, szValueName, NULL, &dwType, NULL, &dwRealSize);
		if(dwRet != ERROR_SUCCESS) {
			return	E_CMN_REG_READ_FAIL;	
		}
		/* check type of data */
		if(dwType != REG_BINARY) {
			return E_CMN_REG_INVALID_TYPE;
		}
		/* user wants only size of value */
		if(lpData == NULL) {
			*lpdwSize = dwRealSize;
			/* close registry */
			dwRet = RegCloseKey(hLocalKey);
			return dwRet;
		}
		/* check the size of input buffer */
		if(*lpdwSize < dwRealSize) {
			return E_CMN_REG_INVALID_BUFFER;
		}
		/* get data from registry */
		dwRet = RegQueryValueEx(hLocalKey, szValueName, NULL, &dwType, lpData, lpdwSize);		
		/* analyze return value */
		if(dwRet != ERROR_SUCCESS) {
			return	E_CMN_REG_READ_FAIL;	
		}	
		/* close registry */
		dwRet = RegCloseKey(hLocalKey);
	}
	return dwRet;
}

/*
 *  Function: cmnWriteRegistryBinW (UNICODE version)
 *  Writes string to registry
 *
 *	hKey parameter specify root key:
 *	HKEY_CURRENT_USER - user specific data
 *	HKEY_LOCAL_MACHINE - computer data (administrator's rights needed)
 *
 *	Data is specified by last parameter:
 *	wszInput - pointer to string (max. len 32767 wide-chars) 
 *
 *	Data is written to location specified by first parameter: 
 *	wszSubkeyName - string of subkey name that is created or opened if exist
 *
 *	Value name is specified by second parameter: (max. 256 wide-chars)
 *	wszValueName - string of value name
 *
 *	Data is written as:
 *	HKEY_CURRENT_USER\wszSubkeyName->wszValueName=wszInput
 *
 *	Returns:
 *	ERROR_SUCCESS(0) - success.
 *	E_CMN_REG_CREATE_FAIL - cannot open or create sub key.
 *  E_CMN_REG_TOO_LARGE_VALUE - data is too large.
 *	E_CMN_REG_WRITE_FAIL - cannot write data.
 *	E_CMN_INVALID_PARAMETER - if one or more pamaters are invalid (NULL).
 *  E_CMN_REG_INVALID_HKEY - if hKey is not HKEY_CURRENT_USER or HKEY_LOCAL_MACHINE
 *	ERROR_CALL_NOT_IMPLEMENTED - returns if user pass HKEY_LOCAL_MACHINE as hKey
 *  
 */
COMMON_API	DWORD	cmnWriteRegistryStrW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, LPCWSTR wszInput)
{
	HKEY hLocalKey = NULL;
	DWORD dwRet, dwSize;
	/* check input */
	if(!wszSubkeyName || !wszValueName || !wszInput) {
		return E_CMN_INVALID_PARAMETER;
	}
	/* check hKey */
	if(!(hKey == HKEY_CURRENT_USER || hKey == HKEY_LOCAL_MACHINE)) {
		return E_CMN_REG_INVALID_HKEY;
	}
	/* check administrator's rights if user wants to write to local machine */
	if(hKey == HKEY_LOCAL_MACHINE) {
		#pragma COMMENT(TODO: Check admin to write to HKEY_LOCAL_MACHINE.)
		return ERROR_CALL_NOT_IMPLEMENTED;
	}
	/*	Create or open desired subkey for writting */		
	dwRet = RegCreateKeyExW(hKey, wszSubkeyName, 0, 
							NULL, REG_OPTION_NON_VOLATILE, 
							KEY_WRITE, NULL, &hLocalKey, NULL);
	if(ERROR_SUCCESS != dwRet) {
		return	E_CMN_REG_CREATE_FAIL;
	}
	/* get size of string */
	dwSize = (DWORD)wcslen(wszInput)+1; // + null char
	/* string is UNICODE - each char has 2 bytes */
	dwSize *= 2;
	/* check if the data size is not too large */
	/* test ANSI because we want to write bytes */
	if(dwSize > MAX_VALUE_ANSI_SIZE) {
		return E_CMN_REG_TOO_LARGE_VALUE;
	}
	/* if dwSize > 2048, recommend external file */
	if(dwSize > MAX_RECOMMENDED_VALUE_SIZE) {
		TRACEW(L"The value size is too large. Windows recommended you to store data in a external file.");
	}
	/* set value in subkey */
	if(hLocalKey) {
		dwRet = RegSetValueExW(hLocalKey, wszValueName, 0, REG_SZ, (LPBYTE)wszInput, dwSize);
		/* close registry */
		RegCloseKey(hLocalKey);
		hLocalKey = NULL;
		/* analyze return value */
		if(dwRet != ERROR_SUCCESS) {
			return	E_CMN_REG_WRITE_FAIL;	
		}	
	}
	return dwRet;	
}
/* ANSI version is same as UNICODE, only call ANSI API functions */
COMMON_API	DWORD	cmnWriteRegistryStrA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, LPCSTR szInput)
{
	HKEY hLocalKey = NULL;
	DWORD dwRet, dwSize;
	/* check input */
	if(!szSubkeyName || !szValueName || !szInput) {
		return E_CMN_INVALID_PARAMETER;
	}
	/* check hKey */
	if(!(hKey == HKEY_CURRENT_USER || hKey == HKEY_LOCAL_MACHINE)) {
		return E_CMN_REG_INVALID_HKEY;
	}
	/* check administrator's rights if user wants to write to local machine */
	if(hKey == HKEY_LOCAL_MACHINE) {
		#pragma COMMENT(TODO: Check admin to write to HKEY_LOCAL_MACHINE.)
		return ERROR_CALL_NOT_IMPLEMENTED;
	}
	/*	Create or open desired subkey for writting */		
	dwRet = RegCreateKeyExA(hKey, szSubkeyName, 0, 
							NULL, REG_OPTION_NON_VOLATILE, 
							KEY_WRITE, NULL, &hLocalKey, NULL);
	if(ERROR_SUCCESS != dwRet) {
		return	E_CMN_REG_CREATE_FAIL;
	}
	/* get size of string */
	dwSize = (DWORD)strlen(szInput)+1; // + null char
	/* check if the data size is not too large */
	/* test ANSI because we want to write bytes */
	if(dwSize > MAX_VALUE_ANSI_SIZE) {
		return E_CMN_REG_TOO_LARGE_VALUE;
	}
	/* if dwSize > 2048, recommend external file */
	if(dwSize > MAX_RECOMMENDED_VALUE_SIZE) {
		TRACEA("The value size is too large. Windows recommended you to store data in a external file.");
	}
	/* set value in subkey */
	if(hLocalKey) {
		dwRet = RegSetValueExA(hLocalKey, szValueName, 0, REG_SZ, (LPBYTE)szInput, dwSize);
		/* close registry */
		RegCloseKey(hLocalKey);
		hLocalKey = NULL;
		/* analyze return value */
		if(dwRet != ERROR_SUCCESS) {
			return	E_CMN_REG_WRITE_FAIL;	
		}	
	}
	return dwRet;	
}

/*
 *  Function: cmnReadRegistryStrW (UNICODE version)
 *	Reads string from registry
 * 
 *	hKey parameter specify root key:
 *	HKEY_CURRENT_USER - user specific data
 *	HKEY_LOCAL_MACHINE - computer data 	
 *
 *	Data is stored to third parameters:
 *	wszOutput - pointer to string, may be NULL, then function returns size of target 
 *			    value in the next parameter
 *	lpdwSize - pointer to DWORD, size in bytes of the array (max. 32767 bytes)
 *
 *	Data is readen from location specified by first parameter: 
 *	wszSubkeyName - string of subkey name that is created or opened if exist
 *
 *	Value name is specified by second parameter: (max. 256 wide-chars)
 *	wszValueName - string of value name
 *
 *	Data is readen from:
 *	HKEY_CURRENT_USER\wszSubkeyName->wszValueName=string->wszOutput
 *
 *	Returns:
 *	ERROR_SUCCESS(0) - success.
 *	E_CMN_REG_INVALID_TYPE - type of target value is not binary
 *  E_CMN_REG_INVALID_HKEY - if hKey is not HKEY_CURRENT_USER or HKEY_LOCAL_MACHINE
 *	E_CMN_REG_INVALID_BUFFER - input buffer is too small.
 *	E_CMN_REG_OPEN_FAIL - cannot open sub key.
 *	E_CMN_REG_KEY_NOT_FOUND - if specified key not found in Windows registry
 *  E_CMN_REG_TOO_LARGE_VALUE - data is too large.
 *	E_CMN_REG_READ_FAIL - cannot read data. 
 *	E_CMN_INVALID_PARAMETER - if one or more pamaters are invalid (NULL).
 *  
 */
COMMON_API	DWORD	cmnReadRegistryStrW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, LPWSTR wszOutput, LPDWORD lpdwSize, LPCWSTR wszDefault)
{
	HKEY hLocalKey = NULL;
	DWORD dwRet, dwRealSize, dwType;	
	/* check input */
	if(!lpdwSize || !wszSubkeyName || !wszValueName) {
		return E_CMN_INVALID_PARAMETER;
	}
	/* check hKey */
	if(!(hKey == HKEY_CURRENT_USER || hKey == HKEY_LOCAL_MACHINE)) {
		return E_CMN_REG_INVALID_HKEY;
	}
	/*	Create or open desired subkey for writting */		
	dwRet = RegOpenKeyExW(hKey, wszSubkeyName, 0, KEY_READ, &hLocalKey);
	if(ERROR_FILE_NOT_FOUND == dwRet) {
		return E_CMN_REG_KEY_NOT_FOUND;
	}
	if(ERROR_SUCCESS != dwRet) {
		return	E_CMN_REG_OPEN_FAIL;
	}	
	if(hLocalKey) {
		/* get size of real value in registry */
		dwRet = RegQueryValueExW(hLocalKey, wszValueName, NULL, &dwType, NULL, &dwRealSize);
		if(dwRet != ERROR_SUCCESS) {
			return	E_CMN_REG_READ_FAIL;	
		}
		/* check type of data */
		if(dwType != REG_SZ) {
			return E_CMN_REG_INVALID_TYPE;
		}
		/* user wants only size of value */
		if(wszOutput == NULL) {
			*lpdwSize = dwRealSize;
			/* close registry */
			dwRet = RegCloseKey(hLocalKey);
			return dwRet;
		}
		/* check the size of input buffer */
		if(*lpdwSize < dwRealSize) {
			return E_CMN_REG_INVALID_BUFFER;
		}
		/* get data from registry */
		dwRet = RegQueryValueExW(hLocalKey, wszValueName, NULL, &dwType, (LPBYTE)wszOutput, lpdwSize);		
		/* analyze return value */
		if(dwRet != ERROR_SUCCESS) {
			return	E_CMN_REG_READ_FAIL;	
		}	
		/* close registry */
		dwRet = RegCloseKey(hLocalKey);
	}
	return dwRet;
}

/* ANSI version is same as UNICODE, only call ANSI API functions */
COMMON_API	DWORD	cmnReadRegistryStrA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, LPSTR szOutput, LPDWORD lpdwSize, LPCSTR szDefault)
{
	HKEY hLocalKey = NULL;
	DWORD dwRet, dwRealSize, dwType;	
	/* check input */
	if(!lpdwSize || !szSubkeyName || !szValueName) {
		return E_CMN_INVALID_PARAMETER;
	}
	/* check hKey */
	if(!(hKey == HKEY_CURRENT_USER || hKey == HKEY_LOCAL_MACHINE)) {
		return E_CMN_REG_INVALID_HKEY;
	}
	/*	Create or open desired subkey for writting */		
	dwRet = RegOpenKeyExA(hKey, szSubkeyName, 0, KEY_READ, &hLocalKey);
	if(ERROR_FILE_NOT_FOUND == dwRet) {
		return E_CMN_REG_KEY_NOT_FOUND;
	}
	if(ERROR_SUCCESS != dwRet) {
		return	E_CMN_REG_OPEN_FAIL;
	}	
	if(hLocalKey) {
		/* get size of real value in registry */
		dwRet = RegQueryValueExA(hLocalKey, szValueName, NULL, &dwType, NULL, &dwRealSize);
		if(dwRet != ERROR_SUCCESS) {
			return	E_CMN_REG_READ_FAIL;	
		}
		/* check type of data */
		if(dwType != REG_SZ) {
			return E_CMN_REG_INVALID_TYPE;
		}
		/* user wants only size of value */
		if(szOutput == NULL) {
			*lpdwSize = dwRealSize;
			/* close registry */
			dwRet = RegCloseKey(hLocalKey);
			return dwRet;
		}
		/* check the size of input buffer */
		if(*lpdwSize < dwRealSize) {
			return E_CMN_REG_INVALID_BUFFER;
		}
		/* get data from registry */
		dwRet = RegQueryValueExA(hLocalKey, szValueName, NULL, &dwType, (LPBYTE)szOutput, lpdwSize);		
		/* analyze return value */
		if(dwRet != ERROR_SUCCESS) {
			return	E_CMN_REG_READ_FAIL;	
		}	
		/* close registry */
		dwRet = RegCloseKey(hLocalKey);
	}
	return dwRet;
}

/*
 *  Function: cmnWriteRegistryIntW (UNICODE version)
 *  Writes integer value to registry
 *
 *	hKey parameter specify root key:
 *	HKEY_CURRENT_USER - user specific data
 *	HKEY_LOCAL_MACHINE - computer data (administrator's rights needed)
 *
 *	Data is specified by last parameter:
 *	nValue - integer value to write
 *
 *	Data is written to location specified by first parameter: 
 *	wszSubkeyName - string of subkey name that is created or opened if exist
 *
 *	Value name is specified by second parameter: (max. 256 wide-chars)
 *	wszValueName - string of value name
 *
 *	Data is written as:
 *	HKEY_CURRENT_USER\wszSubkeyName->wszValueName=nValue
 *
 *	Returns:
 *	ERROR_SUCCESS(0) - success.
 *	E_CMN_REG_CREATE_FAIL - cannot open or create sub key.
 *	E_CMN_REG_WRITE_FAIL - cannot write data.
 *	E_CMN_INVALID_PARAMETER - if one or more pamaters are invalid (NULL).
 *  E_CMN_REG_INVALID_HKEY - if hKey is not HKEY_CURRENT_USER or HKEY_LOCAL_MACHINE
 *	ERROR_CALL_NOT_IMPLEMENTED - returns if user pass HKEY_LOCAL_MACHINE as hKey
 */
COMMON_API	DWORD	cmnWriteRegistryDwordW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, DWORD dwValue)
{
	HKEY hLocalKey = NULL;
	DWORD dwRet;
	/* check input */
	if(!wszSubkeyName || !wszValueName) {
		return E_CMN_INVALID_PARAMETER;
	}
	/* check hKey */
	if(!(hKey == HKEY_CURRENT_USER || hKey == HKEY_LOCAL_MACHINE)) {
		return E_CMN_REG_INVALID_HKEY;
	}
	/* check administrator's rights if user wants to write to local machine */
	if(hKey == HKEY_LOCAL_MACHINE) {
		#pragma COMMENT(TODO: Check admin to write to HKEY_LOCAL_MACHINE.)
		return ERROR_CALL_NOT_IMPLEMENTED;
	}
	/*	Create or open desired subkey for writting */		
	dwRet = RegCreateKeyExW(hKey, wszSubkeyName, 0, 
							NULL, REG_OPTION_NON_VOLATILE, 
							KEY_WRITE, NULL, &hLocalKey, NULL);
	if(ERROR_SUCCESS != dwRet) {
		return	E_CMN_REG_CREATE_FAIL;
	}	
	/* set value in subkey */
	if(hLocalKey) {
		dwRet = RegSetValueExW(hLocalKey, wszValueName, 0, REG_DWORD, (LPBYTE)&dwValue, sizeof(int));
		/* close registry */
		RegCloseKey(hLocalKey);
		hLocalKey = NULL;
		/* analyze return value */
		if(dwRet != ERROR_SUCCESS) {
			return	E_CMN_REG_WRITE_FAIL;	
		}	
	}
	return dwRet;	
}
/* ANSI version is same as UNICODE, only call ANSI API functions */
COMMON_API	DWORD	cmnWriteRegistryDwordA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, DWORD dwValue)
{
	HKEY hLocalKey = NULL;
	DWORD dwRet;
	/* check input */
	if(!szSubkeyName || !szValueName) {
		return E_CMN_INVALID_PARAMETER;
	}	
	/* check hKey */
	if(!(hKey == HKEY_CURRENT_USER || hKey == HKEY_LOCAL_MACHINE)) {
		return E_CMN_REG_INVALID_HKEY;
	}
	/* check administrator's rights if user wants to write to local machine */
	if(hKey == HKEY_LOCAL_MACHINE) {
		#pragma COMMENT(TODO: Check admin to write to HKEY_LOCAL_MACHINE.)
		return ERROR_CALL_NOT_IMPLEMENTED;
	}
	/*	Create or open desired subkey for writting */		
	dwRet = RegCreateKeyExA(hKey, szSubkeyName, 0, 
							NULL, REG_OPTION_NON_VOLATILE, 
							KEY_WRITE, NULL, &hLocalKey, NULL);
	if(ERROR_SUCCESS != dwRet) {
		return	E_CMN_REG_CREATE_FAIL;
	}	
	/* set value in subkey */
	if(hLocalKey) {
		dwRet = RegSetValueExA(hLocalKey, szValueName, 0, REG_DWORD, (LPBYTE)&dwValue, sizeof(DWORD));
		/* close registry */
		RegCloseKey(hLocalKey);
		hLocalKey = NULL;
		/* analyze return value */
		if(dwRet != ERROR_SUCCESS) {
			return	E_CMN_REG_WRITE_FAIL;	
		}	
	}
	return dwRet;	
}

/*
 *  Function: cmnReadRegistryIntW (UNICODE version)
 *	Reads integer value from registry
 * 
 *	hKey parameter specify root key:
 *	HKEY_CURRENT_USER - user specific data
 *	HKEY_LOCAL_MACHINE - computer data 	
 * 
 *	Data is readen from location specified by first parameter: 
 *	wszSubkeyName - string of subkey name that is created or opened if exist
 *
 *	Value name is specified by second parameter: (max. 256 wide-chars)
 *	wszValueName - string of value name
 *
 *	Data is readen from:
 *	HKEY_CURRENT_USER\wszSubkeyName->wszValueName=integer
 *
 *	If function success, returns desired integer value. In the last parameter is default value that
 *  is returned if some error occures:
 *	Returns default integer value if some error occures. Error code returns GetLastError() function.
 *   
 *  Possible error codes:
 *	ERROR_SUCCESS(0) - success.
 *	E_CMN_REG_INVALID_TYPE - type of target value is not DWORD
 *  E_CMN_REG_INVALID_HKEY - if hKey is not HKEY_CURRENT_USER or HKEY_LOCAL_MACHINE
 *	E_CMN_REG_OPEN_FAIL - cannot open sub key.
 *	E_CMN_REG_KEY_NOT_FOUND - if specified key not found in Windows registry
 *	E_CMN_REG_READ_FAIL - cannot read data. 
 *	E_CMN_INVALID_PARAMETER - if one or more pamaters are invalid (NULL). 
 */
COMMON_API	DWORD		cmnReadRegistryDwordW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, DWORD dwDefault)
{
	HKEY hLocalKey = NULL;
	DWORD dwRealSize, dwType, dwRet;
	int nRet;
	/* check input */
	if(!wszSubkeyName || !wszValueName) {
		SetLastError(E_CMN_INVALID_PARAMETER);
		return dwDefault;		
	}
	/* check hKey */
	if(!(hKey == HKEY_CURRENT_USER || hKey == HKEY_LOCAL_MACHINE)) {
		SetLastError(E_CMN_REG_INVALID_HKEY);
		return dwDefault;		
	}
	/*	Create or open desired subkey for writting */		
	dwRet = RegOpenKeyExW(hKey, wszSubkeyName, 0, KEY_READ, &hLocalKey);
	if(ERROR_FILE_NOT_FOUND == dwRet) {
		SetLastError(E_CMN_REG_KEY_NOT_FOUND);
		return dwDefault;		
	}
	if(ERROR_SUCCESS != dwRet) {
		SetLastError(E_CMN_REG_OPEN_FAIL);
		return dwDefault;				
	}	
	if(hLocalKey) {
		/* get size of real value in registry */
		dwRet = RegQueryValueExW(hLocalKey, wszValueName, NULL, &dwType, NULL, &dwRealSize);
		if(dwRet != ERROR_SUCCESS) {
			SetLastError(E_CMN_REG_READ_FAIL);
			return dwDefault;				
		}
		/* check type of data */
		if(dwType != REG_DWORD) {
			SetLastError(E_CMN_REG_INVALID_TYPE);
			return dwDefault;				
		}	
		/* check size of value */
		if(dwRealSize != sizeof(double)) {
			SetLastError(E_CMN_REG_INVALID_TYPE);
			return dwDefault;
		}
		/* get data from registry */
		dwRet = RegQueryValueExW(hLocalKey, wszValueName, NULL, &dwType, (LPBYTE)&nRet, &dwRealSize);		
		/* analyze return value */
		if(dwRet != ERROR_SUCCESS) {
			SetLastError(E_CMN_REG_READ_FAIL);
			return dwDefault;				
		}	
		/* close registry */
		RegCloseKey(hLocalKey);		
	}
	return nRet;	
}
/* ANSI version is same as UNICODE, only call ANSI API functions */
COMMON_API	DWORD		cmnReadRegistryDwordA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, DWORD dwDefault)
{
	HKEY hLocalKey = NULL;
	DWORD dwRealSize, dwType, dwRet, dwRes;

	dwRet = dwDefault;
	/* check input */
	if(!szSubkeyName || !szValueName) {
		SetLastError(E_CMN_INVALID_PARAMETER);
		return dwDefault;		
	}
	/* check hKey */
	if(!(hKey == HKEY_CURRENT_USER || hKey == HKEY_LOCAL_MACHINE)) {
		SetLastError(E_CMN_REG_INVALID_HKEY);
		return dwDefault;		
	}
	/*	Create or open desired subkey for writting */		
	dwRes = RegOpenKeyExA(hKey, szSubkeyName, 0, KEY_READ, &hLocalKey);
	if(ERROR_FILE_NOT_FOUND == dwRes) {
		SetLastError(E_CMN_REG_KEY_NOT_FOUND);
		return dwDefault;		
	}
	if(ERROR_SUCCESS != dwRet) {
		SetLastError(E_CMN_REG_OPEN_FAIL);
		return dwDefault;				
	}	
	if(hLocalKey) {
		/* get size of real value in registry */
		dwRes = RegQueryValueExA(hLocalKey, szValueName, NULL, &dwType, NULL, &dwRealSize);
		if(dwRes != ERROR_SUCCESS) {
			SetLastError(E_CMN_REG_READ_FAIL);
			return dwDefault;				
		}
		/* check type of data */
		if(dwType != REG_DWORD) {
			SetLastError(E_CMN_REG_INVALID_TYPE);
			return dwDefault;				
		}	
		/* check size of value */
		if(dwRealSize != sizeof(double)) {
			SetLastError(E_CMN_REG_INVALID_TYPE);
			return dwDefault;
		}
		/* get data from registry */
		dwRes = RegQueryValueEx(hLocalKey, szValueName, NULL, &dwType, (LPBYTE)&dwRet, &dwRealSize);		
		/* analyze return value */
		if(dwRes != ERROR_SUCCESS) {
			SetLastError(E_CMN_REG_READ_FAIL);
			return dwDefault;				
		}	
		/* close registry */
		RegCloseKey(hLocalKey);		
	}
	return dwRet;	
}

/*
 *	Use write regitry bin to store integer or float data.
 *	Functions for reading returns directly int or double values, use GetLastError() to get error code.
 *
 *	cmnReadRegistryInt() can return E_CMN_REG_INVALID_TYPE (via GetLastError()) if user wants to
 *  read non-int or non-double data.
 */
COMMON_API	DWORD	cmnWriteRegistryIntW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, int iValue)
{
	return cmnWriteRegistryBinW(hKey, wszSubkeyName, wszValueName, (LPBYTE)&iValue, sizeof(int));
}

COMMON_API	DWORD	cmnWriteRegistryIntA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, int iValue)
{
	return cmnWriteRegistryBinA(hKey, szSubkeyName, szValueName, (LPBYTE)&iValue, sizeof(int));
}

COMMON_API	int		cmnReadRegistryIntW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, int iDefault)
{
	DWORD dwSize = 0, dwRet;
    int iValue = iDefault;
	// Check size of written data
	dwRet = cmnReadRegistryBinW(hKey, wszSubkeyName, wszValueName, NULL, &dwSize);
	/* check size of value */
	if(dwSize != sizeof(int)) {
		SetLastError(E_CMN_REG_INVALID_TYPE);
		return iDefault;
	}
	// Read data from registry
	dwRet = cmnReadRegistryBinW(hKey, wszSubkeyName, wszValueName, (LPBYTE) &iValue, &dwSize);
	// Set last error
	SetLastError(dwRet);	
	// If some error occure, return default value
	if(dwRet != ERROR_SUCCESS) {
		return iDefault;
	}
	// Return integer value
	return iValue;
}

COMMON_API	int		cmnReadRegistryIntA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, int iDefault)
{
	DWORD dwSize = 0, dwRet;
    int iValue = iDefault;
	// Check size of written data
	dwRet = cmnReadRegistryBinA(hKey, szSubkeyName, szValueName, NULL, &dwSize);
	/* check size of value */
	if(dwSize != sizeof(int)) {
		SetLastError(E_CMN_REG_INVALID_TYPE);
		return iDefault;
	}
	// Read data from registry
	dwRet = cmnReadRegistryBinA(hKey, szSubkeyName, szValueName, (LPBYTE) &iValue, &dwSize);
	// Set last error
	SetLastError(dwRet);	
	// If some error occure, return default value
	if(dwRet != ERROR_SUCCESS) {
		return iDefault;
	}
	// Return integer value
	return iValue;
}

/*
 *	Use write regitry bin to store integer or float data.
 *	Functions for reading returns directly int or double values, use GetLastError() to get error code.
 *
 *	cmnReadRegistryInt() can return E_CMN_REG_INVALID_TYPE (via GetLastError()) if user wants to
 *  read non-int or non-double data.
 */
COMMON_API	DWORD	cmnWriteRegistryDoubleW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, double fValue)
{
	return cmnWriteRegistryBinW(hKey, wszSubkeyName, wszValueName, (LPBYTE)&fValue, sizeof(double));
}

COMMON_API	DWORD	cmnWriteRegistryDoubleA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, double fValue)
{
	return cmnWriteRegistryBinA(hKey, szSubkeyName, szValueName, (LPBYTE)&fValue, sizeof(double));	
}

COMMON_API	double	cmnReadRegistryDoubleW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, double fDefault)
{
	DWORD dwSize = 0, dwRet;
    double fValue = fDefault;
	// Check size of written data
	dwRet = cmnReadRegistryBinW(hKey, wszSubkeyName, wszValueName, NULL, &dwSize);
	/* check size of value */
	if(dwSize != sizeof(double)) {
		SetLastError(E_CMN_REG_INVALID_TYPE);
		return fDefault;
	}
	// Read data from registry
	dwRet = cmnReadRegistryBinW(hKey, wszSubkeyName, wszValueName, (LPBYTE) &fValue, &dwSize);
	// Set last error
	SetLastError(dwRet);	
	// If some error occure, return default value
	if(dwRet != ERROR_SUCCESS) {
		return fDefault;
	}
	// Return double value
	return fValue;
}

COMMON_API	double	cmnReadRegistryDoubleA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, double fDefault)
{
	DWORD dwSize = sizeof(double)+1, dwRet;
    double fValue = fDefault;
	// Check size of written data
	dwRet = cmnReadRegistryBinA(hKey, szSubkeyName, szValueName, NULL, &dwSize);
	/* check size of value */
	if(dwSize != sizeof(double)) {
		SetLastError(E_CMN_REG_INVALID_TYPE);
		return fDefault;
	}
	// Read data from registry
	dwRet = cmnReadRegistryBinA(hKey, szSubkeyName, szValueName, (LPBYTE) &fValue, &dwSize);
	// Set last error
	SetLastError(dwRet);	
	// If some error occure, return default value
	if(dwRet != ERROR_SUCCESS) {
		return fDefault;
	}
	// Return double value
	return fValue;
}

COMMON_API DWORD	cmnEncryptW(LPWSTR	lpwszDest, DWORD& dwSize, LPCWSTR	lpcwszSource)
{
	WCHAR	wChar;
	double test;
	if(lpwszDest)
	{
		DWORD dwSrcSize = (DWORD)wcslen(lpcwszSource);
		if(dwSize >= dwSrcSize)
		{
			ZeroMemory(lpwszDest, dwSize * sizeof(WCHAR));
			for(int i = 0; i < (int)dwSrcSize; i++)
			{
				wChar = lpcwszSource[i];
				test = pow((int)wChar, 3);
				lpwszDest[i] = (WCHAR)((int)test % 255);
			}
			dwSize = dwSrcSize;
			return S_OK;
		}
	}
	return S_FALSE;
} 

COMMON_API DWORD	cmnEncryptA(LPSTR	lpszDest, DWORD& dwSize, LPCSTR	lpcszSource)
{
	DWORD dwSrcSize;
	if(lpszDest)
	{
		// convert input string to UNICODE,
		// encrypt string as UNICODE
		WCHAR	* lpwszDest = new WCHAR[dwSize];
		dwSrcSize = (int)strlen(lpcszSource) + 1;
		// Allocate space for UNICODE src string
		WCHAR	* lpwszSrc = new WCHAR[dwSrcSize];
		cmnConvertANSIToUNICODE(lpwszSrc, &dwSrcSize, lpcszSource);		
		cmnEncryptW(lpwszDest, dwSize, lpwszSrc);
		// and convert it back to ANSI
		ZeroMemory(lpszDest, dwSize);dwSize++; 
		cmnConvertUNICODEToANSI(lpszDest, &dwSize, lpwszDest);	
		// Release all memory
		SAFE_DELETE_ARRAY(lpwszDest);
		SAFE_DELETE_ARRAY(lpwszSrc);
	}
	return S_FALSE;
} 

ULONG	g_crc32_table[256]; // Lookup table array
// Tuto funkci zavolejte pouze jednou k inicializaci CRC tabulky
void Init_CRC32Table()
{
   // Toto je oficiln polynom pouit v CRC-32
   // (nap. PKZip, WinZip a Ethernet)
   DWORD ulPolynomial = 0x04c11db7;

   for(int i = 0; i <= 0xFF; i++) {
      g_crc32_table[i]=Reflect(i, 8) << 24;
      for (int j = 0; j < 8; j++)
         g_crc32_table[i] = (g_crc32_table[i] << 1) ^ (g_crc32_table[i] & (1 << 31) ? ulPolynomial : 0);
     g_crc32_table[i] = Reflect(g_crc32_table[i], 32);
   }
}

// Pevrcen (Reflection) je vyadovno oficilnm CRC-32 standardem.
// Mete vytvet CRC i bez nj, ale nebude splovat standard.
DWORD Reflect(DWORD ref, char ch)
{
   DWORD value = 0;

   // Prohod bit 0 za bit 7
   // bit 1 za bit 6, atd.
   for(int i = 1; i < (ch + 1); i++) {
      if(ref & 1)
         value |= 1 << (ch - i);
      ref >>= 1;
   }
   return value;
}

COMMON_API DWORD cmnComputeCRC32(LPBYTE lpBuffer, DWORD dwSize)
{
   DWORD crc = 0xFFFFFFFF;
   int index; 
   DWORD i;
   for(i = 0; i < dwSize; i++) {
      index = (crc ^ *(lpBuffer+i)) & 0x000000FF;
      crc = ((crc >> 8) & 0x00FFFFFF) ^ g_crc32_table[index];
   }
   crc = crc ^ 0xFFFFFFFF;
   return crc;
}



