/*
 *	CmnMisc.h - Set/Get common functions, smart timer
 *
 *	Date:	about 2000
 *	Authors:	Jiri Formanek & Ondrej Burisin
 *	Update:	12.1. 2003
 *
 */

#ifndef _CMNMISC_H
	#define _CMNMISC_H

	/*
	 * Function: cmnGetTime()
	 * Smart timer
	 *
	 * Parameters: */
	#define		TIMER_START				0	// start the timer, returns 0.0f
	#define		TIMER_STOP				1	// stop the timer, returns 0.0f
	#define		TIMER_GETELAPSEDTIME	2	// return time between two calls with TIMER_GETELAPSEDTIME
	#define		TIMER_GETELAPSEDTIME1	7	// return time between two calls with TIMER_GETELAPSEDTIME1
	#define		TIMER_GETELAPSEDTIME2	8	// return time between two calls with TIMER_GETELAPSEDTIME2
	#define		TIMER_GETABSOLUTETIME	3	// return time from Windows start
	#define		TIMER_GETAPPTIME		4	// return time from application start (from TIMER_START to TIMER_STOP)
	#define		TIMER_RESET				5	// reset the timer, returns 0.0f
	#define		TIMER_ADVANCE			6	// Advance the timer by 1/10th second, returns 0.0f 
	/*	 
	 * Returns time in miliseconds if success or -1.0f if input parameter is invalid
	 * If you pass TIMER_START, TIMER_STOP, TIMER_RESET or TIMER_ADVANCE, function returns 0.0f
	 */
	COMMON_API	float	cmnGetTime(DWORD dwTimerCommand);
	
	/*
	 *  Function: cmnWriteRegistryBinW (UNICODE version)
	 *  Writes binary data to registry
	 *
	 *	hKey parameter specify root key:
	 *	HKEY_CURRENT_USER - user specific data
	 *	HKEY_LOCAL_MACHINE - computer data (administrator's rights needed)
	 *
	 *	Data is specified by last two parameters:
	 *	lpData - pointer to byte array
	 *	dwSize - size in bytes of the array (max. 32767 bytes)
	 *
	 *	Data is written to location specified by first parameter: 
	 *	wszSubkeyName - string of subkey name that is created or opened if exist
	 *
	 *	Value name is specified by second parameter: (max. 256 wide-chars)
	 *	wszValueName - string of value name
	 *
	 *	Data is written as:
	 *	HKEY_CURRENT_USER\wszSubkeyName->wszValueName=lpData
	 *
	 *	Returns:
	 *	ERROR_SUCCESS(0) - success.
	 *	E_CMN_REG_CREATE_FAIL - cannot open or create sub key.
	 *  E_CMN_REG_TOO_LARGE_VALUE - data is too large.
	 *	E_CMN_REG_WRITE_FAIL - cannot write data.
	 *	E_CMN_INVALID_PARAMETER - if one or more pamaters are invalid (NULL).
	 *  E_CMN_REG_INVALID_HKEY - if hKey is not HKEY_CURRENT_USER or HKEY_LOCAL_MACHINE
	 *	E_CMN_REG_INVALID_RIGHTS - returns if user pass HKEY_LOCAL_MACHINE as hKey and user is  not admin
	 *  
	 */
	COMMON_API	DWORD	cmnWriteRegistryBinW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, LPBYTE lpData, DWORD dwSize);	
	COMMON_API	DWORD	cmnWriteRegistryBinA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, LPBYTE lpData, DWORD dwSize);	
   /*
	*  Function: cmnReadRegistryBinW (UNICODE version)
	*	Reads binary data from registry
	* 
	*	hKey parameter specify root key:
	*	HKEY_CURRENT_USER - user specific data
	*	HKEY_LOCAL_MACHINE - computer data 	
	*
	*	Data is stored to third parameters:
	*	lpData - pointer to byte array, may be NULL, then function returns size of target 
	*			 value in the next parameter
	*	dwSize - size in bytes of the array (max. 32767 bytes)
	*
	*	Data is readen from location specified by first parameter: 
	*	wszSubkeyName - string of subkey name that is created or opened if exist
	*
	*	Value name is specified by second parameter: (max. 256 wide-chars)
	*	wszValueName - string of value name
	*
	*	Data is readen from:
	*	HKEY_CURRENT_USER\wszSubkeyName->wszValueName=lpData
	*
	*	Returns:
	*	ERROR_SUCCESS(0) - success.
	*	E_CMN_REG_INVALID_TYPE - type of target value is not binary
	*   E_CMN_REG_INVALID_HKEY - if hKey is not HKEY_CURRENT_USER or HKEY_LOCAL_MACHINE
	*	E_CMN_REG_INVALID_BUFFER - input buffer is too small.
	*	E_CMN_REG_OPEN_FAIL - cannot open sub key.
	*	E_CMN_REG_KEY_NOT_FOUND - if specified key not found in Windows registry
	*   E_CMN_REG_TOO_LARGE_VALUE - data is too large.
	*	E_CMN_REG_READ_FAIL - cannot read data. 
	*	E_CMN_INVALID_PARAMETER - if one or more pamaters are invalid (NULL).
	*  
	*/
	COMMON_API	DWORD	cmnReadRegistryBinW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, LPBYTE lpData, LPDWORD lpdwSize);
	COMMON_API	DWORD	cmnReadRegistryBinA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, LPBYTE lpData, LPDWORD lpdwSize);

   /*
	*   Function: cmnWriteRegistryBinW (UNICODE version)
	*   Writes string to registry
	*
	*	hKey parameter specify root key:
	*	HKEY_CURRENT_USER - user specific data
	*	HKEY_LOCAL_MACHINE - computer data (administrator's rights needed)
	*
	*	Data is specified by last parameter:
	*	wszInput - pointer to string (max. len 32767 wide-chars) 
	*
	*	Data is written to location specified by first parameter: 
	*	wszSubkeyName - string of subkey name that is created or opened if exist
	*
	*	Value name is specified by second parameter: (max. 256 wide-chars)
	*	wszValueName - string of value name
	*
	*	Data is written as:
	*	HKEY_CURRENT_USER\wszSubkeyName->wszValueName=wszInput
	*
	*	Returns:
	*	ERROR_SUCCESS(0) - success.
	*	E_CMN_REG_CREATE_FAIL - cannot open or create sub key.
	*   E_CMN_REG_TOO_LARGE_VALUE - data is too large.
	*	E_CMN_REG_WRITE_FAIL - cannot write data.
	*	E_CMN_INVALID_PARAMETER - if one or more pamaters are invalid (NULL).
	*   E_CMN_REG_INVALID_HKEY - if hKey is not HKEY_CURRENT_USER or HKEY_LOCAL_MACHINE
	*	ERROR_CALL_NOT_IMPLEMENTED - returns if user pass HKEY_LOCAL_MACHINE as hKey
	*  
	*/
	COMMON_API	DWORD	cmnWriteRegistryStrW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, LPCWSTR wszInput);
	COMMON_API	DWORD	cmnWriteRegistryStrA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, LPCSTR szInput);

   /*
	*   Function: cmnReadRegistryStrW (UNICODE version)
	*	Reads string from registry
	* 
	*	hKey parameter specify root key:
	*	HKEY_CURRENT_USER - user specific data
	*	HKEY_LOCAL_MACHINE - computer data 	
	*
	*	Data is stored to third parameters:
	*	wszOutput - pointer to string, may be NULL, then function returns size of target 
	*			    value in the next parameter
	*	lpdwSize - pointer to DWORD, size in bytes of the array (max. 32767 bytes)
	*
	*	Data is readen from location specified by first parameter: 
	*	wszSubkeyName - string of subkey name that is created or opened if exist
	*
	*	Value name is specified by second parameter: (max. 256 wide-chars)
	*	wszValueName - string of value name
	*
	*	Data is readen from:
	*	HKEY_CURRENT_USER\wszSubkeyName->wszValueName=string->wszOutput
	*
	*	Returns:
	*	ERROR_SUCCESS(0) - success.
	*	E_CMN_REG_INVALID_TYPE - type of target value is not binary
	*   E_CMN_REG_INVALID_HKEY - if hKey is not HKEY_CURRENT_USER or HKEY_LOCAL_MACHINE
	*	E_CMN_REG_INVALID_BUFFER - input buffer is too small.
	*	E_CMN_REG_OPEN_FAIL - cannot open sub key.
	*	E_CMN_REG_KEY_NOT_FOUND - if specified key not found in Windows registry
	*   E_CMN_REG_TOO_LARGE_VALUE - data is too large.
	*	E_CMN_REG_READ_FAIL - cannot read data. 
	*	E_CMN_INVALID_PARAMETER - if one or more pamaters are invalid (NULL).
	*  
	*/
	COMMON_API	DWORD	cmnReadRegistryStrW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, LPWSTR wszOutput, LPDWORD lpdwSize, LPCWSTR wszDefault = _S_EMPTYW);
	COMMON_API	DWORD	cmnReadRegistryStrA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, LPSTR szOutput, LPDWORD lpdwSize, LPCSTR szDefault = _S_EMPTYA);

   /*
	*   Function: cmnWriteRegistryIntW (UNICODE version)
	*   Writes DWORD value to registry
	*
	*	hKey parameter specify root key:
	*	HKEY_CURRENT_USER - user specific data
	*	HKEY_LOCAL_MACHINE - computer data (administrator's rights needed)
	*
	*	Data is specified by last parameter:
	*	dwValue - dword value to write
	*
	*	Data is written to location specified by first parameter: 
	*	wszSubkeyName - string of subkey name that is created or opened if exist
	*
	*	Value name is specified by second parameter: (max. 256 wide-chars)
	*	wszValueName - string of value name
	*
	*	Data is written as:
	*	HKEY_CURRENT_USER\wszSubkeyName->wszValueName=dwValue
	*
	*	Returns:
	*	ERROR_SUCCESS(0) - success.
	*	E_CMN_REG_CREATE_FAIL - cannot open or create sub key.
	*	E_CMN_REG_WRITE_FAIL - cannot write data.
	*	E_CMN_INVALID_PARAMETER - if one or more pamaters are invalid (NULL).
	*   E_CMN_REG_INVALID_HKEY - if hKey is not HKEY_CURRENT_USER or HKEY_LOCAL_MACHINE
	*	ERROR_CALL_NOT_IMPLEMENTED - returns if user pass HKEY_LOCAL_MACHINE as hKey
	*/
	COMMON_API	DWORD	cmnWriteRegistryDwordW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, DWORD dwValue);
	COMMON_API	DWORD	cmnWriteRegistryDwordA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, DWORD dwValue);
   
   /*
	*   Function: cmnReadRegistryIntW (UNICODE version)
	*	Reads DWORD value from registry
	* 
	*	hKey parameter specify root key:
	*	HKEY_CURRENT_USER - user specific data
	*	HKEY_LOCAL_MACHINE - computer data 	
	* 
	*	Data is readen from location specified by first parameter: 
	*	wszSubkeyName - string of subkey name that is created or opened if exist
	*
	*	Value name is specified by second parameter: (max. 256 wide-chars)
	*	wszValueName - string of value name
	*
	*	Data is readen from:
	*	HKEY_CURRENT_USER\wszSubkeyName->wszValueName=dword
	*
	*	If function success, returns desired dword value. In the last parameter is default value that
	*   is returned if some error occures:
	*	Returns default integer value if some error occures. Error code returns GetLastError() function.
	*   
	*   Possible error codes:
	*	ERROR_SUCCESS(0) - success.
	*	E_CMN_REG_INVALID_TYPE - type of target value is not DWORD
	*   E_CMN_REG_INVALID_HKEY - if hKey is not HKEY_CURRENT_USER or HKEY_LOCAL_MACHINE
	*	E_CMN_REG_OPEN_FAIL - cannot open sub key.
	*	E_CMN_REG_KEY_NOT_FOUND - if specified key not found in Windows registry	
	*	E_CMN_REG_WRITE_FAIL - cannot write data.
	*	E_CMN_INVALID_PARAMETER - if one or more pamaters are invalid (NULL). 
	*/
	COMMON_API	DWORD		cmnReadRegistryDwordW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, DWORD dwDefault = 0);	
	COMMON_API	DWORD		cmnReadRegistryDwordA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, DWORD dwDefault = 0);

   /*
    *	Use write regitry bin to store integer or float data.
	*	Functions for reading returns directly int or double values, use GetLastError() to get error code.
    *
	*   Possible error codes:
	*	ERROR_SUCCESS(0) - success.
	*	E_CMN_REG_INVALID_TYPE - type of target value is not DWORD
	*   E_CMN_REG_INVALID_HKEY - if hKey is not HKEY_CURRENT_USER or HKEY_LOCAL_MACHINE
	*	E_CMN_REG_OPEN_FAIL - cannot open sub key.
	*	E_CMN_REG_KEY_NOT_FOUND - if specified key not found in Windows registry
	*	E_CMN_REG_READ_FAIL - cannot read data. 
	*	E_CMN_REG_READ_FAIL - cannot read data. 
	*	E_CMN_INVALID_PARAMETER - if one or more pamaters are invalid (NULL). 
	*
	*/
	COMMON_API	DWORD	cmnWriteRegistryIntW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, int iValue);
	COMMON_API	DWORD	cmnWriteRegistryIntA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, int iValue);
	COMMON_API	int		cmnReadRegistryIntW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, int iDefault = 0);	
	COMMON_API	int		cmnReadRegistryIntA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, int iDefault = 0);
	COMMON_API	DWORD	cmnWriteRegistryDoubleW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, double fValue);
	COMMON_API	DWORD	cmnWriteRegistryDoubleA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, double fValue);
	COMMON_API	double	cmnReadRegistryDoubleW(HKEY hKey, LPCWSTR wszSubkeyName, LPCWSTR wszValueName, double fDefault = 0);	
	COMMON_API	double	cmnReadRegistryDoubleA(HKEY hKey, LPCSTR szSubkeyName, LPCSTR szValueName, double fDefault = 0);

#ifdef UNICODE
	#define	cmnWriteRegistryBin		cmnWriteRegistryBinW
	#define cmnReadRegistryBin		cmnReadRegistryBinW
	#define	cmnWriteRegistryStr		cmnWriteRegistryStrW
	#define	cmnReadRegistryStr		cmnReadRegistryStrW
	#define	cmnWriteRegistryInt		cmnWriteRegistryIntW
	#define	cmnReadRegistryInt		cmnReadRegistryIntW
	#define	cmnWriteRegistryDword	cmnWriteRegistryDwordW
	#define	cmnReadRegistryDword	cmnReadRegistryDwordW
	#define	cmnWriteRegistryInt		cmnWriteRegistryIntW
	#define	cmnReadRegistryInt		cmnReadRegistryIntW
	#define	cmnWriteRegistryDouble	cmnWriteRegistryDoubleW
	#define	cmnReadRegistryDouble	cmnReadRegistryDoubleW
#else 
	#define	cmnWriteRegistryBin		cmnWriteRegistryBinA
	#define cmnReadRegistryBin		cmnReadRegistryBinA
	#define	cmnWriteRegistryStr		cmnWriteRegistryStrA
	#define	cmnReadRegistryStr		cmnReadRegistryStrA
	#define	cmnWriteRegistryInt		cmnWriteRegistryIntA
	#define	cmnReadRegistryInt		cmnReadRegistryIntA
	#define	cmnWriteRegistryDword	cmnWriteRegistryDwordA
	#define	cmnReadRegistryDword	cmnReadRegistryDwordA
	#define	cmnWriteRegistryInt		cmnWriteRegistryIntA
	#define	cmnReadRegistryInt		cmnReadRegistryIntA
	#define	cmnWriteRegistryDouble	cmnWriteRegistryDoubleA
	#define	cmnReadRegistryDouble	cmnReadRegistryDoubleA
#endif	// UNICODE

	#define		MAX_KEY_ANSI_SIZE				512
	#define		MAX_KEY_UNICODE_SIZE			256
	#define		MAX_VALUE_UNICODE_SIZE			16838
	#define		MAX_VALUE_ANSI_SIZE				32767
	#define		MAX_RECOMMENDED_VALUE_SIZE		2048

	/* Obsolete function, use registry function above */
	/* default .ini file */
	#define		_S_SETUP_INI	"setup.ini"
	/*
	 *	Reading and writing string and integer values to .ini file 	 
	 *	.ini file is structured file:	 
	 *	[section]
	 *	key=value
	 *  .........
	 *	Filename of .ini file is set by cmnSetIniFileName()
	 *	If user does not call this function, default filename is used 
	 *	By this function is also defined string format - ANSI or UNICODE - in the file
	 */
	COMMON_API DWORD		cmnSetIniFileNameA(LPCSTR	szIniFileName);
	COMMON_API DWORD		cmnSetIniFileNameW(LPCWSTR	wszIniFileName);

	/* ANSI version */
	COMMON_API DWORD		cmnReadSetupStringA(LPCSTR szSection, LPCSTR szKey, LPSTR szOutput, DWORD nSize);
	COMMON_API int			cmnReadSetupIntA(LPCSTR szSection, LPCSTR szKey);
	COMMON_API DWORD		cmnWriteSetupStringA(LPCSTR szSection, LPCSTR szKey, LPSTR szInput);
	COMMON_API DWORD		cmnWriteSetupIntA(LPCSTR szSection, LPCSTR szKey, int iValue);
	/* UNICODE version */
	COMMON_API DWORD		cmnReadSetupStringW(LPCWSTR wszSection, LPCWSTR wszKey, LPWSTR wszOutput, DWORD nSize);
	COMMON_API int			cmnReadSetupIntW(LPCWSTR wszSection, LPCWSTR wszKey);
	COMMON_API DWORD		cmnWriteSetupStringW(LPCWSTR wszSection, LPCWSTR wszKey, LPWSTR wszInput);
	COMMON_API DWORD		cmnWriteSetupIntW(LPCWSTR wszSection, LPCWSTR wszKey, int iValue);

#ifdef UNICODE
	#define	cmnSetIniFileName	cmnSetIniFileNameW
	#define	cmnReadSetupString	cmnReadSetupStringW
	#define	cmnReadSetupInt		cmnReadSetupIntW
	#define	cmnWriteSetupString	cmnWriteSetupStringW
	#define	cmnWriteSetupInt	cmnWriteSetupIntW
#else
	#define	cmnSetIniFileName	cmnSetIniFileNameA
	#define	cmnReadSetupString	cmnReadSetupStringA
	#define	cmnReadSetupInt		cmnReadSetupIntA
	#define	cmnWriteSetupString	cmnWriteSetupStringA
	#define	cmnWriteSetupInt	cmnWriteSetupIntA
#endif // UNICODE
	//
	// simple encryption
	COMMON_API DWORD	cmnEncryptW(LPWSTR	lpwszDest, DWORD& dwSize, LPCWSTR	lpcwszSource);
	COMMON_API DWORD	cmnEncryptA(LPSTR	lpwszDest, DWORD& dwSize, LPCSTR	lpcszSource);
	//	
	void	Init_CRC32Table();
	DWORD	Reflect(DWORD ref, char ch);
	// 32-bit CRC computing
	COMMON_API DWORD	cmnComputeCRC32(LPBYTE lpBuffer, DWORD dwSize);

#ifdef UNICODE
	#define	cmnEncrypt	cmnEncryptW
#else
	#define	cmnEncrypt	cmnEncryptA
#endif // UNICODE

#endif // _CMNMISC_H

