/*
*	Adds network support to Common.dll 
*
*	Filename: CmnNet.h
*	Authors: Swap a MrAndrew
*	Date: 27.7. 03
*	Update: 27.7. 03
*
*/
#include <WinSock.h>

typedef const struct sockaddr* LPCSOCKADDR;
COMMON_API DWORD	InitNetwork();
COMMON_API DWORD	DestroyNetwork();

struct COMMON_API SockAddr : public sockaddr_in {
public:
	// constructors
	SockAddr()
		{ sin_family = AF_INET;
		  sin_port = 0;
		  sin_addr.s_addr = 0; } // Default
	SockAddr(const SOCKADDR& sa) { memcpy(this, &sa, sizeof(SOCKADDR)); }
	SockAddr(const SOCKADDR_IN& sin) { memcpy(this, &sin, sizeof(SOCKADDR_IN)); }
	SockAddr(const ULONG ulAddr, const USHORT ushPort = 0) // parms are host byte ordered
		{ sin_family = AF_INET;
		  sin_port = htons(ushPort);
	      sin_addr.s_addr = htonl(ulAddr); }
	SockAddr(const char* pchIP, const USHORT ushPort = 0) // dotted IP addr string
		{ sin_family = AF_INET;
		  sin_port = htons(ushPort);
		  sin_addr.s_addr = inet_addr(pchIP); } // already network byte ordered
	// Return the address in dotted-decimal format
	void DottedDecimal(LPSTR lpIP)
		{ if(lpIP) strcpy(lpIP, inet_ntoa(sin_addr)); } // constructs a new CString object
	// Get port and address (even though they're public)
	USHORT Port() const
		{ return ntohs(sin_port); }
	ULONG IPAddr() const
		{ return ntohl(sin_addr.s_addr); }
	// operators added for efficiency
	const SockAddr& operator=(const SOCKADDR& sa)
		{ memcpy(this, &sa, sizeof(SOCKADDR));
		  return *this; }
	const SockAddr& operator=(const SOCKADDR_IN& sin)
		{ memcpy(this, &sin, sizeof(SOCKADDR_IN));
		  return *this; }
	operator SOCKADDR()
		{ return *((LPSOCKADDR) this); }
	operator LPSOCKADDR()
		{ return (LPSOCKADDR) this; }
	operator LPSOCKADDR_IN()
		{ return (LPSOCKADDR_IN) this; }
	operator LPCSOCKADDR()
		{ return (LPCSOCKADDR) this; }
};

class IBlockingSocket
{	
public:
	// Creation of server or client
	virtual void Create(int nType = SOCK_STREAM) = 0;
	virtual void Create(SOCKET hSocket) = 0;
	// For servers
	virtual void Bind(LPCSOCKADDR psa) = 0;
	virtual void Listen() = 0;
	// Clean up
	virtual void Close() = 0;
	//
	// TCP protocol
	virtual void Connect(LPCSOCKADDR psa) = 0;
	virtual BOOL Accept(IBlockingSocket& s, LPSOCKADDR psa) = 0;
	virtual int  Send(const char* pch, const int nSize, const int nSecs) = 0;
	virtual int  Write(const char* pch, const int nSize, const int nSecs) = 0;
	virtual int  Receive(char* pch, const int nSize, const int nSecs) = 0;
	//
	// UDP protocol
	virtual int SendDatagram(const char* pch, const int nSize, LPCSOCKADDR psa, const int nSecs) = 0;
	virtual int ReceiveDatagram(char* pch, const int nSize, LPSOCKADDR psa, const int nSecs) = 0;
	//
	// Common network functions
	virtual void GetPeerAddr(LPSOCKADDR psa) = 0;
	virtual void GetSockAddr(LPSOCKADDR psa) = 0;
	virtual SockAddr GetHostByName(const char* pchName, 	const USHORT ushPort = 0) = 0;
	virtual const char* GetHostByAddr(LPCSOCKADDR psa) = 0; 
	// interface functions
	virtual HRESULT		AddRef() = 0;
	virtual HRESULT		Release() = 0;
	//
	// Operators
	virtual operator SOCKET() = 0;
};
// member functions truly block and must not be used in UI threads
// use this class as an alternative to the MFC CSocket class
class XBlockingSocket : public IBlockingSocket
{
	SOCKET	m_hSocket;
	DWORD	m_dwRef;
public:
	// Internal functions
	XBlockingSocket() { m_hSocket = NULL; m_dwRef = 0; }	
	void Cleanup();
public:		
	//
	// Creation of server or client
	virtual void Create(int nType = SOCK_STREAM) throw( XException );
	virtual void Create(SOCKET hSocket) throw( XException ) { m_hSocket = hSocket; }
	// For servers
	virtual void Bind(LPCSOCKADDR psa) throw( XException );
	virtual void Listen() throw( XException );
	// Clean up
	virtual void Close();
	//
	// TCP protocol
	virtual void Connect(LPCSOCKADDR psa) throw( XException );
	virtual BOOL Accept(IBlockingSocket& s, LPSOCKADDR psa) throw( XException );
	virtual int  Send(const char* pch, const int nSize, const int nSecs) throw( XException );
	virtual int  Write(const char* pch, const int nSize, const int nSecs) throw( XException );
	virtual int  Receive(char* pch, const int nSize, const int nSecs) throw( XException );
	//
	// UDP protocol
	virtual int SendDatagram(const char* pch, const int nSize, LPCSOCKADDR psa, const int nSecs) throw( XException );
	virtual int ReceiveDatagram(char* pch, const int nSize, LPSOCKADDR psa, const int nSecs) throw( XException );
	//
	// Common network functions
	virtual void GetPeerAddr(LPSOCKADDR psa) throw( XException );
	virtual void GetSockAddr(LPSOCKADDR psa) throw( XException );
	virtual SockAddr GetHostByName(const char* pchName, const USHORT ushPort = 0) throw( XException );
	virtual const char* GetHostByAddr(LPCSOCKADDR psa) throw( XException ); 
	//
	// Operators
	virtual operator SOCKET()	{ return m_hSocket; }
	//
	// Interface functions
	virtual HRESULT		AddRef();
	virtual HRESULT		Release();
};