#include "StdAfx.h"

#define COMMON_API __declspec(dllexport)

#include "CmnLog.h"
#include "CmnStg.h"
#include "CmnStr.h"
#include "CmnZip.h"
#include "CmnFile.h"

XStorageFile::XStorageFile() 
{
	m_pRoot = NULL;
	m_dwDepth = 0;
	m_pFunc = NULL;
	m_dwRef = 0;
	m_dwOpenFlags = 0;
}

XStorageFile::~XStorageFile()
{

}

// interface functions
HRESULT		XStorageFile::AddRef()
{	
	return m_dwRef++;
}

HRESULT		XStorageFile::Release()
{
	if(m_dwRef > 0)
	{
		m_dwRef--;
	}
	if(m_dwRef == 0)
	{
		delete this;
	}
	return m_dwRef;
}

// create, open and close storage file
HRESULT		XStorageFile::CreateStorage(LPCSTR szExtPath, IStorage **lplpRoot)
{
	HRESULT dwReturn;
	DWORD dwSize;	
	// Check input parameters
	if(!szExtPath) 
		return 1;	
	m_dwOpenFlags = STGM_READWRITE|STGM_SHARE_EXCLUSIVE;
	// Try to create new data file
	cmnConvertANSIToUNICODE(NULL, &dwSize, szExtPath);
	if(dwSize > 0) 
	{
		wchar_t *wszExtPath = new wchar_t[dwSize];
		if(wszExtPath) 
		{
			// convert to UNICODE
			cmnConvertANSIToUNICODE(wszExtPath, &dwSize, szExtPath);
			// try to open storage
			dwReturn = ::StgCreateDocfile(wszExtPath, 
						m_dwOpenFlags|STGM_CREATE, 0, &m_pRoot);
			switch(dwReturn) {
			case S_OK:		
				TRACE("New root has been created.");
				dwReturn = ERROR_SUCCESS;
				if(lplpRoot) 
				{
					*lplpRoot = m_pRoot;
				}
				m_dwDepth = 0;
				break;
			default:
				TRACE("Cannot create root storage.");
				dwReturn = ERROR_CANNOTCREATEROOT;
				break;
			}
			SAFE_DELETE_ARRAY(wszExtPath);
		}				
	}
	return dwReturn;
}

HRESULT		XStorageFile::OpenStorage(LPCSTR szExtPath, DWORD dwOpenFlags, IStorage **lplpRoot)
{
	HRESULT	dwReturn;	
	DWORD dwSize;
	// convert input string to UNICODE
	cmnConvertANSIToUNICODE(NULL, &dwSize, szExtPath);
	if(dwSize > 0) 
	{
		wchar_t *wszExtPath = new wchar_t[dwSize];
		if(wszExtPath) 
		{
			cmnConvertANSIToUNICODE(wszExtPath, &dwSize, szExtPath);
			//
			// test if specified file exist or if is storage file
			dwReturn = ::StgIsStorageFile(wszExtPath);
			switch(dwReturn) {
				// file exists, but is not storage
			case S_FALSE:
				TRACE("Specified file is not storage.");
				dwReturn = ERROR_FILEISNOTSTG;
				break;
				// file doen't exist
			case STG_E_FILENOTFOUND:
				TRACE("I cannot find specified file.");
				dwReturn = ERROR_FILENOTFOUND;
				break;
			case S_OK:
				//
				m_dwOpenFlags = (dwOpenFlags |= STGM_SHARE_EXCLUSIVE);
				// try to open true storage file and save storage pointer
				dwReturn = ::StgOpenStorage(wszExtPath, NULL, 
											m_dwOpenFlags, NULL, 0, &m_pRoot);
				//
				//m_pstgRoot...storage pointer (root)
				switch(dwReturn) {
					//
					// all is OK
				case S_OK:
					TRACE("Root storage has been opened.");					
					//
					// save local copy of root
					if(lplpRoot)
					{
						*lplpRoot = m_pRoot;		
					}
					//
					// Clear array of items
					dwReturn = ERROR_SUCCESS;
					break;
					//
					// some error occured
				default:
					ERR("Some error is occured when I tried to open storage. Error code ", dwReturn);
					dwReturn = ERROR_INVALIDSTORAGE;
					break;
				}
				break;
			}
			SAFE_DELETE_ARRAY(wszExtPath);
		}
	}	
	return dwReturn;
}

HRESULT		XStorageFile::CloseStorage()
{
	SAFE_RELEASE(m_pRoot);
	return 0;
}

// add or remove dir from straoge
HRESULT		XStorageFile::CreateDir(LPCSTR szName, IStorage *lpParent, IStorage **lplpDir)
{
	return 0;
}
HRESULT		XStorageFile::DeleteDir(LPCSTR szName, IStorage *lpParent)
{
	return 0;
}

// add or remove file from storage
HRESULT		XStorageFile::CreateFile(LPCSTR szName, IStorage *lpParent, IStream **lplpFile)
{
	return 0;
}

HRESULT		XStorageFile::DeleteFile(LPCSTR szName, IStorage *lpParent)
{
	return 0;
}

// Go through whole root and call callback for each item 
// Save shortcut to the all files for optimalization
HRESULT		XStorageFile::ExploreRoot(FILEPROC pFunc)
{
	if(pFunc && m_pRoot) 
	{	
		// Save callback function
		m_pFunc = pFunc;
		// Call first level of recursion
		rcExploreRoot(m_pRoot);			
		return ERROR_SUCCESS;
	}
	return 1;
}

void		XStorageFile::rcExploreRoot(IStorage * pstgParent)
{
	IEnumSTATSTG* pEnum;
	IMalloc* pMalloc;
	STATSTG Stat;
	IStorage *pSubStg;
	IStream *pFile;
	DWORD dwSize;
	// Initialize unknown :-)
	::CoGetMalloc(MEMCTX_TASK, &pMalloc);
	//
	// Init Enumerator	
	if(pstgParent->EnumElements(0, NULL, 0, &pEnum) == S_OK) 
	{
		//
		// Go! (or Twist?)
		while(pEnum->Next(1, &Stat, NULL) == NOERROR) 
		{
			//
			// convert string to ANSI
			cmnConvertUNICODEToANSI(NULL, &dwSize, Stat.pwcsName);
			if(dwSize > 0) 
			{
				char *szName = new char[dwSize];
				if(szName) 
				{
					//
					// Is item storage?
					if(Stat.type == STGTY_STORAGE) 
					{ // Yes
						//
						// Open Storage and ...
						if(ERROR_SUCCESS == OpenStorage(Stat.pwcsName, pstgParent, &pSubStg)) 
						{ 				
							// Call call back func if is not set silent mode				
							m_pFunc(szName, IT_DIRECTORY, (void*)pSubStg, m_dwDepth);
						}								
						//
						// For storages call this method again :o)))	
						m_dwDepth++;
						rcExploreRoot(pSubStg);				
						m_dwDepth--;										
					}
					if(Stat.type == STGTY_STREAM) 
					{ // No
						if(ERROR_SUCCESS == OpenStream(Stat.pwcsName, pstgParent, &pFile))
						{
							//
							// Call call back func if is not set silent mode
							m_pFunc(szName, IT_FILE, (void*)pFile, m_dwDepth);						
						}
					}	
					SAFE_DELETE_ARRAY(szName);
				}		
			}
			//
			// Free resources
			pMalloc->Free(Stat.pwcsName);
		}
		// release all
		pMalloc->Release();
		pEnum->Release();
	}
}

// gets data from specified file, compress it nad write to the stream
HRESULT		XStorageFile::SaveFile(IStream* lpFile, LPCSTR szExtPath)
{
	//
	// Definitions
	HANDLE hExtFile;
	DWORD dwUCSize, dwCSize, dwWritten, dwRet, dwReadBytes;
	LPBYTE lpExtFile, lpCData;
	ULARGE_INTEGER uiSize;
	//
	// Open external file to save
	hExtFile = cmnOpenExistingFile(szExtPath, GENERIC_READ, FILE_SHARE_READ);
	if(hExtFile != INVALID_HANDLE_VALUE) 
	{	
		// Get size of opened external file
		dwUCSize = GetFileSize(hExtFile, NULL);
		// Get external file data
		lpExtFile = new BYTE[dwUCSize];
		if(!ReadFile(hExtFile, lpExtFile, dwUCSize, &dwReadBytes, NULL))
		{	
			XException	exp("Cannot read the whole file.");
			THROW(exp);						
		}
		// Close External file
		cmnCloseFile(hExtFile);
		// Create buffer to compress data
		dwCSize = dwUCSize + DWORD(dwUCSize * 0.1) + 12;
		lpCData = new BYTE[dwCSize];
		// Compress data
		if(Z_OK == (dwRet = cmnCompressBuffer(lpCData, &dwCSize, lpExtFile, dwUCSize)))
		{
			uiSize.QuadPart = dwCSize + 2 * sizeof(DWORD);
			// Set size of stream
			lpFile->SetSize(uiSize);
			// Copy size of compress data and uncompress
			lpFile->Write(&dwCSize, sizeof(DWORD), &dwWritten);
			if(dwWritten != sizeof(DWORD))
			{				
				XException	exp("Cannot write compressed size to file.");
				THROW(exp);	
			}
			lpFile->Write(&dwUCSize, sizeof(DWORD), &dwWritten);
			if(dwWritten != sizeof(DWORD)) 
			{				
				XException	exp("Cannot write uncompressed size to file.");
				THROW(exp);	
			}
			// Copy compress data to the stream
			lpFile->Write(lpCData, dwCSize, &dwWritten);
			if(dwWritten != sizeof(DWORD)) 
			{
				XException	exp("Cannot write DATA to file.");
				THROW(exp);					
			}
			dwRet = ERROR_SUCCESS;									
		}
		//
		// Releases temporary buffers
		SAFE_DELETE_ARRAY(lpExtFile);				
		SAFE_DELETE_ARRAY(lpCData);
	}			
	return dwRet;
}

// defragmentation
HRESULT		XStorageFile::Defragment()
{
	IStorage* pNewRoot;
	HRESULT dwReturn;
	DWORD dwSize;
	wchar_t	wszTempPath[MAX_PATH];
	char	szTempPath[MAX_PATH];
	// convert path to UNICODE
	cmnConvertANSIToUNICODE(NULL, &dwSize, m_szExtPath);
	if(dwSize > 0) 
	{
		wchar_t *wszExtPath = new wchar_t[dwSize];
		if(wszExtPath) 
		{
			// convert to UNICODE
			cmnConvertANSIToUNICODE(wszExtPath, &dwSize, m_szExtPath);
			//
			// Build path to the new root
			cmnCutFileW(wszTempPath, wszExtPath);
			cmnCutFile(szTempPath, m_szExtPath);
			wcscat(wszTempPath,  _S_BACKSLASHW);
			strcat(szTempPath,  _S_BACKSLASH);
			wcscat(wszTempPath, _S_TEMPORARYFILEW);
			strcat(szTempPath, _S_TEMPORARYFILE);
			//
			// Try to create new data file with temp name
			dwReturn = ::StgCreateDocfile(wszTempPath, 
							STGM_READWRITE | STGM_SHARE_EXCLUSIVE | STGM_CREATE,
							0, &pNewRoot);	
			if(dwReturn == S_OK) {	
				//
				// Make a copy of old data file
				m_pRoot->CopyTo(NULL, NULL, NULL, pNewRoot);
				//
				// Close old data file
				m_pRoot->Revert();
				SAFE_RELEASE(m_pRoot);
				//
				// Delete old data file
				::DeleteFile(m_szExtPath);
				//
				// Close new data file
				pNewRoot->Commit(STGC_OVERWRITE);
				SAFE_RELEASE(pNewRoot);
				//
				// Rename new data file		
				if(rename(szTempPath, m_szExtPath)) {
					TRACE("Error occured while renaming file");
				}		
				// reopen renamed storage
				dwReturn = OpenStorage(m_szExtPath, m_dwOpenFlags, NULL);				
			}	
			else {
				dwReturn = ERROR_CANNOTCREATETEMPSTG;								
			}	
			SAFE_DELETE_ARRAY(wszExtPath);
		}
	}
	return dwReturn;
}

// load file from storage
HRESULT		XStorageFile::GetFile(LPCSTR szFullIntPath, LPBYTE* lplpData, DWORD &dwSize)
{
	DWORD dwStringSize;
	cmnConvertANSIToUNICODE(NULL, &dwStringSize, szFullIntPath);
	if(dwSize > 0) 
	{
		wchar_t *wszFullIntPath = new wchar_t[dwStringSize];
		if(wszFullIntPath) 
		{
			// convert to UNICODE
			cmnConvertANSIToUNICODE(wszFullIntPath, &dwSize, szFullIntPath);
			return rcGetFile(wszFullIntPath, lplpData, dwSize, m_pRoot);	
			SAFE_DELETE_ARRAY(wszFullIntPath);
		}		
	}
	return -1;
}

// recursive functions
HRESULT		XStorageFile::rcGetFile(LPWSTR wszFullIntPath, LPBYTE *lplpData, DWORD &dwSize, IStorage *pParent)
{	
	DWORD dwRet;
	IStorage * pDir;
	IStream  * pFile;	
	wchar_t wszTemp[MAX_PATH];	
	if(pParent) 
	{
		// cut first backslash
		wszFullIntPath++;
		// copy to temporary string
		wcscpy(wszTemp, wszFullIntPath);
		// find first backslash 
		wszFullIntPath = wcschr(wszFullIntPath, _S_BACKSLASH_CHARW);
		if(wszFullIntPath) 
		{
			int i = 0;
			while(wszTemp[i] != _S_BACKSLASH_CHARW) {
				i++;
			}
			// and terminate temp char after that
			wszTemp[i] = _S_NULL_CHARW;		
			// Try to open storage
			if(ERROR_SUCCESS == (dwRet = OpenStorage(wszTemp, pParent, &pDir))) {
				// Call recursive func. again
				dwRet = rcGetFile(wszFullIntPath, lplpData, dwSize, pDir);
				// Close open directory
				pDir->Revert();
				SAFE_RELEASE(pDir);
			}	
		}
		else 
		{
			// Else open file, unpack it and return buffer
			DWORD   dwCSize, dwUCSize, dwReaden;
			LPBYTE	lpCData;
			//
			// Open file
			if(ERROR_SUCCESS == (dwRet = OpenStream(wszTemp, pParent, &pFile))) {
				//
				// Read compressed and original size of file
				pFile->Read(&dwCSize, sizeof(DWORD), &dwReaden);
				if(dwReaden != sizeof(DWORD))
				{
					XException exp("Cannot read compressed size to file.");
					THROW(exp); 	
				}
				pFile->Read(&dwUCSize, sizeof(DWORD), &dwReaden);
				if(dwReaden != sizeof(DWORD))
				{
					XException exp("Cannot read uncompressed size to file.");
					THROW(exp); 					 	
				}
				//
				//	Allocate buffer for compressed stream
				lpCData = new BYTE[dwCSize];
				if(lpCData) 
				{
					(*lplpData) = new BYTE[dwUCSize];
					if((*lplpData)) 
					{
						//
						// Read buffer to memory
						dwRet = pFile->Read(lpCData, dwCSize, &dwReaden);
						if(dwReaden != dwCSize)
						{
							XException exp("Cannot read DATA to file.");
							THROW(exp); 							 			
						}
						//
						// Unpack direct to user buffer
						if(Z_OK == (dwRet = cmnUncompressBuffer((*lplpData), &dwUCSize, lpCData, dwCSize))) {
							//
							// Return size of file to user
							dwSize = dwUCSize;
						}						
					}
					SAFE_DELETE_ARRAY(lpCData);
				}
				pFile->Revert();
				SAFE_RELEASE(pFile);			
			}				
		}			
	}
	return dwRet;
}


// Internal func for fast open of directory and files
HRESULT		XStorageFile::OpenStorage(LPWSTR wszStorageName, IStorage* pParentStorage, IStorage**pStorage)
{
	HRESULT dwReturn = -1;
	if(pParentStorage) {
		//
		// Try to open Storage - directory	
		dwReturn = pParentStorage->OpenStorage( wszStorageName, NULL, 
												m_dwOpenFlags, 
												NULL, 0, pStorage);
		switch(dwReturn) 
		{
		case S_OK:
			dwReturn = ERROR_SUCCESS;
			break;	
		case STG_E_ACCESSDENIED:
			TRACE("OpenStorage '%s': STG_E_ACCESSDENIED", wszStorageName);
			break;
		case STG_E_INVALIDFLAG:
			TRACE("OpenStorage '%s': STG_E_INVALIDFLA", wszStorageName);
			break;
		case STG_E_FILENOTFOUND:
			TRACE("OpenStorage '%s': STG_E_FILENOTFOUND ", wszStorageName);
			break;
		case STG_E_INVALIDPARAMETER:
			TRACE("OpenStorage '%s': STG_E_INVALIDPARAMETER ", wszStorageName);
			break;
		case STG_E_INVALIDFUNCTION:
			TRACE("OpenStorage '%s': STG_E_INVALIDFUNCTION ", wszStorageName);
			break;
		case STG_E_INVALIDPOINTER :
			TRACE("OpenStorage '%s': STG_E_INVALIDPOINTER  ", wszStorageName);
			break;
		}	
	}
	return dwReturn;
}

HRESULT		XStorageFile::OpenStream(LPWSTR wszStreamName, IStorage* pParentStorage, IStream**pStream)
{
	HRESULT dwReturn = -1;
	if(pParentStorage) {
		//
		// Try to open Stream - file
		dwReturn = pParentStorage->OpenStream(	wszStreamName, NULL, 
												m_dwOpenFlags, 
												0, pStream);
		switch(dwReturn) 
		{
		case S_OK:
			dwReturn = ERROR_SUCCESS;
			break;	
		case STG_E_ACCESSDENIED:
			TRACE("OpenStream '%s': STG_E_ACCESSDENIED", wszStreamName);
			break;
		case STG_E_INVALIDFLAG:
			TRACE("OpenStream '%s': STG_E_INVALIDFLA", wszStreamName);
			break;
		case STG_E_FILENOTFOUND:
			TRACE("OpenStream '%s': STG_E_FILENOTFOUND ", wszStreamName);
			break;
		case STG_E_INVALIDPARAMETER:
			TRACE("OpenStream %s: STG_E_INVALIDPARAMETER ", wszStreamName);
			break;
		case STG_E_INVALIDFUNCTION:
			TRACE("OpenStream '%s': STG_E_INVALIDFUNCTION ", wszStreamName);
			break;
		case STG_E_INVALIDPOINTER :
			TRACE("OpenStream '%s': STG_E_INVALIDPOINTER  ", wszStreamName);
			break;
		}	
	}
	return dwReturn;
}


