#pragma once


//
// Some user defined returns values
#define		ERROR_CANNOTCREATETEMPSTG		20026L
#define		ERROR_FILENOTFOUND				20035L
#define		ERROR_FILEISNOTSTG				20036L
#define		ERROR_INVALIDSTORAGE			20006L
#define		ERROR_CANNOTCREATEROOT			20042L

//
// For storage class
// Item types, root-main storage, directory-storage, file-stream, none-error
#define		IT_ROOT						0
#define		IT_DIRECTORY				1
#define		IT_FILE						2
#define		IT_NONE						3
#define		_S_TEMPORARYFILEW			L"oh.x"
#define		_S_TEMPORARYFILE			"oh.x"




#include <ole2.h>
//#pragma comment( lib, "ole32.lib" )
typedef void (CALLBACK* FILEPROC)(LPTSTR, UINT, LPVOID, DWORD);
// interface for storage operations
class IStorageFile
{
public:
	// create, open and close storage file
	virtual HRESULT		CreateStorage(LPCSTR szExtPath, IStorage **lplpRoot) = 0;
	virtual HRESULT		OpenStorage(LPCSTR szExtPath, DWORD dwOpenFlags, IStorage **lplpRoot) = 0;
	virtual HRESULT		CloseStorage() = 0;
	// add or remove dir from straoge
	virtual HRESULT		CreateDir(LPCSTR szName, IStorage *lpParent, IStorage **lplpDir) = 0;
	virtual HRESULT		DeleteDir(LPCSTR szName, IStorage *lpParent) = 0;
	// add or remove file from storage
	virtual HRESULT		CreateFile(LPCSTR szName, IStorage *lpParent, IStream **lplpFile) = 0;
	virtual HRESULT		DeleteFile(LPCSTR szName, IStorage *lpParent) = 0;
	// Go through whole root and call callback for each item 
	// Save shortcut to the all files for optimalization
	virtual HRESULT		ExploreRoot(FILEPROC pFunc) = 0;
	// extract file from storage
	virtual HRESULT		SaveFile(IStream* lpFile, LPCSTR szExtPath) = 0;
	// defragmentation
	virtual HRESULT		Defragment() = 0;
	// load file from storage
	virtual HRESULT		GetFile(LPCSTR szFullIntPath, LPBYTE* lplpData, DWORD &dwSize) = 0;
	// interface functions
	virtual HRESULT		AddRef() = 0;
	virtual HRESULT		Release() = 0;
};

class XStorageFile : public IStorageFile
{
	// references
	DWORD		m_dwRef;
	// pointer to callback explore function
	FILEPROC	m_pFunc;
	// depth of tree
	DWORD		m_dwDepth;
	// open flags
	DWORD		m_dwOpenFlags;
	// pointer to the ROOT
	IStorage	*m_pRoot;
	// path to storage file
	char		m_szExtPath[MAX_PATH];
// interface implementation
public:
	// create, open and close storage file
	virtual HRESULT		CreateStorage(LPCSTR szExtPath, IStorage **lplpRoot);
	virtual HRESULT		OpenStorage(LPCSTR szExtPath, DWORD dwOpenFlags, IStorage **lplpRoot);
	virtual HRESULT		CloseStorage();
	// add or remove dir from straoge
	virtual HRESULT		CreateDir(LPCSTR szName, IStorage *lpParent, IStorage **lplpDir);
	virtual HRESULT		DeleteDir(LPCSTR szName, IStorage *lpParent);
	// add or remove file from storage
	virtual HRESULT		CreateFile(LPCSTR szName, IStorage *lpParent, IStream **lplpFile);
	virtual HRESULT		DeleteFile(LPCSTR szName, IStorage *lpParent);
	// Go through whole root and call callback for each item 
	// Save shortcut to the all files for optimalization
	virtual HRESULT		ExploreRoot(FILEPROC pFunc);
	// extract file from storage
	virtual HRESULT		SaveFile(IStream* lpFile, LPCSTR szExtPath) throw( XException );
	// defragmentation
	virtual HRESULT		Defragment();
	// load file from storage
	virtual HRESULT		GetFile(LPCSTR szFullIntPath, LPBYTE* lplpData, DWORD &dwSize);
	// interface functions
	virtual HRESULT		AddRef();
	virtual HRESULT		Release();
// internal functions
private:
	// recursive functions
	HRESULT		rcGetFile(LPWSTR wszFullIntPath, LPBYTE *lplpData, DWORD &dwSize, IStorage *pParent) throw( XException );
	void		rcExploreRoot(IStorage * pstgParent);
	// Internal func for fast open of directory and files
	HRESULT		OpenStorage(LPWSTR wszStorageName, IStorage* pParentStorage, IStorage**pStorage);
	HRESULT		OpenStream(LPWSTR wszStreamName, IStorage* pParentStorage, IStream**pStream);

public:
	XStorageFile();
	~XStorageFile();
};  


