// CmnStr.h - string functions (creating paths, string ops ...)
// Date :	25.12.2002
// Update :	31.12.2002
// Author :	Ondrej Burisin, Jiri Formanek
// Updater: Ondrej Burisin

#include "stdafx.h"

#include <stdlib.h>
#include <stdio.h>
#include <conio.h>

#define COMMON_API __declspec(dllexport)
#include "CmnLog.h"
#include "CmnStr.h"



// Makes the full path from path and filename and returns the path in parameter _lpszDest. _lpcszPath and _lpcszFile
// must be zero terminated strings. _lpszDest must be allocated by the caller. In case needed it adds backslashes.
// Doesn't check if the filename contains a backslash as it's initial character. The string is ended by '\0' char 
// and if _lpSize is passed by caller, then it contains the size of _lpszDest including the terminating '\0'. If the
// parameter _lpszDest is NULL, then the function just returns the size of the string to be passed as _lpszDest. The
// size of the string is in characters (NOT bytes!!!), so for allocating memory use buffer = new TCHAR[returnedsize];
// And in every byte based function add *sizeof(TCHAR) for full support
COMMON_API LPWSTR cmnCreatePathW(LPWSTR _lpszDest, LPCWSTR _lpcszPath, LPCWSTR _lpcszFile, LPDWORD _lpdwSize)
{
	LPWSTR lpszRet = NULL;

	if(_lpcszPath && _lpcszFile)
	{
		DWORD dwPathLen = (DWORD)wcslen(_lpcszPath);	// Without the terminating zero
		DWORD dwFileLen = (DWORD)wcslen(_lpcszFile);	// Without the terminating zero
		DWORD dwFullPathLen = dwPathLen + dwFileLen + 1;	// + 1 is for the null char
		
		// Now check if we need to add backslashes in between the two strings
		BOOL bAddSlash = FALSE;
		
		if(!(L'\\' == *(_lpcszPath + dwPathLen - 1)))
		{
			bAddSlash = TRUE;
			dwFullPathLen += 1;	// We need to add a backslash in between the two strings
		}

		if(_lpszDest)
		{
			wcscpy(_lpszDest, _lpcszPath);	// Copies also the terminating zero char, but it will be overwritten

			if(bAddSlash)
			{
				*(_lpszDest + dwPathLen) = L'\\';	// Add the slash
			}

			// Copy the filename
			wcscpy(_lpszDest + dwPathLen + ((bAddSlash) ? 1 : 0), _lpcszFile);
			// We don't need to copy the terminating zero char, because _tcscpy copies the string with the terminating
			// zero char
			
			lpszRet = _lpszDest;	// Return the pointer to the full path
		}
		else
		{
			*_lpdwSize = dwFullPathLen;
		}
	}
	
	return lpszRet;
}

// Convert the strings using the same algorithm rewritten for non-UNICODE strings, because the allocating of the mem-
// ory would probably take longer than the function body itself
COMMON_API LPSTR cmnCreatePathA(LPSTR _lpszDest, LPCSTR _lpcszPath, LPCSTR _lpcszFile, LPDWORD _lpdwSize)
{
	LPSTR lpszRet = NULL;

	if(_lpcszPath && _lpcszFile)
	{
		DWORD dwPathLen = (DWORD)strlen(_lpcszPath);	// Without the terminating zero
		DWORD dwFileLen = (DWORD)strlen(_lpcszFile);	// Without the terminating zero
		DWORD dwFullPathLen = dwPathLen + dwFileLen + 1;	// + 1 is for the null char
		
		// Now check if we need to add backslashes in between the two strings
		BOOL bAddSlash = FALSE;
		
		if(!('\\' == *(_lpcszPath + dwPathLen - 1)))
		{
			bAddSlash = TRUE;
			dwFullPathLen += 1;	// We need to add a backslash in between the two strings
		}

		if(_lpszDest)
		{
			strcpy(_lpszDest, _lpcszPath);	// Copies also the terminating zero char, but it will be overwritten

			if(bAddSlash)
			{
				*(_lpszDest + dwPathLen) = '\\';	// Add the slash
			}

			// Copy the filename
			strcpy(_lpszDest + dwPathLen + ((bAddSlash) ? 1 : 0), _lpcszFile);
			// We don't need to copy the terminating zero char, because _tcscpy copies the string with the terminating
			// zero char
			
			lpszRet = _lpszDest;	// Return the pointer to the full path
		}
		else
		{
			*_lpdwSize = dwFullPathLen;
		}
	}
	
	return lpszRet;
}

/* 
 *	Adds _lpcszFile to program path
 *	Uses GetModuleFileName() func to get program path
 *
 *	Parameters:		1. Output buffer
 *					2. Filename to add *					
 *
 *	Return pointer to output buffer (same as _lpszDest) in case of success or NULL
 *	Func may throw string exception!
 */
COMMON_API LPWSTR	cmnGetDataFilePathW(LPWSTR _lpwszDest, LPCWSTR _lpcwszFile)
{
	LPWSTR lpwszRet = NULL;
	//
	// Check input parameters
	if(_lpwszDest && _lpcwszFile) {
		//
		wchar_t wszModuleFileName[MAX_PATH];
		// Get program path
		if(!GetModuleFileNameW(GetModuleHandle(NULL), wszModuleFileName, MAX_PATH)) {
			XException exp("Cannot get module filename.",  GetLastError());
			THROW(exp);			
		}
		//
		// Then, cut the module file name (probably common.dll)
		// We need replace this file by own _lpcszFile
		cmnCutFileW(_lpwszDest, wszModuleFileName);
		//
		// Add backslash  
		wcscat(_lpwszDest, _S_BACKSLASHW);
		// Add the new file
		wcscat(_lpwszDest, _lpcwszFile);
		//
		// Set return value
		lpwszRet = _lpwszDest;		
	}
	return lpwszRet;
}

COMMON_API LPSTR	cmnGetDataFilePathA(LPSTR _lpszDest, LPCSTR _lpcszFile)
{
	LPSTR lpszRet = NULL;
	//
	// Check input parameters
	if(_lpszDest && _lpcszFile) {
		//
		char szModuleFileName[MAX_PATH];
		// Get program path
		if(!GetModuleFileNameA(GetModuleHandle(NULL), szModuleFileName, MAX_PATH)) {
			XException exp("Cannot get module filename.",  GetLastError());
			THROW(exp);				
		}
		//
		// Then, cut the module file name (probably common.dll)
		// We need replace this file by own _lpcszFile
		cmnCutFileA(_lpszDest, szModuleFileName);
		//
		// Add backslash  
		strcat(_lpszDest, _S_BACKSLASHA);
		// Add the new file
		strcat(_lpszDest, _lpcszFile);
		//
		// Set return value
		lpszRet = _lpszDest;		
	}
	return lpszRet;
}

COMMON_API LPWSTR	cmnCutDirectoryW(LPWSTR _lpwszDest, LPCWSTR _lpcwszPath)
{
	LPWSTR lpwszRet = NULL;
	//
	// Check input parameters
	if(_lpwszDest && _lpcwszPath) {
		//
		LPWSTR	lpFileName;
		// We must find last backslash in the input buffer
		lpFileName = wcsrchr(_lpcwszPath, _S_BACKSLASH_CHARW);
		// Check if the backslash is in the input buffer
		if(lpFileName) {		
			//
			// Move cursor to first character of the file
			lpFileName++;
			//
			// Copy file to output buffer
			wcscpy(_lpwszDest, lpFileName);			
		}
		// Or copy input directly to output
		else {
			//
			// Copy input to output buffer
			wcscpy(_lpwszDest, _lpcwszPath);			
		}
		//
		// Set return value
		lpwszRet = _lpwszDest;
	}
	return lpwszRet;
}

/*
 *	Cut directory part from full path, for example: C:\Pokus\Test.txt--->>>Test.txt
 *
 *	Parameters:		1. Output buffer for filename
 *					2. Input full path					
 *
 *	Return pointer to output buffer (same as _lpszDest) in case of success or NULL
 *	Func may throw string exception!
 */
COMMON_API LPSTR	cmnCutDirectoryA(LPSTR _lpszDest, LPCSTR _lpcszPath)
{
	LPSTR lpszRet = NULL;
	//
	// Check input parameters
	if(_lpszDest && _lpcszPath) {
		//
		LPSTR	lpFileName;
		// We must find last backslash in the input buffer
		lpFileName = strrchr(_lpcszPath, _S_BACKSLASH_CHARA);
		// Check if the backslash is in the input buffer
		if(lpFileName) {		
			//
			// Move cursor to first character of the file
			lpFileName++;
			//
			// Copy file to output buffer
			strcpy(_lpszDest, lpFileName);			
		}
		// Or copy input directly to output
		else {
			//
			// Copy input to output buffer
			strcpy(_lpszDest, _lpcszPath);			
		}
		//
		// Set return value
		lpszRet = _lpszDest;
	}
	return lpszRet;
}

COMMON_API LPWSTR	cmnCutFileW(LPWSTR _lpwszDest, LPCWSTR _lpcwszPath)
{
	LPWSTR lpwszRet = NULL;
	//
	// Check input parameters
	if(_lpwszDest && _lpcwszPath) {
		//
		LPWSTR	lpLastBackslash;
		wchar_t	wszTempInput[MAX_PATH];
		//
		// Create writeable backup of input buffer
		wcscpy(wszTempInput, _lpcwszPath);
		//
		// We must find last backslash in the input buffer (in backup)
		lpLastBackslash = wcsrchr(wszTempInput, _S_BACKSLASH_CHARW);
		// Check if the backslash is in the input buffer
		if(lpLastBackslash) {
			//
			// Then change this backslash to terminating char
			*lpLastBackslash = _S_NULL_CHARW;
			//
			// Copy new string to output, strcpy stops at new '\0'
			wcscpy(_lpwszDest, wszTempInput);	
		}
		// Or copy input directly to output
		else {
			//
			// Copy input to output buffer
			wcscpy(_lpwszDest, _lpcwszPath);	
		}
		//
		// Set return value
		lpwszRet = _lpwszDest;
	}
	return lpwszRet;
}

/*
 *	Cut filename part from full path: C:\Pokus\Test.txt--->>>C:\Pokus
 *
 *	Parameters:		1. Output buffer for directory
 *					2. Input full path					
 *
 *	Return pointer to output buffer (same as _lpszDest) in case of success or NULL
 *	Func may throw string exception!
 */
COMMON_API LPSTR	cmnCutFileA(LPSTR _lpszDest, LPCSTR _lpcszPath)
{
	LPSTR lpszRet = NULL;
	//
	// Check input parameters
	if(_lpszDest && _lpcszPath) {
		//
		LPSTR	lpLastBackslash;
		char	szTempInput[MAX_PATH];
		//
		// Create writeable backup of input buffer
		strcpy(szTempInput, _lpcszPath);
		//
		// We must find last backslash in the input buffer (in backup)
		lpLastBackslash = strrchr(szTempInput, _S_BACKSLASH_CHARA);
		// Check if the backslash is in the input buffer
		if(lpLastBackslash) {
			//
			// Then change this backslash to terminating char
			*lpLastBackslash = _S_NULL_CHARA;
			//
			// Copy new string to output, strcpy stops at new '\0'
			strcpy(_lpszDest, szTempInput);	
		}
		// Or copy input directly to output
		else {
			//
			// Copy input to output buffer
			strcpy(_lpszDest, _lpcszPath);	
		}
		//
		// Set return value
		lpszRet = _lpszDest;
	}
	return lpszRet;
}

COMMON_API LPWSTR	cmnCutPathFrontW(LPWSTR _lpwszDest, LPCWSTR _lpcwszPath)
{
	LPWSTR lpwszRet = NULL;

	if(_lpwszDest && _lpcwszPath)
	{
		LPWSTR	lpFirstBackslash = NULL;
		WCHAR	wszTempInput[MAX_PATH];
		//
		// Create writeable backup of input buffer
		wcscpy(wszTempInput, _lpcwszPath);
		//
		// We must find last backslash in the input buffer (in backup)
		lpFirstBackslash = wcschr(wszTempInput, _S_BACKSLASH_CHARW);
		// Check if the backslash is in the input buffer
		if(lpFirstBackslash) {
			//
			// Then change this backslash to terminating char
			*(lpFirstBackslash + 1) = _S_NULL_CHARW;	// No sizeof(char) or sizeof(wchar_t), because size
														// is determined by the type of the pointer
			
			// Copy new string to output, strcpy stops at new '\0'
			wcscpy(_lpwszDest, wszTempInput);	
		}
		// Or copy input directly to output
		else {
			//
			// Copy input to output buffer
			wcscpy(_lpwszDest, _lpcwszPath);	
		}
		//
		// Set return value
		lpwszRet = _lpwszDest;
	}
	
	return lpwszRet;
}

COMMON_API LPSTR	cmnCutPathFrontA(LPSTR _lpszDest, LPCSTR _lpcszPath)
{
	LPSTR lpszRet = NULL;

	if(_lpszDest && _lpcszPath)
	{
		LPSTR	lpFirstBackslash = NULL;
		char	szTempInput[MAX_PATH];
		//
		// Create writeable backup of input buffer
		strcpy(szTempInput, _lpcszPath);
		//
		// We must find last backslash in the input buffer (in backup)
		lpFirstBackslash = strchr(szTempInput, _S_BACKSLASH_CHARA);
		// Check if the backslash is in the input buffer
		if(lpFirstBackslash) {
			
			// Then change this backslash to terminating char
			*(lpFirstBackslash + 1) = _S_NULL_CHARA;	// No sizeof(char) or sizeof(wchar_t), because size
														// is determined by the type of the pointer
			
			// Copy new string to output, strcpy stops at new '\0'
			strcpy(_lpszDest, szTempInput);	
		}
		// Or copy input directly to output
		else {
			//
			// Copy input to output buffer
			strcpy(_lpszDest, _lpcszPath);
		}
		//
		// Set return value
		lpszRet = _lpszDest;
	}
	
	return lpszRet;
}

#pragma COMMENT("FindValueIndex cannot find substrings smaller than the size of the DWORD. Fix it!!!")
// Extract the index of the substring in the memory block of specified length (for example a memory mapped file)
//
// The lpdwLength points to the length of the string and after the function returns
// it is an index to the buffer, where the value after the string begins
// If the substring hasn't been found, then return index of 0 - the start of the string
// !!!Works only with non-UNICODE substrings, i.e. ANSI strings
COMMON_API DWORD cmnFindValueIndex(LPBYTE _lpBuffer, LPCSTR _lpszSubString, LPDWORD _lpdwLength)
{
	DWORD dwRet = ERROR_INVALID_PARAMETER;

	if(_lpBuffer && _lpdwLength)
	{
		if(*_lpdwLength <= 0) { return dwRet; }

		DWORD i = 0, j = 0;
		DWORD dwSubStringLen = (DWORD)strlen(_lpszSubString);
		
		LPDWORD lpdwSubStrValue = reinterpret_cast<LPDWORD>((LPBYTE)_lpszSubString);	// The pointer to the first
																						// four bytes of the substring
																						// in reversed order(LITTLE
																						// ENDIAN)
        LPDWORD lpdwBufferValue = NULL;	// The pointer to first four bytes of buffer

		while(i < (*_lpdwLength))	// While we have data
		{
			lpdwBufferValue = reinterpret_cast<LPDWORD>(_lpBuffer + i);	// The four bytes of buffer

			if(*lpdwBufferValue == *lpdwSubStrValue)
			{
				//TRACEA("Match '%c%c%c%c'\n", _lpBuffer[i], _lpBuffer[i+1], _lpBuffer[i+2], _lpBuffer[i+3]);
				DWORD j;

				// Check the rest of the string (dwSubStringLen - 4)
				for(j = 4; j < dwSubStringLen; j++)
				{
					//TRACEA("Checking '%c'", _lpszSubString[j]);
					if(!(_lpBuffer[i+j] == _lpszSubString[j]))
					#ifdef _DEBUG
					{
					  	//TRACEA(" ... No Match(it is '%c')\n", _lpBuffer[i+j]);
						break;	// If don't match, break the loop
					}
					else
					{
						  //TRACEA(" ... Match\n");
					}
					#else
					{ 
						break;
					}
					#endif
				}
				
				if(j >= (dwSubStringLen))	// We've got a whole match
				{
					*_lpdwLength = i+j;	// Move to the begining of the substring
					
					return ERROR_SUCCESS;
				}
			}

			i++;	// Move on to next byte
		}
		dwRet = ERROR_NOT_FOUND;
		*_lpdwLength = 0;	// We return a pos to the string start if not succeeded
	}

	return dwRet;
}

// Read the value to the string(ANSI/UNICODE) variable from the buffer. The value is ended by the character spec-
// ified as a parameter
// If the _lpdwSize is not NULL and _lpDest is NULL the function will compute the number of characters required
// for storage of the value
// If the caller isn't sure, that the value he wants is ended by the specified char, he should specify the size of
// the source buffer, so we don't get to the other's memory. If not specified, the dwMaxBufferSize has size 0xFFFF
// This version is for ANSI strings only. UNICODE version is coming soon ;))))
COMMON_API DWORD cmnExtractValueFromIndexA(LPSTR _lpszDest, LPBYTE _lpBuffer, const CHAR _chEnd, LPDWORD _lpdwSize,
									    DWORD _dwMaxBufferSize)
{
	DWORD dwRet = ERROR_INVALID_PARAMETER;

	if(_lpBuffer)	// If we have a valid buffer
	{
		DWORD dwStrLength = 0;	// Character counter

		// While not data equal to the selected char or we are at the end of the buffer
		while((_chEnd != *(_lpBuffer + dwStrLength)) && (dwStrLength <= _dwMaxBufferSize))
		{
			dwStrLength++;
		}

		dwStrLength++;	// The 1 is for the terminating NULL character

		if(_lpdwSize)	// If the LPDWORD is NULL, then the parameters are invalid
		{
			if(_lpszDest && (dwStrLength <= *_lpdwSize))	// If the buffer exists and we have enough space
			{
				//TRACEA("Copying %u characters\n", dwStrLength);
				// Copy the characters
				strncpy(_lpszDest, (LPCSTR)_lpBuffer, dwStrLength);
				_lpszDest[dwStrLength - 1] = '\0';	// Add the terminating zero char to the output string
				dwRet = ERROR_SUCCESS;
			}
			else
			{
				// The user just wants to know the length
				if(dwStrLength >= _dwMaxBufferSize)
				{
					//TRACEA("End of buffer reached at %u\n", dwStrLength);
					*_lpdwSize = 0;	// Set to zero
					return ERROR_NOT_FOUND;	// Specify, we haven't found the termination char
				}
				
				// For debugging writes the data found to the debugger (very slow ;)))
				/*TRACEA("The value is > ");
				for(DWORD i = 0; i < dwStrLength; i++)
				{
					TRACEA("%c", _lpBuffer[i]);
				}
				TRACEA("\n");*/ 

				*_lpdwSize = dwStrLength;
				dwRet = ERROR_SUCCESS;	// return success
			}
		}
	}

	return dwRet;
}

#pragma COMMENT("ExtractValue() generates first-chance exception, when the search string isn't contained in the buffer")

// The following function searches the buffer for the specified value and extracts it's value. The functions
// FindValueIndex() and ExtractValueFromIndex() are used
// _lpszDest ... output string
// _lpBuffer ... source buffer
// _chEnd    ... character ending the value
// _lpdwSize ... the size needed for storing the value (if _lpszDest == NULL the function computes the size needed)
// _dwMaxBufferSize ... the size of the whole buffer, for overflow checking
COMMON_API DWORD cmnExtractValueA(LPSTR _lpszDest, LPBYTE _lpBuffer, LPCSTR _lpszValue, const CHAR _chEnd, LPDWORD _lpdwSize,
								DWORD _dwMaxBufferSize)
{
	DWORD dwRet = ERROR_INVALID_PARAMETER;
	
	// First find the value
	DWORD dwIndex = _dwMaxBufferSize;

	dwRet = cmnFindValueIndex(_lpBuffer, _lpszValue, &dwIndex);	// Use the MaxBufferSize to limit the search
	
	if((ERROR_SUCCESS == dwRet) && (_lpdwSize) && (dwIndex < _dwMaxBufferSize))
	{
		// We have found the index, so get the size of the value
		DWORD dwSize;

		dwRet = cmnExtractValueFromIndexA(NULL, (_lpBuffer + dwIndex), _chEnd, &dwSize, _dwMaxBufferSize - dwIndex + 1);

		if(SUCCEEDED(dwRet))
		{
			if((_lpszDest) && (dwSize <= *(_lpdwSize)))
			{
				*_lpdwSize = dwSize;	// Return the correct size
				dwRet = cmnExtractValueFromIndexA(_lpszDest, _lpBuffer + dwIndex, _chEnd, _lpdwSize, _dwMaxBufferSize - dwIndex + 1);
				
				return dwRet;	// Possibly found the string
			}
			
			if(!_lpszDest)
			{
				// The user just wants the size of the buffer to allocate

				*_lpdwSize = dwSize;
				dwRet = ERROR_SUCCESS;
			}
		}
		else
		{
			dwRet = ERROR_NOT_FOUND;	// Probably not found the terminating character
		}
	}
	else
	{
		//TRACEA("The search string hasn't been found\n");
		dwRet = ERROR_NOT_FOUND;
	}

	return dwRet;
}

COMMON_API DWORD cmnConvertANSIToUNICODE(LPWSTR _lpwszUnicode, LPDWORD _lpdwUnicodeLen, LPCSTR _lpcszAnsi, DWORD _dwAnsiLen)
{
	DWORD dwRet = ERROR_INVALID_PARAMETER;

	if((_lpcszAnsi) && (_lpdwUnicodeLen))
	{
		DWORD dwUnicodeLen = 0;

		dwUnicodeLen = ::MultiByteToWideChar(CP_ACP, 0, _lpcszAnsi, _dwAnsiLen, NULL, 0);

		if((_lpwszUnicode) && (dwUnicodeLen <= (*_lpdwUnicodeLen)))
		{
            // If the buffer exists and there is enough memory in the buffer, convert the string
			::MultiByteToWideChar(CP_ACP, 0, _lpcszAnsi, _dwAnsiLen, _lpwszUnicode, dwUnicodeLen);
		}
		else
		{
			// The user just wants to know the correct length
			*_lpdwUnicodeLen = dwUnicodeLen;
		}
		
		dwRet = ERROR_SUCCESS;
	}

	return dwRet;
}

COMMON_API DWORD cmnConvertUNICODEToANSI(LPSTR _lpszAnsi, LPDWORD _lpdwAnsiLen, LPCWSTR _lpcwszUnicode, DWORD _dwUnicodeLen)
{
	DWORD dwRet = ERROR_INVALID_PARAMETER;

	if((_lpcwszUnicode) && (_dwUnicodeLen))
	{
		DWORD dwAnsiLen = 0;

		dwAnsiLen = ::WideCharToMultiByte(CP_ACP, 0, _lpcwszUnicode, _dwUnicodeLen, NULL, 0, NULL, NULL);

		if((_lpszAnsi) && (dwAnsiLen <= (*_lpdwAnsiLen)))
		{
            // If the buffer exists and there is enough memory in the buffer, convert the string
			::WideCharToMultiByte(CP_ACP, 0, _lpcwszUnicode, _dwUnicodeLen, _lpszAnsi, dwAnsiLen, NULL, NULL);
		}
		else
		{
			// The user just wants to know the correct length
			*_lpdwAnsiLen = dwAnsiLen;
		}
		
		dwRet = ERROR_SUCCESS;
	}

	return dwRet;
}


