/*
 * CmnStr.h - string functions
 * Date :	24.12.2002
 * Update : 22.06.2003
 * Author :	MrAndrew, Swap
 * Updater:	MrAndrew 
 *
 */

#ifndef _CMNSTR_H_
	#define _CMNSTR_H_

	// Makes the full path from path and filename
	COMMON_API LPWSTR	cmnCreatePathW(LPWSTR _lpszDest, LPCWSTR _lpcszPath, LPCWSTR _lpcszFile, LPDWORD _lpdwSize = NULL);
	COMMON_API LPSTR	cmnCreatePathA(LPSTR _lpszDest, LPCSTR _lpcszPath, LPCSTR _lpcszFile, LPDWORD _lpdwSize = NULL);
	//
	// Adds _lpcszFile to program path
	COMMON_API LPWSTR	cmnGetDataFilePathW(LPWSTR _lpwszDest, LPCWSTR _lpcwszFile) throw( XException );
	COMMON_API LPSTR	cmnGetDataFilePathA(LPSTR _lpszDest, LPCSTR _lpcszFile) throw( XException );
	// Cut directory part from full path: C:\Pokus\Test.txt--->>>Test.txt
	COMMON_API LPWSTR	cmnCutDirectoryW(LPWSTR _lpwszDest, LPCWSTR _lpcwszPath);
	COMMON_API LPSTR	cmnCutDirectoryA(LPSTR _lpszDest, LPCSTR _lpcszPath);
	// Cut filename part from full path: C:\Pokus\Test.txt--->>>C:\Pokus
	COMMON_API LPWSTR	cmnCutFileW(LPWSTR _lpwszDest, LPCWSTR _lpcwszPath);
	COMMON_API LPSTR	cmnCutFileA(LPSTR _lpszDest, LPCSTR _lpcszPath);
	// Cut the path from front by backslashes: "C:\Pokus\"--->>>"C:\"
	COMMON_API LPWSTR	cmnCutPathFrontW(LPWSTR _lpwszDest, LPCWSTR _lpcwszPath);
	COMMON_API LPSTR	cmnCutPathFrontA(LPSTR _lpszDest, LPCSTR _lpcszPath);
	// Get the length of the string without the terminating zero char
	inline COMMON_API DWORD cmnGetStrLenW(LPCWSTR _lpwszStr)
	{
		return (DWORD)wcslen(_lpwszStr);
	}

	inline COMMON_API DWORD cmnGetStrLenA(LPCSTR _lpszStr)
	{
		return (DWORD)strlen(_lpszStr);
	}

	#ifdef UNICODE
		#define cmnCreatePath		cmnCreatePathW
		#define	cmnGetDataFilePath	cmnGetDataFilePathW
		#define cmnCutDirectory		cmnCutDirectoryW
		#define cmnCutFile			cmnCutFileW
		#define cmnCutPathFront		cmnCutPathFrontW
		#define cmnGetStrLen		cmnGetStrLenW
	#else
		#define cmnCreatePath		cmnCreatePathA
		#define	cmnGetDataFilePath	cmnGetDataFilePathA
		#define cmnCutDirectory		cmnCutDirectoryA
		#define cmnCutFile			cmnCutFileA
		#define cmnCutPathFront		cmnCutPathFrontA
		#define cmnGetStrLen		cmnGetStrLenA
	#endif
  
	// Extract the index of the substring in the memory block of specified length (for example a memory mapped file)
	//
	// The lpdwLength points to the length of the string and after the function returns
	// it is an index to the buffer, where the value after the string begins
	// If the substring hasn't been found, then return index of 0 - the start of the string
	// !!!Works only with non-UNICODE substrings, i.e. ANSI strings
	COMMON_API DWORD cmnFindValueIndex(LPBYTE _lpBuffer, LPCSTR _lpszSubString, LPDWORD _lpdwLength);

	// Read the value to the string(ANSI/UNICODE) variable from the buffer. The value is ended by the character spec-
	// ified as a parameter
	// If the _lpdwSize is not NULL and _lpDest is NULL the function will compute the number of characters required
	// for storage of the value
	// Currently supports only ANSI
	COMMON_API DWORD cmnExtractValueFromIndexA(LPSTR _lpszDest, LPBYTE _lpBuffer, const CHAR _chEnd, LPDWORD _lpdwSize = NULL,
											DWORD _dwMaxBufferSize = 0xFFFF);
	
	// No UNICODE version
	#define cmnExtractValueFromIndex	cmnExtractValueFromIndexA

	// The following function searches the buffer for the specified value and extracts it's value. The functions
	// FindValueIndex() and ExtractValueFromIndex() are used
	// _lpszDest ... output string
	// _lpBuffer ... source buffer
	// _chEnd    ... character ending the value
	// _lpdwSize ... the size needed for storing the value (if _lpszDest == NULL the function computes the size needed)
	// _dwMaxBufferSize ... the size of the whole buffer, for overflow checking
	COMMON_API DWORD cmnExtractValueA(LPSTR _lpszDest, LPBYTE _lpBuffer, LPCSTR _lpszValue, const CHAR _chEnd, LPDWORD _lpdwSize = NULL,
									DWORD _dwMaxBufferSize = 0xFFFF);

	// No UNICODE version
	#define cmnExtractValue	cmnExtractValueA

	// The following functions are used to convert strings between UNICODE and ANSI. The function is usually
	// called two times. The first call has the destination string set to NULL and the function just returns
	// the length of the converted string via its parameter _lpdwUnicodeLen. The caller then allocates the
	// buffer and calls the function the second time
	COMMON_API DWORD cmnConvertANSIToUNICODE(LPWSTR _lpwszUnicode, LPDWORD _lpdwUnicodeLen, LPCSTR _lpcszAnsi, DWORD _dwAnsiLen = -1);
	COMMON_API DWORD cmnConvertUNICODEToANSI(LPSTR _lpszAnsi, LPDWORD _lpdwAnsiLen, LPCWSTR _lpcwszUnicode, DWORD _dwUnicodeLen = -1);

#endif
