/*
 *	Adds zip suppoport to Common.dll when zip.dll is available
 *
 *	Filename: CmnZip.h
 *	Authors: Swap a MrAndrew
 *	Date: 28.12. 02
 *	Update: 30.12. 02
 *
 */

#ifndef _CMNZIP_H_
	#define _CMNZIP_H_

	/* information about compression */
	 COMMON_API typedef struct tagCOMPRESSIONINFO {

		DWORD	dwCompressedSize;
		DWORD	dwUncompressedSize;
		float	fRatio;

	} COMPRESSIONINFO, *LPCOMPRESSIONINFO;

	/*     Compresses the source buffer into the destination buffer.  sourceLen is
   the byte length of the source buffer. Upon entry, destLen is the total
   size of the destination buffer, which must be at least 0.1% larger than
   sourceLen plus 12 bytes. Upon exit, destLen is the actual size of the
   compressed buffer.
     This function can be used to compress a whole file at once if the
   input file is mmap'ed.
     Compress returns: 
	 1. ERROR_SUCCESS if success.
	 2. ERROR_NOT_ENOUGH_MEMORY if there was not  enough memory.
	 3. ERROR_INSUFFICIENT_BUFFER if there was not enough room in the output buffer
     4. ERROR_CALL_NOT_IMPLEMENTED if Zip support is not available, function returns  
	 5. E_CMN_UNKNOWN if unknown error occured */
	COMMON_API DWORD	cmnCompressBuffer(LPBYTE lpDest, LPDWORD lpdwDestLen, LPBYTE lpSrc, DWORD dwSrcLen);

	/*     Decompresses the source buffer into the destination buffer.  sourceLen is
   the byte length of the source buffer. Upon entry, destLen is the total
   size of the destination buffer, which must be large enough to hold the
   entire uncompressed data. (The size of the uncompressed data must have
   been saved previously by the compressor and transmitted to the decompressor
   by some mechanism outside the scope of this compression library.)
   Upon exit, destLen is the actual size of the compressed buffer.
     This function can be used to decompress a whole file at once if the
   input file is mmap'ed.
    Uncompress returns: 
	 1. ERROR_SUCCESS if success.
	 2. ERROR_NOT_ENOUGH_MEMORY if there was not  enough memory.
	 3. ERROR_INSUFFICIENT_BUFFER if there was not enough room in the output buffer
	 4. ERROR_INVALID_DATA if the input data was corrupted.
     4. ERROR_CALL_NOT_IMPLEMENTED if Zip support is not available, function returns  
	 5. E_CMN_UNKNOWN if unknown error occured */   
	COMMON_API DWORD	cmnUncompressBuffer(LPBYTE lpDest, LPDWORD lpdwDestLen, LPBYTE lpSrc, DWORD dwSrcLen);
	
	/* compress and uncompress specified file 
	returns same values as cmnUncompressBuffer() + error codes from cmnOpenExistingFile()
	cmnCreateFile etc.*/
	COMMON_API DWORD	cmnCompressFileA(LPCSTR lpszPath, LPCOMPRESSIONINFO  pCompInfo = NULL);
	/* returns uncompressed file in lplpData buffer of pdwSize */
	COMMON_API DWORD	cmnUncompressFileA(LPCSTR lpszPath, LPVOID* lplpData = NULL, DWORD *pdwSize = NULL, LPCOMPRESSIONINFO  pCompInfo = NULL);	

	/* compress and uncompress specified file 
	returns same values as cmnCompressBuffer() + error codes from cmnOpenExistingFile()
	cmnCreateFile etc.*/
	COMMON_API DWORD	cmnCompressFileW(LPCWSTR lpwszPath, LPCOMPRESSIONINFO  pCompInfo = NULL);
	/* returns uncompressed file in lplpData buffer of pdwSize */
	COMMON_API DWORD	cmnUncompressFileW(LPCWSTR lpwszPath, LPVOID* lplpData = NULL, DWORD *pdwSize = NULL, LPCOMPRESSIONINFO  pCompInfo = NULL);	


	#ifdef UNICODE 
		#define cmnCompressFile		cmnCompressFileW
		#define cmnUncompressFile	cmnUncompressFileW
	#else
		#define cmnCompressFile		cmnCompressFileA
		#define cmnUncompressFile	cmnUncompressFileA
	#endif

	/* name of zip.dll */
	#define	_S_ZIP_DLLW	L"zip.dll"
	#define	_S_ZIP_DLLA	 "zip.dll"
	
	//
	// Local function called from DLLMain()	to locate zip.dll and save 
	// poitner to compress and uncompress funcs
	DWORD	InitZipLib() throw( XException );	

	/* define names and types of compression functions */
	#define	FUNC_COMPRESS		"compress"
	#define	FUNC_UNCOMPRESS		"uncompress"

	typedef int (*compress_func)(LPBYTE, LPDWORD, LPBYTE, DWORD); 
	typedef int (*uncompress_func)(LPBYTE, LPDWORD, LPBYTE, DWORD); 

	/* error codes of compress and uncompress functions from zip.dll */
	#define Z_OK            0	
	#define Z_DATA_ERROR   (-3)
	#define Z_MEM_ERROR    (-4)
	#define Z_BUF_ERROR    (-5)
	
#endif	// _CMNZIP_H_
