#include "stdafx.h"
#define		DISPLAY_API	__declspec(dllexport)


#include "../Common/Common.h"
#include "xcamera.h"
#include "XTexture.h"
#include "XMesh.h"
#include "XVertexBuffer.h"
#include "XIndexBuffer.h"
#include "xresourcemanager.h"
#include "xdisplay.h"
#include "x3dobject.h"

X3DObject::X3DObject(void)
{
	m_lpDevice = NULL;
	m_lpTransMat = new D3DXMATRIX;
	D3DXMatrixIdentity(m_lpTransMat);
}

X3DObject::~X3DObject(void)
{
	SAFE_DELETE(m_lpTransMat);
}

// standard methods
int						X3DObject::Init(LPDIRECT3DDEVICE8 lpDevice, 
										LPCSTR szTexture, int iMipMap, D3DFORMAT formatTexture,  
										DWORD dwVBSize, 
										DWORD dwIBSize, 
										DWORD dwFlags)
{
	DWORD dwUsage = 0;
	int iRet;
	m_lpDevice = lpDevice;	
	if(m_lpDevice)
	{
		if(dwVBSize != 0)
		{	
			if(VB_WRITEONLY & dwFlags)
			{
				dwUsage |= D3DUSAGE_WRITEONLY;
			}
			iRet = m_VB.Create(dwVBSize, dwUsage, VERTEXFORMAT, D3DPOOL_DEFAULT, m_lpDevice);
			if(iRet != 0)
			{
				TRACE("Cannot create VB for 3DObject.");
			}
		}
		if(dwIBSize != 0)
		{			
			dwUsage = 0;
			if(IB_WRITEONLY & dwFlags)
			{
				dwUsage |= D3DUSAGE_WRITEONLY;
			}
			iRet = m_IB.Create(dwIBSize, dwUsage, D3DFMT_INDEX16, D3DPOOL_DEFAULT, m_lpDevice);
			if(iRet != 0)
			{
				TRACE("Cannot create IB for 3DObject.");
			}
		}
		if(szTexture)
		{			
			iRet = m_Texture.LoadTextureFromFileEx(szTexture, iMipMap, formatTexture, lpDevice);
			if(iRet != 0)
			{
				TRACE("Cannot create texure.");
			}
		}
		m_bVisible = TRUE;
		m_bApplyTransformations = TRUE;
		return S_OK;
	}
	return S_FALSE;
}
int						X3DObject::Draw(D3DPRIMITIVETYPE Type,
										UINT MinIndex,
										UINT NumVertices,
										UINT StartIndex,
										UINT PrimitiveCount)
{	
	if(m_lpDevice)
	{	
		if(m_lpTransMat)
		{			
			m_lpDevice->SetTransform(D3DTS_WORLD, m_lpTransMat);	
		}
		if(m_bVisible)
		{			
			m_lpDevice->SetVertexShader(VERTEXFORMAT);
			m_lpDevice->SetStreamSource(0, m_VB.GetBuffer(), sizeof(VERTEX));
			m_lpDevice->SetIndices(m_IB.GetBuffer(), 0);
			m_lpDevice->SetTexture(0, m_Texture.GetTexture());
			m_lpDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, 4, 0, 2);	
		}
		// set indentity matrix
		if(m_lpTransMat)
		{
			D3DXMATRIX mat;
			D3DXMatrixIdentity(&mat);
			m_lpDevice->SetTransform(D3DTS_WORLD, &mat);
		}
		return S_OK;
	}
	return S_FALSE;
}

int						X3DObject::Restore()
{
	if(m_lpDevice) 
	{
		m_Texture.Restore();
		m_VB.Restore();
		m_IB.Restore();
		// after that, caller must refill buffers
		return S_OK;
	}
	return S_FALSE;
}

void					X3DObject::Release()
{
	if(m_lpDevice) 
	{
		m_Texture.Release();
		m_VB.Release();
		m_IB.Release();
	}
}

void	X3DObject::Translate(float fX, float fY, float fZ)
{
	if(m_bApplyTransformations)
	{
		D3DXMATRIX	matTemp;
		// Create translation matrix
		D3DXMatrixTranslation(&matTemp, fX, fY, fZ);
		// modify transformation matrix of the object
		D3DXMatrixMultiply(m_lpTransMat, m_lpTransMat, &matTemp);
	}
}

void	X3DObject::RotateX(float fAngle)
{
	if(m_bApplyTransformations)
	{
		D3DXMATRIX	matTemp;
		// create rotation matrix
		D3DXMatrixRotationX(&matTemp, fAngle);
		// modify transformation matrix of the object
		D3DXMatrixMultiply(m_lpTransMat, m_lpTransMat, &matTemp);
	}

}

void	X3DObject::RotateY(float fAngle)
{
	if(m_bApplyTransformations)
	{
		D3DXMATRIX	matTemp;
		// create rotation matrix
		D3DXMatrixRotationY(&matTemp, fAngle);
		// modify transformation matrix of the object
		D3DXMatrixMultiply(m_lpTransMat, m_lpTransMat, &matTemp);
	}
}

void	X3DObject::RotateZ(float fAngle)
{
	if(m_bApplyTransformations)
	{
		D3DXMATRIX	matTemp;
		// create rotation matrix
		D3DXMatrixRotationZ(&matTemp, fAngle);
		// modify transformation matrix of the object
		D3DXMatrixMultiply(m_lpTransMat, m_lpTransMat, &matTemp);
	}
}

void	X3DObject::ScaleXYZ(float fFactor)
{
	if(m_bApplyTransformations)
	{
		D3DXMATRIX	matTemp;
		// Create scaling matrix
		D3DXMatrixScaling(&matTemp, fFactor, fFactor, fFactor);
		// modify transformation matrix of the object
		D3DXMatrixMultiply(m_lpTransMat, m_lpTransMat, &matTemp);
	}
}

void	X3DObject::ScaleX(float fFactor)
{
	if(m_bApplyTransformations)
	{
		D3DXMATRIX	matTemp;
		// Create scaling matrix
		D3DXMatrixScaling(&matTemp, fFactor, 1.0f, 1.0f);
		// modify transformation matrix of the object
		D3DXMatrixMultiply(m_lpTransMat, m_lpTransMat, &matTemp);
	}
}

void	X3DObject::ScaleY(float fFactor)
{
	if(m_bApplyTransformations)
	{
		D3DXMATRIX	matTemp;
		// Create scaling matrix
		D3DXMatrixScaling(&matTemp, 1.0f, fFactor, 1.0f);
		// modify transformation matrix of the object
		D3DXMatrixMultiply(m_lpTransMat, m_lpTransMat, &matTemp);
	}
}

void	X3DObject::ScaleZ(float fFactor)
{
	if(m_bApplyTransformations)
	{
		D3DXMATRIX	matTemp;
		// Create scaling matrix
		D3DXMatrixScaling(&matTemp, 1.0f, 1.0f, fFactor);
		// modify transformation matrix of the object
		D3DXMatrixMultiply(m_lpTransMat, m_lpTransMat, &matTemp);
	}
}