#pragma once

#define		VB_WRITEONLY		0x00000001
#define		IB_WRITEONLY		0x00000002

class DISPLAY_API X3DObject
{
	// is object visible?
	BOOL					m_bVisible;
	// set trans matrix before render?
	BOOL					m_bApplyTransformations;
	// vertex buffer
	XVertexBuffer			m_VB;
	// index buffer
	XIndexBuffer			m_IB;
	// texture
	XTexture				m_Texture;
	// transformation matrix
	D3DXMATRIX				*m_lpTransMat;
	// pointer to device
	LPDIRECT3DDEVICE8		m_lpDevice;
	
public:
	// standard methods
	int						Init(	LPDIRECT3DDEVICE8 lpDevice, 
									LPCSTR szTexture, int iMipMap, D3DFORMAT formatTexture ,
									DWORD dwVBSize, 
									DWORD dwIBSize, 
									DWORD dwFlags);
	int						Draw(	D3DPRIMITIVETYPE Type,
									UINT MinIndex,
									UINT NumVertices,
									UINT StartIndex,
									UINT PrimitiveCount);
	int						Restore();
	void					Release();
	XVertexBuffer*			GetVB()			{ return &m_VB; }
	XIndexBuffer*			GetIB()			{ return &m_IB; }
	XTexture*				GetTexture()	{ return &m_Texture; }
	LPDIRECT3DDEVICE8		GetDevice()		{ return m_lpDevice; }
	BOOL					IsVisible()		{ return m_bVisible; }
	void					Visible(BOOL bVis = TRUE) { m_bVisible = bVis; }
	void					EnableTransformation(BOOL bEnable = TRUE) {m_bApplyTransformations = bEnable;}
	BOOL					IsTransformationEnabled() { return m_bApplyTransformations;}

	// transformations methods
	void					Translate(float fX, float fY, float fZ);
	void					RotateX(float fAngle);
	void					RotateY(float fAngle);
	void					RotateZ(float fAngle);
	void					ScaleXYZ(float fFactor);
	void					ScaleX(float fFactor);
	void					ScaleY(float fFactor);
	void					ScaleZ(float fFactor);	

public:
	X3DObject(void);
	~X3DObject(void);
};
