// XCamera.h: interface for the XCamera class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_XCAMERA_H__A484C795_627C_46F3_949B_152D20ABFA84__INCLUDED_)
#define AFX_XCAMERA_H__A484C795_627C_46F3_949B_152D20ABFA84__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define		_C_MAX_ROTATION_SPEED		5.0f
#define		_C_MAX_ZOOMING_SPEED		5.0f
#define		_C_MAX_SCROLLING_SPEED		20.0f
#define		_C_MAX_ZANGLE_SPEED			2.0f

#define		MAX_ZOOM					40
#define		MIN_ZOOM					5

#define		Z_ANGLE						D3DX_PI / 2

#define		MAX_Z_ANGLE					D3DX_PI / 2
#define		MIN_Z_ANGLE					D3DX_PI / 7 

const float NEAR_CLIP = 0.1f;
const float FAR_CLIP = 200.0f;
const float FOV = D3DX_PI / 4.0f;
const float ASPECT = 4.0f / 3.0f;


class DISPLAY_API XCamera  
{
	// D3D device object...
	LPDIRECT3DDEVICE8	m_lpDevice;
	//
	// Camera bounds
	int					m_iMinPos;
	int					m_iMaxPos; 
	//
	// Velocities in all directions
	float				m_fScrlVelX;	
	float				m_fScrlVelY;	
	float				m_fRotVel;
	float				m_fZoomVel;
	float				m_fZAngleVel;
	//
	// Camera position
	D3DXVECTOR3			m_vLookAtPoint;
	D3DXVECTOR3			m_vEyePt;
	float				m_fZoomDistance;
	float				m_fZAngle;		
	float				m_fViewAngle;
	float				m_fCameraHeight;
	//
	// Camera matrices
	D3DXMATRIX			m_matBillboard;		
	D3DXMATRIX			m_matProjection;
	D3DXMATRIX			m_matView;	

public:
	//
	// Inicialization
	HRESULT				InitCamera(IDirect3DDevice8* lpDevice, int iMinBound, int iMaxBound);
	// Update
	HRESULT				ProcessCamera(float	fElapsedTime);	
	// Camera motion
	void				ZoomIn() {m_fZoomVel = -(_C_MAX_ZOOMING_SPEED);}
	void				ZoomOut() {m_fZoomVel = _C_MAX_ZOOMING_SPEED;}
	void				RotateLeft() {m_fRotVel = _C_MAX_ROTATION_SPEED;}
	void				RotateRight() {m_fRotVel = -(_C_MAX_ROTATION_SPEED);}
	void				MoveLeft() {m_fScrlVelX = _C_MAX_SCROLLING_SPEED;}
	void				MoveRight() {m_fScrlVelX = -(_C_MAX_SCROLLING_SPEED);}
	void				MoveUp() {m_fScrlVelY = -(_C_MAX_SCROLLING_SPEED);}
	void				MoveDown() {m_fScrlVelY = _C_MAX_SCROLLING_SPEED;}
	void				IncreaseZAngle() {m_fZAngleVel =   _C_MAX_ZANGLE_SPEED;}
	void				DecreaseZAngle() {m_fZAngleVel = -(_C_MAX_ZANGLE_SPEED);}
	void				SetCameraHeight(float fHeight) {m_fCameraHeight = fHeight;}		
	//
	// Get methods
	float				GetViewAngle() {return m_fViewAngle;}
	float				GetZoom() {	return m_fZoomDistance;}
	float				GetZAngle() {return m_fZAngle;}
	D3DXVECTOR3*		GetCameraPos() {return &m_vLookAtPoint;}
	D3DXVECTOR3*		GetEyePoint() {return &m_vEyePt;}
	D3DXMATRIX*			GetBillboardMatrix() {return &m_matBillboard;}
	void				GetWorldViewProjMatrix(D3DXMATRIX * matWVP);

	XCamera();
	~XCamera();
};

#endif // !defined(AFX_XCAMERA_H__A484C795_627C_46F3_949B_152D20ABFA84__INCLUDED_)
