#include "StdAfx.h"

#define		DISPLAY_API	__declspec(dllexport)

#include "../common/common.h"
#include "xcamera.h"
#include "xtexture.h"
#include "xmesh.h"
#include "xvertexbuffer.h"
#include "xindexbuffer.h"
#include "xresourcemanager.h"

#include "xdisplay.h"

XDisplay::XDisplay(void)
{
	m_lpD3DObject = NULL;
	m_lpD3DDevice = NULL;
}

XDisplay::~XDisplay(void)
{
	Clean();
}

void XDisplay::Clean(void)
{
	if(m_lpD3DDevice) {
 
		m_lpD3DDevice->SetTexture(0, NULL);
	
		SAFE_RELEASE(m_lpD3DDevice);
		SAFE_RELEASE(m_lpD3DObject);
	}
}

int XDisplay::Init(HWND hWnd)
{
	TRACE("Initializing Direct3D system...");	
	HRESULT dwRet;
	DWORD BehaviorFlags;	
	//
	// Already initialized?
	if(m_lpD3DDevice) {
		XException exp("Direct3D system is already initialized.");
		THROW(exp);	
	}
	//	
	// As first, create one Direct3D object	
	m_lpD3DObject = Direct3DCreate8(D3D_SDK_VERSION);
	if(!m_lpD3DObject) {		
		cmnMessageBoxA("Please, install DirectX 8.1.", MSG_ERROR);
		XException exp("Cannot create object of Direct3D.");
		THROW(exp);			
	}
	//
	//
	// Read desired display format from ini file
	//
	// Get resolution values from setup.ini file	
	m_sRes.x = cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_WIDTH);	
	m_sRes.y = cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_HEIGHT);	
	//
	// Get screen depth from ini file (may be 16 or 32)	
	m_formatScreenDepth  = (D3DFORMAT)cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_FORMAT);		
	//
	// If system fullsreen or windowed
	if(cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_WINDOWED)) {
		m_wFlags |= FLAG_WINDOWED;
	}
	//
	// Is FPS enabled?	Default is disabled.
	if(cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_FPS)) {
		m_wFlags |= FLAG_FPS;
	}
	//
	// Get desired FPS from ini. Default is off.
	// If m_iDesiredFPS == 0, then FPS limiter is off.
	m_iDesiredFPS = cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_DESIREDFPS);	
	//
	// Read type of vertex processing
	BehaviorFlags = cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_BEHAVIOR);		
	//
	// Read texture format
	m_formatTextureDepth = (D3DFORMAT)cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_TEXDEPTH);	
	//
	// Set size of back buffer from window position
	if(m_wFlags & FLAG_WINDOWED) {	
		D3DDISPLAYMODE displaymode;
		RECT rcWindowClient;
		//
		// Get window position
		GetClientRect(hWnd, &rcWindowClient);
		// Get current display mode
		dwRet = m_lpD3DObject->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &displaymode);
		if(dwRet != D3D_OK) {
			XException exp("Cannot get display format in window mode.", dwRet);
			THROW(exp);				
		}
		//
		// Set size of back buffer
        m_sRes.x  = rcWindowClient.right - rcWindowClient.left;
        m_sRes.y  = rcWindowClient.bottom - rcWindowClient.top;
		// Set display mode to current
        m_formatScreenDepth = displaymode.Format;
	}
	//
	// Read ordinal number of used adapter
	m_iAdapterOrdinal = cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_ADAPTER);
	// Read type of device on this adapter
	m_typeDeviceType = (D3DDEVTYPE) cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_DEVICE);	
	m_tftMagTextureFilter = (D3DTEXTUREFILTERTYPE) cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_MAGTEXTUREFILTER);	
	m_tftMinTextureFilter = (D3DTEXTUREFILTERTYPE) cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_MINTEXTUREFILTER);	
	m_tftMipTextureFilter = (D3DTEXTUREFILTERTYPE) cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_MIPTEXTUREFILTER);	
	// 
	// Set properties of device	
	ZeroMemory(&m_d3dDeviceParam, sizeof(m_d3dDeviceParam));
	m_d3dDeviceParam.Windowed							= (m_wFlags & FLAG_WINDOWED) ? TRUE : FALSE;
	m_d3dDeviceParam.SwapEffect							= D3DSWAPEFFECT_DISCARD;
    m_d3dDeviceParam.BackBufferWidth					= m_sRes.x;
    m_d3dDeviceParam.BackBufferHeight					= m_sRes.y;
    m_d3dDeviceParam.BackBufferFormat					= m_formatScreenDepth;
	m_d3dDeviceParam.BackBufferCount	 				= 1;	
	m_d3dDeviceParam.hDeviceWindow						= hWnd;
	m_d3dDeviceParam.Flags								= 0;
	//
	// -> Fullscreen AA
	m_d3dDeviceParam.MultiSampleType = (D3DMULTISAMPLE_TYPE)cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_FSAA);
	m_d3dDeviceParam.EnableAutoDepthStencil				= TRUE;
	m_d3dDeviceParam.AutoDepthStencilFormat				= D3DFMT_D16;
	//
	// Set some settings for fullscreen mode
	// -> PresentationInterval
	// -> RefreshRate
	if(!(m_wFlags & FLAG_WINDOWED)) {
		//
		// -> PresentationInterval
		if(cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_VSYNC) == 0) {
			m_d3dDeviceParam.FullScreen_PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;
		}
		else {
			m_d3dDeviceParam.FullScreen_PresentationInterval = D3DPRESENT_INTERVAL_ONE;
		}
		//
		// -> RefreshRate
		m_d3dDeviceParam.FullScreen_RefreshRateInHz	= cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_REFRESHRATE);		
	}	
	// Now we can create D3D device
	dwRet = m_lpD3DObject->CreateDevice(m_iAdapterOrdinal, m_typeDeviceType, hWnd,
										 BehaviorFlags,	&m_d3dDeviceParam, &m_lpD3DDevice);
	if(dwRet != D3D_OK) {
		cmnMessageBoxA("Please, run Setup to set display settings.", MSG_ERROR);
		Clean();
		XException exp("Cannot create D3D device.", dwRet);
		THROW(exp);			
	}

	D3DCAPS8 d3dc;
	// get max active lights
	if(S_OK == m_lpD3DDevice->GetDeviceCaps(&d3dc))
	{
		m_iMaxLights = d3dc.MaxActiveLights;
	}
	else
	{
		// light support off
		m_iMaxLights = 0;
	}

	
	m_dwBackgroundColor = 56564;

	BuildUpMatrices(&D3DXVECTOR3(8.0f, 8.0f, 8.0f), &D3DXVECTOR3(0.0f, 0.0f, 0.0f));

	return 0;
}

void		XDisplay::BuildUpMatrices(D3DXVECTOR3 *pvEyePt, D3DXVECTOR3 *pvLookAtPt)
{
	D3DXMATRIX matWorld, matView, matProj;		
	D3DXVECTOR3	vUpDir;	
	// Check parameters
	if(!pvEyePt || !pvLookAtPt) {
		XException exp("BuildUpMatrices: Invalid input vectors.");
		THROW(exp);						
	}
	//
	// Vertical axis is Z
	vUpDir = D3DXVECTOR3(0.0f, 0.0f, 1.0f);
	//
	// Set transformations	
	D3DXMatrixIdentity(&matWorld);
	//
	// Create view and projection matrices
	D3DXMatrixLookAtLH(&matView, pvEyePt, pvLookAtPt, &vUpDir);	
	D3DXMatrixPerspectiveFovLH(&matProj, D3DX_PI / 4, 4.0f/3.0f, 1.0f, 100.0f);	
	//
	// Set matrices
	m_lpD3DDevice->SetTransform(D3DTS_WORLD, &matWorld);	
	m_lpD3DDevice->SetTransform(D3DTS_VIEW, &matView);
	m_lpD3DDevice->SetTransform(D3DTS_PROJECTION, &matProj);
	//
	// Enable Z-buffer
	m_lpD3DDevice->SetRenderState(D3DRS_ZENABLE, TRUE);		
	//
	// Set wireframe, if user wants to render wireframe model
	if(cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_WIREFRAME)) {
		m_lpD3DDevice->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);
		m_lpD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	}	
	// set standard material
	D3DMATERIAL8 mtrl;
	ZeroMemory( &mtrl, sizeof(D3DMATERIAL8) );
	mtrl.Diffuse.r = mtrl.Ambient.r = 1.0f;
	mtrl.Diffuse.g = mtrl.Ambient.g = 1.0f;
	mtrl.Diffuse.b = mtrl.Ambient.b = 1.0f;
	mtrl.Diffuse.a = mtrl.Ambient.a = 1.0f;
	m_lpD3DDevice->SetMaterial(&mtrl);
	// turn on lighting
	m_lpD3DDevice->SetRenderState(D3DRS_LIGHTING, TRUE);

	//
	// Set texture filter for first stage
	m_lpD3DDevice->SetTextureStageState(0, D3DTSS_MIPFILTER, m_tftMipTextureFilter);
	m_lpD3DDevice->SetTextureStageState(0, D3DTSS_MAGFILTER, m_tftMagTextureFilter);
	m_lpD3DDevice->SetTextureStageState(0, D3DTSS_MINFILTER, m_tftMinTextureFilter);

}

int			XDisplay::UpdateBackground()
{	
	if(m_lpD3DDevice) {
		//
		// Clear the back buffer to a blue color
		m_lpD3DDevice->Clear(0, NULL, D3DCLEAR_TARGET| D3DCLEAR_ZBUFFER, m_dwBackgroundColor, 1.0f, 0);					
		return 0;
	}	
	return 1;
}

int			XDisplay::Present()
{
	DWORD dwRet;
	if(m_lpD3DDevice) {
		// Get time
		m_fTime = cmnGetTime(TIMER_GETELAPSEDTIME);
		m_fTotalTime += m_fTime;

		// Test the cooperative level to see if it's okay to render
		dwRet = m_lpD3DDevice->TestCooperativeLevel();
		if(dwRet != D3D_OK)
		{
			// If the device was lost, do not render until we get it back
			if( D3DERR_DEVICELOST == dwRet ) {
				TRACE("Device is lost.");
				//return (int)dwRet;
			}
			// Check if the device needs to be resized.
			if( D3DERR_DEVICENOTRESET == dwRet ) {				
				RestoreDisplay();					
			}						
			return (int)dwRet;
		}	
		
		if(m_wFlags & FLAG_FPS) {
			UpdateFPS();
		}
		// Set limitations
		if(m_iDesiredFPS != 0) {
			LimitFPS();
		}	
		
		m_lpD3DDevice->Present(NULL, NULL, NULL, NULL);				
	}
	return 0;
}

int		XDisplay::UpdateFPS()
{
	//
	// Get time of system start
	m_fStartTime = cmnGetTime(TIMER_GETAPPTIME);		
	//
	// Increment frames counter	
	m_Frames++;							
	//
	// Compute number of frames per one second
	if((m_fStartTime - m_fStopTime) >= FPS_REFRESH_RATE) {	

		m_fCurrentFPS = m_Frames / (m_fStartTime - m_fStopTime);
		m_Frames = 0;
		//
		// Save old-new time
		m_fStopTime = m_fStartTime;		
		TRACE("%4.1f FPS - ElapsedT: %4.8f: %d", m_fCurrentFPS, m_fTime);
	}	 
	//	
	
	//return disDrawText(0, m_sRes.cx * 45 / 1024, "%4.1f", m_fCurrentFPS);
	return 1;
}

void	XDisplay::LimitFPS()
{
	static float last = 0;
    do
    {
        float now = cmnGetTime(TIMER_GETAPPTIME);
        float passed = now - last;
        m_fFrameRate = float(1.0 / float(passed));
    }
    while(m_fFrameRate > m_iDesiredFPS);
    last = cmnGetTime(TIMER_GETAPPTIME); 
}

int	XDisplay::RestoreDisplay()
{
	TRACE("Restoring device...");
	//
	// Release all resources
	if(!m_resManager.ReleaseAll()) 
	{
		TRACE("Resources released.");
	}
	//	
	// Reset the device
	if(D3D_OK != m_lpD3DDevice->Reset(&m_d3dDeviceParam))
	{
		TRACE("Cannot RESET device object.");			
	}
	else {
		TRACE("Device is reset.");
	}
	// Wait until device is reset
	Sleep(150);

	// restore world/view/proj matrices and set render stages
	BuildUpMatrices(&D3DXVECTOR3(8.0f, 8.0f, 8.0f), &D3DXVECTOR3(0.0f, 0.0f, 0.0f));
	
	// reinit resources
	m_resManager.RestoreAll();
			
	return 0;
}

int	XDisplay::EnableLight(int iIndex, BOOL bEnable)
{
	if(m_lpD3DDevice && (iIndex >= 0 && iIndex < m_iMaxLights))
	{
		return m_lpD3DDevice->LightEnable(iIndex, bEnable);
	}
	return -1;
}

int	XDisplay::SetLight(int iIndex, D3DLIGHT8 * pLight)
{
	if(m_lpD3DDevice && pLight && (iIndex >= 0 && iIndex < m_iMaxLights))
	{
		return m_lpD3DDevice->SetLight(iIndex, pLight);
	}
	return -1;
}
