#include "StdAfx.h"

#define		DISPLAY_API	__declspec(dllexport)


#include "../Common/Common.h"
#include "xtexture.h"
#include "xmesh.h"
#include "xvertexbuffer.h"
#include "xindexbuffer.h"
#include "xresourcemanager.h"


XMesh::XMesh(void)
{
	m_lpMesh = NULL;
	m_lpDevice = NULL;
	m_bVisible = FALSE;	
	m_lpTransMat = new D3DXMATRIX;
	D3DXMatrixIdentity(m_lpTransMat);
}

XMesh::~XMesh(void)
{
	Release();
	m_lpDevice = NULL;
	SAFE_DELETE_ARRAY(m_pMaterials);
	SAFE_DELETE_ARRAY(m_pTextures);
	SAFE_DELETE(m_lpTransMat);
}

int		XMesh::LoadMeshFromFile(LPCSTR szFileName, LPDIRECT3DDEVICE8	lpDevice, XResourceManager * pResMan)
{
	LPD3DXBUFFER pD3DXMtrlBuffer;

	if(!lpDevice) 
	{
		XException exp("LoadMeshFromFile: Device is invalid.");
		THROW(exp);			
	}
	if(!m_lpMesh) {

		// create full path to the texture
		if(!cmnGetDataFilePath(m_szFilePath, szFileName)) 
		{
			TRACE("Cannot create full path to the texture.");
		}
		// load mesh
		if(D3D_OK == D3DXLoadMeshFromX(m_szFilePath, 0, lpDevice, NULL, &pD3DXMtrlBuffer, &m_dwNumMat, &m_lpMesh)) 
		{
			TRACE("Mesh '%s' was loaded.", szFileName);
			// get material buffer
			D3DXMATERIAL* d3dxMaterials = (D3DXMATERIAL*)pD3DXMtrlBuffer->GetBufferPointer();
			// create arrays of materials and textures
			m_pTextures = new XTexture[m_dwNumMat];
			m_pMaterials = new D3DMATERIAL8[m_dwNumMat];
			//
			// get materials and load textures
			for(int i = 0; i < (int)m_dwNumMat; i++) 
			{
				m_pMaterials[i] = d3dxMaterials[i].MatD3D;
				m_pMaterials[i].Ambient = m_pMaterials[i].Diffuse;

				if(d3dxMaterials[i].pTextureFilename)
				{
					m_pTextures[i].LoadTextureFromFile(d3dxMaterials[i].pTextureFilename, lpDevice);
					pResMan->AddTexture(&m_pTextures[i]);
				}				
			}
			pD3DXMtrlBuffer->Release();
		}
		else
		{
			TRACE("Mesh '%s' wasn't loaded.", szFileName);
		}
		// save par to restore
		m_meshType = CUSTOM;
		m_lpDevice = lpDevice;

		return 0;
	}
	return -1;
}

int		XMesh::CreateSphere(float fRadius, UINT uSlices, UINT uStacks, LPDIRECT3DDEVICE8	lpDevice)
{
	if(!lpDevice) 
	{
		XException exp("CreateSphere: Device is invalid.");
		THROW(exp);			
	}
	if(!m_lpMesh) {

		if(D3D_OK == D3DXCreateSphere(lpDevice, fRadius, uSlices, uStacks, &m_lpMesh, NULL)) 
		{
			TRACE("Sphere was loaded.");
		}
		else
		{
			TRACE("Sphere wasn't loaded.");
		}
		// save par to restore
		m_meshType = SPHERE;
		m_lpDevice = lpDevice;
		m_fPar1 = fRadius;
		m_uPar4 = uSlices;
		m_uPar5 = uStacks;

		return 0;
	}
	return -1;
}

int		XMesh::CreateBox(float fWidth, float fHeight, float fDepth, LPDIRECT3DDEVICE8	lpDevice)
{
	if(!lpDevice) 
	{
		XException exp("CreateBox: Device is invalid.");
		THROW(exp);			
	}
	if(!m_lpMesh) {

		if(D3D_OK == D3DXCreateBox(lpDevice, fWidth, fHeight, fDepth, &m_lpMesh, NULL)) 
		{
			TRACE("Box was loaded.");
		}
		else
		{
			TRACE("Box wasn't loaded.");
		}
		// save par to restore
		m_meshType = BOX;
		m_lpDevice = lpDevice;
		m_fPar1 = fWidth;
		m_fPar2 = fHeight;
		m_fPar3 = fDepth;

		return 0;
	}
	return -1;
}

int		XMesh::CreateCylinder(float fRadius1, float fRadius2, float fLenght, UINT uSlices, UINT uStacks, LPDIRECT3DDEVICE8	lpDevice)
{
	if(!lpDevice) 
	{
		XException exp("CreateCylinder: Device is invalid.");
		THROW(exp);					
	}
	if(!m_lpMesh) {

		if(D3D_OK == D3DXCreateCylinder(lpDevice, fRadius1, fRadius2, fLenght, uSlices, uStacks, &m_lpMesh, NULL)) 
		{
			TRACE("Cylinder was loaded.");
		}
		else
		{
			TRACE("Cylinder wasn't loaded.");
		}
		// save par to restore
		m_meshType = CYLINDER;
		m_lpDevice = lpDevice;
		m_fPar1 = fRadius1;
		m_fPar2 = fRadius2;
		m_fPar3 = fLenght;
		m_uPar4 = uSlices;
		m_uPar5 = uStacks;

		return 0;
	}
	return -1;
}

int		XMesh::CreateTeapot(LPDIRECT3DDEVICE8	lpDevice)
{
	if(!lpDevice) 
	{
		XException exp("CreateTeapot: Device is invalid.");
		THROW(exp);							
	}
	if(!m_lpMesh) {

		if(D3D_OK == D3DXCreateTeapot(lpDevice, &m_lpMesh, NULL)) 
		{
			TRACE("Teapot was loaded.");
		}
		else
		{
			TRACE("Teapot wasn't loaded.");
		}
		// save par to restore
		m_meshType = TEAPOT;
		m_lpDevice = lpDevice;
		
		return 0;
	}
	return -1;
}

int		XMesh::CreateTorus(float fInnerRadius, float fOuterRadius, UINT uSides, UINT uRings, LPDIRECT3DDEVICE8	lpDevice)
{
	if(!lpDevice) 
	{
		XException exp("CreateTorus: Device is invalid.");
		THROW(exp);			
	}
	if(!m_lpMesh) {

		if(D3D_OK == D3DXCreateTorus(lpDevice, fInnerRadius, fOuterRadius, uSides, uRings, &m_lpMesh, NULL)) 
		{
			TRACE("Torus was loaded.");
		}
		else
		{
			TRACE("Torus wasn't loaded.");
		}
		// save par to restore
		m_meshType = TORUS;
		m_lpDevice = lpDevice;
		m_fPar1 = fInnerRadius;
		m_fPar2 = fOuterRadius;
		m_uPar4 = uSides;
		m_uPar5 = uRings;
		return 0;
	}
	return -1;
}

int	XMesh::Draw()
{
	if(m_lpDevice && m_bVisible && m_lpMesh)
	{
		// Set transformation matrix before rendering
		if(m_lpTransMat)
		{
			m_lpDevice->SetTransform(D3DTS_WORLD, m_lpTransMat);
		}
		if(m_meshType == CUSTOM) 
		{
			for(int i = 0; i < (int)m_dwNumMat; i++) 
			{
				m_lpDevice->SetMaterial(&m_pMaterials[i]);
				m_lpDevice->SetTexture(0, m_pTextures[i].GetTexture());
				m_lpMesh->DrawSubset(i);
			}
		}
		else 
		{			
			// set default material
			D3DMATERIAL8 mtrl;
			ZeroMemory( &mtrl, sizeof(D3DMATERIAL8) );
			mtrl.Diffuse.r = mtrl.Ambient.r = 1.0f;
			mtrl.Diffuse.g = mtrl.Ambient.g = 1.0f;
			mtrl.Diffuse.b = mtrl.Ambient.b = 1.0f;
			mtrl.Diffuse.a = mtrl.Ambient.a = 1.0f;
			m_lpDevice->SetMaterial(&mtrl);

			m_lpDevice->SetTexture(0, NULL);
			m_lpMesh->DrawSubset(0);
		}
		// set indentity matrix
		if(m_lpTransMat)
		{
			D3DXMATRIX mat;
			D3DXMatrixIdentity(&mat);
			m_lpDevice->SetTransform(D3DTS_WORLD, &mat);
		}
		return 0;
	}
	return -1;
}

int		XMesh::Restore()
{
	if(m_lpDevice) 
	{
		switch(m_meshType) {
			case CUSTOM:				
				// load mesh
				if(D3D_OK == D3DXLoadMeshFromX(m_szFilePath, 0, m_lpDevice, NULL, NULL, NULL, &m_lpMesh)) 
				{
					TRACE("Mesh '%s' was loaded.", m_szFilePath);								
					return 0;
				}
				else
				{
					TRACE("Mesh '%s' wasn't loaded.", m_szFilePath);
				}
				break;
			case SPHERE:
				return CreateSphere(m_fPar1, m_uPar4, m_uPar5, m_lpDevice);
			case BOX:
				return CreateBox(m_fPar1, m_fPar2, m_fPar3, m_lpDevice);
			case CYLINDER:
				return CreateCylinder(m_fPar1, m_fPar2, m_fPar3, m_uPar4, m_uPar5, m_lpDevice);
			case TORUS:
				return CreateTorus(m_fPar1, m_fPar2, m_uPar4, m_uPar5, m_lpDevice);
			case TEAPOT:
				return CreateTeapot(m_lpDevice);
			}
	}
	return -1;
}

void	XMesh::Release()
{		
	SAFE_RELEASE(m_lpMesh);
}

void	XMesh::Translate(float fX, float fY, float fZ)
{
	D3DXMATRIX	matTemp;
	// Create translation matrix
	D3DXMatrixTranslation(&matTemp, fX, fY, fZ);
	// modify transformation matrix of the object
	D3DXMatrixMultiply(m_lpTransMat, m_lpTransMat, &matTemp);
}

void	XMesh::RotateX(float fAngle)
{
	D3DXMATRIX	matTemp;
	// create rotation matrix
	D3DXMatrixRotationX(&matTemp, fAngle);
	// modify transformation matrix of the object
	D3DXMatrixMultiply(m_lpTransMat, m_lpTransMat, &matTemp);

}

void	XMesh::RotateY(float fAngle)
{
	D3DXMATRIX	matTemp;
	// create rotation matrix
	D3DXMatrixRotationY(&matTemp, fAngle);
	// modify transformation matrix of the object
	D3DXMatrixMultiply(m_lpTransMat, m_lpTransMat, &matTemp);
}

void	XMesh::RotateZ(float fAngle)
{
	D3DXMATRIX	matTemp;
	// create rotation matrix
	D3DXMatrixRotationZ(&matTemp, fAngle);
	// modify transformation matrix of the object
	D3DXMatrixMultiply(m_lpTransMat, m_lpTransMat, &matTemp);
}

void					XMesh::ScaleXYZ(float fFactor)
{
	D3DXMATRIX	matTemp;
	// Create scaling matrix
	D3DXMatrixScaling(&matTemp, fFactor, fFactor, fFactor);
	// modify transformation matrix of the object
	D3DXMatrixMultiply(m_lpTransMat, m_lpTransMat, &matTemp);
}

void	XMesh::ScaleX(float fFactor)
{
	D3DXMATRIX	matTemp;
	// Create scaling matrix
	D3DXMatrixScaling(&matTemp, fFactor, 1.0f, 1.0f);
	// modify transformation matrix of the object
	D3DXMatrixMultiply(m_lpTransMat, m_lpTransMat, &matTemp);
}

void	XMesh::ScaleY(float fFactor)
{
	D3DXMATRIX	matTemp;
	// Create scaling matrix
	D3DXMatrixScaling(&matTemp, 1.0f, fFactor, 1.0f);
	// modify transformation matrix of the object
	D3DXMatrixMultiply(m_lpTransMat, m_lpTransMat, &matTemp);
}

void	XMesh::ScaleZ(float fFactor)
{
	D3DXMATRIX	matTemp;
	// Create scaling matrix
	D3DXMatrixScaling(&matTemp, 1.0f, 1.0f, fFactor);
	// modify transformation matrix of the object
	D3DXMatrixMultiply(m_lpTransMat, m_lpTransMat, &matTemp);
}