#include "StdAfx.h"

#define		DISPLAY_API	__declspec(dllexport)

#include "../Common/Common.h"
#include "xtexture.h"

XTexture::XTexture(void)
{
	m_lpTexture = NULL;
	m_lpDevice = NULL;
	strcpy(m_szFilePath, _S_EMPTY);
}

XTexture::~XTexture(void)
{
	if(m_lpTexture)
	{
		Release();
	}
	m_lpDevice = NULL;
}

int		XTexture::LoadTextureFromFile(LPCSTR szFileName, LPDIRECT3DDEVICE8	lpDevice)
{	
	if(!lpDevice) 
	{
		XException exp("LoadTextureFromFile: Device is invalid.");
		THROW(exp);			
	}

	if(!m_lpTexture) 
	{		
		// create full path to the texture
		if(!cmnGetDataFilePath(m_szFilePath, szFileName)) 
		{
			TRACE("Cannot create full path to the texture.");
		}		
		if(D3D_OK == D3DXCreateTextureFromFile(lpDevice,  m_szFilePath, &m_lpTexture))
		{
			TRACE("Texture '%s' was loaded.", szFileName);
		}
		else 
		{
			TRACE("Texture '%s' wasn't loaded.", szFileName);
			return S_FALSE;
		}
		// save information to restore
		m_lpDevice = lpDevice;	
		m_iMipLevels = 1;
		// Get some info about texture
		D3DSURFACE_DESC	texdesc;
		m_lpTexture->GetLevelDesc(0, &texdesc);
		m_iWidth = texdesc.Width;
		m_iHeight = texdesc.Height;
		return 0;
	}
	return -1;		
}

int		XTexture::LoadTextureFromFileEx(LPCSTR szFileName, int iMipLevels, D3DFORMAT formatTextureFormat, LPDIRECT3DDEVICE8	lpDevice)
{	
	if(!lpDevice) 
	{
		XException exp("LoadTextureFromFileEx: Device is invalid.");
		THROW(exp);			
	}

	if(!m_lpTexture) 
	{		
		// create full path to the texture
		if(!cmnGetDataFilePath(m_szFilePath, szFileName)) 
		{
			TRACE("Cannot create full path to the texture.");
		}		
		if(D3D_OK == D3DXCreateTextureFromFileEx(lpDevice,  m_szFilePath, 0, 0, iMipLevels, 
												D3DUSAGE_RENDERTARGET, formatTextureFormat, D3DPOOL_DEFAULT, 
												D3DX_DEFAULT , D3DX_DEFAULT, 0, NULL, NULL, &m_lpTexture))
		{
			TRACE("Texture '%s' was loaded.", szFileName);
		}
		else 
		{
			TRACE("Texture '%s' wasn't loaded.", szFileName);	
			return S_FALSE;
		}
		// save information to restore
		m_lpDevice = lpDevice;	
		m_iMipLevels = iMipLevels;
		m_formatTextureFormat = formatTextureFormat;
		// Get some info about texture
		D3DSURFACE_DESC	texdesc;
		m_lpTexture->GetLevelDesc(0, &texdesc);
		m_iWidth = texdesc.Width;
		m_iHeight = texdesc.Height;
		return 0;
	}
	return -1;	
}

void	XTexture::Release()
{ 
	TRACE("Texture %s released.", m_szFilePath);
	// release texture
	SAFE_RELEASE(m_lpTexture); 	
}

int		XTexture::Restore()
{
	// reinit texture
	if(m_lpDevice) 
	{
		if(m_iMipLevels == 1) 
		{
			// try to create texture	
			if(D3D_OK == D3DXCreateTextureFromFile(m_lpDevice, m_szFilePath, &m_lpTexture)) 
			{
				TRACE("Texture '%s' was reloaded.", m_szFilePath);
			}
			else 
			{
				TRACE("Texture '%s' wasn't reloaded.", m_szFilePath);			
			}
		}
		else
		{
			if(D3D_OK != D3DXCreateTextureFromFileEx(m_lpDevice,  m_szFilePath, 0, 0, m_iMipLevels, 
													D3DUSAGE_RENDERTARGET, m_formatTextureFormat, D3DPOOL_DEFAULT, 
													D3DX_DEFAULT , D3DX_DEFAULT, 0, NULL, NULL, &m_lpTexture))
			{
				TRACE("Texture '%s' was reloaded.", m_szFilePath);
			}
			else 
			{
				TRACE("Texture '%s' wasn't reloaded.", m_szFilePath);			
			}
		}
	}
	return -1;
}
