// Input1.cpp: implementation of the XInput class.
//
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"

#define   INPUT_API __declspec(dllexport)

#include "..\Common\Common.h"
#include "..\Display\Common.h"

#include "Input1.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// Globalni objekt DI
XInput	g_theInput;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

XInput::XInput() 
{
	m_bInit = FALSE;
	m_bShowCursor = TRUE;

	m_lpDI = NULL;
	m_lpDIDKeyboard = NULL;
	m_lpDIDMouse = NULL;
}

XInput::~XInput()
{
	Clean();
}

void XInput::Clean()
{
	if(m_bInit) {

		if(m_lpDIDKeyboard) 
			m_lpDIDKeyboard->Release();

		if(m_lpDIDMouse) 
			m_lpDIDMouse->Release();

		if(m_lpDI) 
			m_lpDI->Release();

		m_bInit = FALSE;
	}
}

HRESULT XInput::Init(HINSTANCE hInst, HWND hWnd, D3DXVECTOR2* csResolution, LPDIRECT3DDEVICE8 lpDevice)
{
	TRACE("Initializing direct input...");
	HRESULT dwRet = 1;
	//
	// You cannot call this function twice
	if(!m_bInit) {
		//
		// Creation of direct input object
		dwRet = DirectInput8Create(hInst, DIRECTINPUT_VERSION, IID_IDirectInput8, (VOID**)&m_lpDI, NULL);		
		if(dwRet != DI_OK) {
			ERR("Cannot create direct input object due", dwRet);
			Clean();
			return dwRet;
		}
		//
		// Create keyboard object
		dwRet = m_lpDI->CreateDevice(GUID_SysKeyboard, &m_lpDIDKeyboard, NULL);		
		if(dwRet != DI_OK) {
			ERR("Cannot create keyboard object due ", dwRet);
			Clean();
			return dwRet;
		}
		//
		// Create mouse object
		dwRet = m_lpDI->CreateDevice(GUID_SysMouse, &m_lpDIDMouse, NULL);		
		if(dwRet != DI_OK) {
			ERR("Cannot create mouse object due ", dwRet);
			Clean();
			return dwRet;
		}
		//
		// Set data format for both devices
		dwRet = m_lpDIDMouse->SetDataFormat(&c_dfDIMouse);		
		if(dwRet != DI_OK) {
			ERR("Cannot set data format for mouse due ", dwRet);
			Clean();
			return dwRet;
		}
		dwRet = m_lpDIDKeyboard->SetDataFormat(&c_dfDIKeyboard);		
		if(dwRet != DI_OK) {
			ERR("Cannot set data format for keyboard due ", dwRet);
			Clean();
			return dwRet;
		}
		//
		// Set cooperative level for both devices
		dwRet = m_lpDIDKeyboard->SetCooperativeLevel(hWnd, DISCL_FOREGROUND|DISCL_NONEXCLUSIVE);		
		if(dwRet != DI_OK) {
			ERR("Cannot set cooperative level for keyboard due ", dwRet);
			Clean();
			return dwRet;
		}	
		dwRet = m_lpDIDMouse->SetCooperativeLevel(hWnd, DISCL_EXCLUSIVE|DISCL_FOREGROUND);		
		if(dwRet != DI_OK) {
			ERR("Cannot set cooperative level for mouse due ", dwRet);
			Clean();
			return dwRet;
		}
		//	
		// Acquire mouse and keyboard
		dwRet = m_lpDIDKeyboard->Acquire();		
		if(dwRet != DI_OK) {
			ERR("Cannot acquire keyboard due ", dwRet);
			Clean();
			return dwRet;
		}
		
		dwRet = m_lpDIDMouse->Acquire();		
		if(dwRet != DI_OK) {
			ERR("Cannot acquire mouse due ", dwRet);
			Clean();
			return dwRet;
		}		
		//
		m_csResolution = *csResolution;
		//
		// Create new cursor
		dwRet = m_theCursor.Create(_S_CURSOR, 32, 32, lpDevice);
		if(dwRet != DI_OK) {
			ERR("Cannot create cursor due ", dwRet);
			Clean();
			return dwRet;
		}
		//
		// Pocatecni vystredeni kurzoru
		m_ptCursor = D3DXVECTOR2(m_csResolution.x/2, m_csResolution.y/2);

		m_bInit = TRUE;
	}
	return dwRet;
}

HRESULT	XInput::ProcessInput()
{
	HRESULT dwRet = 1;
	if(m_bInit) {
		//
		// Get state of keyboard and save it to internal structure (array)
		m_lpDIDKeyboard->GetDeviceState(sizeof(m_arKeyboard), &m_arKeyboard);
		//
		// Update cursor if is showed
		if(m_bShowCursor) {
			//
			// Get relative position of cursor.
			// Save state of mouse.
			m_lpDIDMouse->GetDeviceState(sizeof(m_MouseState) , &m_MouseState);			
			//
			// Add or substract position of cursor
			MoveCursor(m_MouseState.lX, m_MouseState.lY);
			//
			// Bounds cursor on the screen
			if(m_ptCursor.x < 0) m_ptCursor.x = 0;
			if(m_ptCursor.x > m_csResolution.x) m_ptCursor.x = m_csResolution.x;
			if(m_ptCursor.y < 0) m_ptCursor.y = 0;
			if(m_ptCursor.y > m_csResolution.y) m_ptCursor.y = m_csResolution.y;		
		}		
	}
	return dwRet;
}

HRESULT XInput::UpdateCursor()
{
	HRESULT dwRet = 1;
	if(m_bInit && m_bShowCursor) {

		m_theCursor.Update(&m_ptCursor);	
	}
	return dwRet;
}

BOOL	XInput::IsRButtonDown(BOOL _Handling)
{
	if(m_bInit) {
		//
		// Checking left mouse button
		if(m_MouseState.rgbButtons[MOUSEBUTTON_RIGHT]) {
			//
			// If handling is not set, set it
			if(_Handling) {
				//
				// User press mouse button...handling activated
				if(!m_IsHandled[MOUSEBUTTON_RIGHT]) {	
					m_IsHandled[MOUSEBUTTON_RIGHT] = TRUE;					
					return TRUE;
				}
				else {
					//
					// Return false because handling is already set.
					return FALSE;				
				}
			}
			else {
				//
				// User does not want hadling, simply return TRUE
				return TRUE;					
			}
			
		}
		else {
			//
			// Reset handling because mouse button is up
			m_IsHandled[MOUSEBUTTON_RIGHT] = FALSE;
			//			
			// Button is up so return FALSE
			return FALSE;	
		}
	}
	//
	// DirectInput is not initialized, return FALSE
	return FALSE;	
}
BOOL	XInput::IsLButtonDown(BOOL _Handling)
{
	if(m_bInit) {
		//
		// Checking left mouse button
		if(m_MouseState.rgbButtons[MOUSEBUTTON_LEFT]) {
			//
			// If handling is not set, set it
			if(_Handling) {
				//
				// User press mouse button...handling activated
				if(!m_IsHandled[MOUSEBUTTON_LEFT]) {	
					m_IsHandled[MOUSEBUTTON_LEFT] = TRUE;
					m_bButtonDown = TRUE;
					return TRUE;
				}
				else {
					//
					// Return false because handling is already set.
					return FALSE;				
				}
			}
			else {
				//
				// User does not want hadling, simply return TRUE
				m_bButtonDown = TRUE;
				return TRUE;					
			}
			
		}
		else {
			//
			// Reset handling because mouse button is up
			m_IsHandled[MOUSEBUTTON_LEFT] = FALSE;			
			//			
			// Button is up so return FALSE
			return FALSE;	
		}
	}
	//
	// DirectInput is not initialized, return FALSE
	return FALSE;
}

BOOL	XInput::IsRButtonUp()
{
	if(m_bInit) {
		if(m_bButtonDown && !m_MouseState.rgbButtons[MOUSEBUTTON_RIGHT]) {
			m_bButtonDown = FALSE;
			return TRUE;
		}
		
		//return (m_MouseState.rgbButtons[MOUSEBUTTON_RIGHT]) ? FALSE : TRUE;
	}
	
	return FALSE;
}

BOOL	XInput::IsLButtonUp()
{
	if(m_bInit) {
		if(m_bButtonDown && !m_MouseState.rgbButtons[MOUSEBUTTON_LEFT]) {
			m_bButtonDown = FALSE;
			return TRUE;
		}
		//return (m_MouseState.rgbButtons[MOUSEBUTTON_LEFT]) ? FALSE : TRUE;
	}
	
	return FALSE;	
}

BOOL	XInput::IsKeyDown(int Key, BOOL bHandle)
{
	if(m_bInit) {
		//
		// Check validity of input param
		if(Key >= 256) {		
			return FALSE;
		}
		//
		// Checking specified key
		if(KEYDOWN(m_arKeyboard, Key)) {
			if(bHandle) {
				//
				// User press key button...handling activated
				if(!m_IsHandled[Key]) {
					m_IsHandled[Key] = TRUE;
					return TRUE;
				}
				else {
					//
					// Return false because handling is already set.
					return FALSE;				
				}
			}
			//
			// User does not want handling, simply return TRUE
			else {
				return TRUE;
			}			
		}
		//
		// Key is up, reset handling, return FALSE
		else {
			m_IsHandled[Key] = FALSE;
			return FALSE;
			
		}
	}
	//
	// DirectInput is not initialized, return FALSE
	return FALSE;	
}


HRESULT XInput::RestoreDevices()
{
    DWORD dwRet = 1; // INPUT IS NOT INIT
    if(m_bInit) {
        dwRet = m_lpDIDKeyboard->Acquire();
        if(dwRet != DI_OK) {
            ERR("Cannot reacquire the keyboard due ", dwRet);
            return dwRet;
        }
        dwRet = m_lpDIDMouse->Acquire();
        if(dwRet != DI_OK) {
            ERR("Cannot reacquire the mouse due ", dwRet);
            return dwRet;
        }
    }
    return dwRet;
}


