// DisplaySetup.cpp : implementation file
//

#include "stdafx.h"
#include "Setup.h"
#include "DisplaySetup.h"

#include "../common/common.h"

#define		ALLOWED_RESOLUTIONS_COUNT	7
#define		FORMATS_COUNT				13
///
// List of allowed resolutions
CSize	cAllowedResolutions[ALLOWED_RESOLUTIONS_COUNT] = 
{		
	CSize(640, 400),
	CSize(640, 480),  
	CSize(800, 600),
	CSize(1024, 768), 
	CSize(1280, 1024),
	CSize(1600, 1200)  
};

struct	Format {

	int		x, y;
	BOOL	bHasAlpha;

	Format(int nx, int ny, BOOL nbHasAlpha) {
		x = nx; y = ny; bHasAlpha = nbHasAlpha;
	}
};

Format cFormats[FORMATS_COUNT] = 
{	
	Format(20, 24, FALSE),
	Format(21, 32, TRUE),
	Format(22, 32, FALSE),
	Format(23, 16, FALSE),
	Format(24, 16, FALSE),
	Format(25, 16, FALSE),
	Format(26, 16, TRUE),
	Format(27, 8, FALSE),
	Format(28, 8, TRUE),
	Format(29, 16, TRUE),
	Format(30, 16, FALSE),
	Format(31, 32, TRUE),
	Format(34, 32, FALSE)
};

// CDisplaySetup dialog
IMPLEMENT_DYNAMIC(CDisplaySetup, CPropertyPage)
CDisplaySetup::CDisplaySetup()
	: CPropertyPage(CDisplaySetup::IDD)		
	, m_bWindowedMode(FALSE)
	, m_bFPSCounter(FALSE)
	, m_bWireframe(FALSE)
	, m_iDesiredFPS(0)
	, m_bVSync(FALSE)
{
	m_psp.dwFlags &= ~PSP_HASHELP;  // Lose the Help button
}

CDisplaySetup::~CDisplaySetup()
{
	

}

void CDisplaySetup::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_ADAPTER, m_ctrlAdapterList);
	DDX_Control(pDX, IDC_RESOLUTIONS, m_ctrlDisplayModes);
	DDX_Control(pDX, ID_ADAPTER_DESC, m_ctrlAdapterDesc);
	DDX_Check(pDX, IDC_WINDOWED, m_bWindowedMode);
	DDX_Check(pDX, IDC_FPS, m_bFPSCounter);
	DDX_Check(pDX, IDC_WIREFRAME, m_bWireframe);
	DDX_Text(pDX, IDC_MAXFPS, m_iDesiredFPS);
	DDX_Check(pDX, IDC_VSZNC, m_bVSync);
	DDX_Control(pDX, IDC_TLSUPPORT, m_ctrlTLSupport);
	DDX_Control(pDX, IDC_FSAA, m_ctrlFSAAList);
	DDX_Control(pDX, IDC_TEXFORMAT, m_ctrlTextureFormat);
	DDX_Control(pDX, IDC_TEXFILTER, m_ctrlTextureFilter);
}


BEGIN_MESSAGE_MAP(CDisplaySetup, CPropertyPage)
	ON_WM_DESTROY()
	ON_CBN_SELCHANGE(IDC_ADAPTER, OnCbnSelchangeAdapter)
	ON_LBN_SELCHANGE(IDC_RESOLUTIONS, OnLbnSelchangeResolutions)
	ON_BN_CLICKED(IDC_TEST, OnBnClickedTest)
	ON_CBN_SELCHANGE(IDC_TEXFORMAT, OnCbnSelchangeTexformat)
	ON_CBN_SELCHANGE(IDC_TEXFILTER, OnCbnSelchangeTexfilter)
	ON_CBN_SELCHANGE(IDC_TLSUPPORT, OnCbnSelchangeTlsupport)
	ON_CBN_SELCHANGE(IDC_FSAA, OnCbnSelchangeFsaa)
	ON_BN_CLICKED(IDC_WINDOWED, OnBnClickedWindowed)
END_MESSAGE_MAP()


// CDisplaySetup message handlers

BOOL CDisplaySetup::OnInitDialog()
{
 	CPropertyPage::OnInitDialog();

	//	
	// As first, create D3D object	
	m_lpD3DObject = Direct3DCreate8(D3D_SDK_VERSION);
	if(!m_lpD3DObject) {		
//		THROW("Cannot create Direct3D object!");		
	}
	//
	// Then lod data from ini file to data struct
	LoadSettings();	
	//	
	// Get list of adapters
	GetAdapters();
	// Get device caps
	GetDeviceCaps();
	//
	// Fill dialog with data
	GetDisplayModes();	
	GetTextureFormats();
	GetFSAA();
	GetTextureFilter();
	UpdateDialog(FALSE);
	//
	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

/*
 *	Fill list of adapters
 */
void	CDisplaySetup::GetAdapters()
{
	D3DADAPTER_IDENTIFIER8 Identifier;
	int iAdapterSelection = 0;
	//
	// Reset list before refilling it
	m_ctrlAdapterList.ResetContent();
	//
	// Try to get all apdaters
	for(int i = 0; i < (int)m_lpD3DObject->GetAdapterCount(); i++) {
		//
		// Get adapter informations		
		m_lpD3DObject->GetAdapterIdentifier(i, 0, &Identifier);
		//
		// Add string to the list
		int iIndex = m_ctrlAdapterList.AddString(Identifier.Description);
		//
		// Save ordinal of each adapter
		m_ctrlAdapterList.SetItemData(iIndex, i);
		//
		// Remember position of selected adapter
		if(m_CurrentDisplaySettings.AdapterOrdinal == i) {
			iAdapterSelection = iIndex;
		}
	}
	// Select saved adapter
	m_ctrlAdapterList.SetCurSel(iAdapterSelection);	
}

void	CDisplaySetup::GetDisplayModes() 
{
	int iCurSel = m_ctrlAdapterList.GetCurSel();
	int iSelectedAdapter = (int)m_ctrlAdapterList.GetItemData(iCurSel);
	D3DDISPLAYMODE* pDisplayMode;
	TCHAR	szMode[MAX_PATH];
	BOOL	bAddRes;
	int		iFormat;
	int		iIndex;
	int		iSelection = 0;
	//
	// Delete previous array of modes
	DestroyModesList();
	//
	// Check all modes
	for(int i = 0; i < (int)m_lpD3DObject->GetAdapterModeCount(iSelectedAdapter); i++) {
		//
		// Create new mode
		pDisplayMode = new D3DDISPLAYMODE;
		//
		// Get mode desc.
        m_lpD3DObject->EnumAdapterModes(iSelectedAdapter, i, pDisplayMode);
		//
		// Set res to no add to list
		bAddRes = FALSE;
		//
		// Check if the founded resolution is in allowed res array
		for(int r = 0; r < ALLOWED_RESOLUTIONS_COUNT; r++) {

			if(pDisplayMode->Width == cAllowedResolutions[r].cx &&
			   pDisplayMode->Height == cAllowedResolutions[r].cy) {
				   //
				   // Resolution is allowed, so add it
				   bAddRes = TRUE;		
				   break;
			}
		}
		//
		// Add resoltion
		if(bAddRes) {	
			// Find depth in bit of screen format
			for(int f = 0; f < FORMATS_COUNT; f++) {
				if(cFormats[f].x == pDisplayMode->Format) {
					iFormat = cFormats[f].y;
				}
			}
			// Format user text
			sprintf(szMode, "%dx%dx%d@%d", pDisplayMode->Width, pDisplayMode->Height, iFormat, pDisplayMode->RefreshRate);
			//
			// Add string to list
			iIndex = m_ctrlDisplayModes.AddString(szMode);
			//
			// Remember display mode in array
			m_ctrlDisplayModes.SetItemDataPtr(iIndex, pDisplayMode);
			//
			// Get selected mode in the list
			if(m_CurrentDisplaySettings.DisplayMode.Width == pDisplayMode->Width) {
				if(m_CurrentDisplaySettings.DisplayMode.Height == pDisplayMode->Height) {
					if(m_CurrentDisplaySettings.DisplayMode.Format == pDisplayMode->Format) {
						if(m_CurrentDisplaySettings.DisplayMode.RefreshRate == pDisplayMode->RefreshRate) {
							iSelection = iIndex;
						}
					}
				}
			}
		}
		else {
			SAFE_DELETE(pDisplayMode);
		}
	}
	// Select saved mode
	m_ctrlDisplayModes.SetCurSel(iSelection);
}

/*
	Bilinear filtering - linear magnification, linear minification, point mipmapping. 
	Trilinear filtering - linear magnification, linear minification, linear mipmapping. 
	Bilinear Anisotropic filtering - anisotropic magnification, anisotropic minification, point mipmapping. 
	Trilinear Anisotropic filtering - anisotropic magnification, anisotropic minification, linear
	mipmapping.
*/
void	CDisplaySetup::GetTextureFilter()
{	
	int iIndex;	
	//
	// Reset filters list
	m_ctrlTextureFilter.ResetContent();
	// Add filtering support
	if(m_DeviceCaps.TextureFilterCaps & D3DPTFILTERCAPS_MIPFPOINT
		&& m_DeviceCaps.TextureFilterCaps & D3DPTFILTERCAPS_MINFLINEAR
		&& m_DeviceCaps.TextureFilterCaps & D3DPTFILTERCAPS_MAGFLINEAR) {
		// Add string
        iIndex = m_ctrlTextureFilter.AddString("Bilinear");		
		//
		// Set filtering structure
		FilteringData	*pFData1 = new FilteringData;
		pFData1->MagFilterType = D3DTEXF_LINEAR;
		pFData1->MinFilterType = D3DTEXF_LINEAR;
		pFData1->MipFilterType = D3DTEXF_POINT;
		// Set poiner to control
		m_ctrlTextureFilter.SetItemDataPtr(iIndex, (void*)pFData1);
		
		if(m_CurrentDisplaySettings.Filtering.MipFilterType == D3DTEXF_POINT &&
			m_CurrentDisplaySettings.Filtering.MagFilterType == D3DTEXF_LINEAR) {
			m_ctrlTextureFilter.SetCurSel(iIndex);
		}
	}
	if(m_DeviceCaps.TextureFilterCaps & D3DPTFILTERCAPS_MIPFLINEAR
		&& m_DeviceCaps.TextureFilterCaps & D3DPTFILTERCAPS_MINFLINEAR
		&& m_DeviceCaps.TextureFilterCaps & D3DPTFILTERCAPS_MAGFLINEAR) {
		// Add string
        iIndex = m_ctrlTextureFilter.AddString("Trilinear");		
		//
		// Set filtering structure
		FilteringData	*pFData2 = new FilteringData;
		pFData2->MagFilterType = D3DTEXF_LINEAR;
		pFData2->MinFilterType = D3DTEXF_LINEAR;
		pFData2->MipFilterType = D3DTEXF_LINEAR;
		// Set poiner to control
		m_ctrlTextureFilter.SetItemDataPtr(iIndex, (void*)pFData2);
		
		if(m_CurrentDisplaySettings.Filtering.MipFilterType == D3DTEXF_LINEAR &&
			m_CurrentDisplaySettings.Filtering.MagFilterType == D3DTEXF_LINEAR) {
			m_ctrlTextureFilter.SetCurSel(iIndex);
		}       
	}
	if(m_DeviceCaps.RasterCaps & D3DPRASTERCAPS_ANISOTROPY &&
		m_DeviceCaps.TextureFilterCaps & D3DPTFILTERCAPS_MIPFPOINT) {

		// Add string
        iIndex = m_ctrlTextureFilter.AddString("Bilinear Anisotropic");		
		//
		// Set filtering structure
		FilteringData	*pFData3 = new FilteringData;
		pFData3->MagFilterType = D3DTEXF_ANISOTROPIC;
		pFData3->MinFilterType = D3DTEXF_ANISOTROPIC;
		pFData3->MipFilterType = D3DTEXF_POINT;
		// Set poiner to control
		m_ctrlTextureFilter.SetItemDataPtr(iIndex, (void*)pFData3);
		
		if(m_CurrentDisplaySettings.Filtering.MipFilterType == D3DTEXF_POINT &&
			m_CurrentDisplaySettings.Filtering.MagFilterType == D3DTEXF_ANISOTROPIC) {
			m_ctrlTextureFilter.SetCurSel(iIndex);
		}             
	}
	if(m_DeviceCaps.RasterCaps & D3DPRASTERCAPS_ANISOTROPY) {
		// Add string
        iIndex = m_ctrlTextureFilter.AddString("Trilinear Anisotropic");		
		//
		// Set filtering structure
		FilteringData	*pFData4 = new FilteringData;
		pFData4->MagFilterType = D3DTEXF_ANISOTROPIC;
		pFData4->MinFilterType = D3DTEXF_ANISOTROPIC;
		pFData4->MipFilterType = D3DTEXF_LINEAR;
		// Set poiner to control
		m_ctrlTextureFilter.SetItemDataPtr(iIndex, (void*)pFData4);
		
		if(m_CurrentDisplaySettings.Filtering.MipFilterType == D3DTEXF_LINEAR &&
			m_CurrentDisplaySettings.Filtering.MagFilterType == D3DTEXF_ANISOTROPIC) {
			m_ctrlTextureFilter.SetCurSel(iIndex);
		}            
	}
}

void	CDisplaySetup::GetFSAA()
{
	TCHAR	szMultisample[MAX_PATH];
	DWORD	dwRet;
	//
	// Reset control
	m_ctrlFSAAList.ResetContent();
	m_ctrlFSAAList.EnableWindow(TRUE);
	//
	// Refill list
	for(int ff = 0; ff < 17; ff++) {		
		//
		// Check multisample mode
		dwRet = m_lpD3DObject->CheckDeviceMultiSampleType(m_CurrentDisplaySettings.AdapterOrdinal, 
												  m_CurrentDisplaySettings.DeviceType, 
												  m_CurrentDisplaySettings.DisplayMode.Format, 
												  m_CurrentDisplaySettings.bWindowed, (D3DMULTISAMPLE_TYPE )ff);
		if(dwRet == S_OK) {			
			//
			// Refill control and save values of FSAA
			// Build string to add
			if(ff == 0) {
				strcpy(szMultisample, "Multisampling off");
			}	
			else {
				sprintf(szMultisample, "Multisampling %dx", ff);
			}
			//
			// Add new item to list box
			int p = m_ctrlFSAAList.AddString(szMultisample);
			//
			// Save multisample level
			m_ctrlFSAAList.SetItemData(p, (DWORD)ff);
			//
			// If this FSAA mode is in setup.ini, set it in list box
			if(m_CurrentDisplaySettings.MultisampleType == ff) {
				m_ctrlFSAAList.SetCurSel(p);				
			}			
		}
	}	
	//
	// If none sampling is supprted, disable sampling combo box
	if(m_ctrlFSAAList.GetCount() == 1) {
		m_ctrlFSAAList.ResetContent();
		m_ctrlFSAAList.AddString("Not supported");
		m_ctrlFSAAList.SetCurSel(0);
		m_ctrlFSAAList.EnableWindow(FALSE);
	}
}

void	CDisplaySetup::GetTextureFormats()
{
	DWORD dwRet;
	TCHAR szToAdd[MAX_PATH];
	int i;
	int iCurTexFormat = cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_TEXDEPTH);
	//
	// Reset control
	m_ctrlTextureFormat.ResetContent();
	//
	// Refill it, check each format
	for(int f = 0; f < FORMATS_COUNT; f++) {

		if(cFormats[f].bHasAlpha) {
			dwRet = m_lpD3DObject->CheckDeviceFormat(m_CurrentDisplaySettings.AdapterOrdinal, 
													m_CurrentDisplaySettings.DeviceType,
													m_CurrentDisplaySettings.DisplayMode.Format
													, 0, D3DRTYPE_TEXTURE, 
													(D3DFORMAT)cFormats[f].x);
			if(dwRet == D3D_OK) {
				sprintf(szToAdd, "%d-bit", cFormats[f].y);
				i = m_ctrlTextureFormat.AddString(szToAdd);
				m_ctrlTextureFormat.SetItemData(i, cFormats[f].x);
				if(iCurTexFormat == cFormats[f].x) {
					m_ctrlTextureFormat.SetCurSel(i);
				}
			}					
		}
	}

}

void	CDisplaySetup::GetDeviceCaps()
{
	int iCurSel, iIndex;
	iCurSel = 0;
	// Get current support selection from ini
	if(m_CurrentDisplaySettings.dwBehavior & D3DCREATE_SOFTWARE_VERTEXPROCESSING) {
		iCurSel = 0;
	}
	else {
		if(m_CurrentDisplaySettings.dwBehavior & D3DCREATE_HARDWARE_VERTEXPROCESSING) {
			iCurSel = 1;	
			if(m_CurrentDisplaySettings.dwBehavior & D3DCREATE_PUREDEVICE) {
				iCurSel = 2;
			}
		}	
	}
	//
	// Reset TL support control
	m_ctrlTLSupport.ResetContent();
	//
	// Get device caps of selected adapter (try HAL device)	
	DWORD dwRet = m_lpD3DObject->GetDeviceCaps(m_CurrentDisplaySettings.AdapterOrdinal, D3DDEVTYPE_HAL, &m_DeviceCaps);		
	if(m_DeviceCaps.DeviceType == 0) {				
		// Try REF device
		dwRet = m_lpD3DObject->GetDeviceCaps(m_CurrentDisplaySettings.AdapterOrdinal, D3DDEVTYPE_REF, &m_DeviceCaps);			
		if(dwRet == S_OK) {
			// Set that device is REF
			m_ctrlAdapterDesc.SetWindowText("Device type: REF");						
			iIndex = m_ctrlTLSupport.AddString("Simulated Software TnL");	
			m_ctrlTLSupport.SetItemData(iIndex, D3DCREATE_SOFTWARE_VERTEXPROCESSING);
			//
			// Add simulated hw t&l
			if(m_DeviceCaps.DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT) {
				// Add hardware support if supported
				iIndex = m_ctrlTLSupport.AddString("Simulated Hardware TnL");		
				m_ctrlTLSupport.SetItemData(iIndex, D3DCREATE_HARDWARE_VERTEXPROCESSING);
			}
			m_CurrentDisplaySettings.DeviceType = D3DDEVTYPE_REF;
		}
	}	
	else {		
		// Set that device is HAL
		m_ctrlAdapterDesc.SetWindowText("Device type: HAL");
		// Add software support
		iIndex = m_ctrlTLSupport.AddString("Software TnL");
		m_ctrlTLSupport.SetItemData(iIndex, D3DCREATE_SOFTWARE_VERTEXPROCESSING);
		//
		// Check hardware caps
		if(m_DeviceCaps.DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT) {
			// Add hardware support if supported
			iIndex = m_ctrlTLSupport.AddString("Hardware TnL");
			m_ctrlTLSupport.SetItemData(iIndex, D3DCREATE_HARDWARE_VERTEXPROCESSING);
			//
			// Check hw pure device
			if(m_DeviceCaps.DevCaps & D3DDEVCAPS_PUREDEVICE) {
				iIndex = m_ctrlTLSupport.AddString("Pure Hardware TnL");
				m_ctrlTLSupport.SetItemData(iIndex, D3DCREATE_PUREDEVICE|D3DCREATE_HARDWARE_VERTEXPROCESSING);
			}
		}
		m_CurrentDisplaySettings.DeviceType = D3DDEVTYPE_HAL;
	}
	// Set current support from ini file if available
	if(m_ctrlTLSupport.GetCount() >= 3) {
		m_ctrlTLSupport.SetCurSel(iCurSel);
	}
	else {
		m_ctrlTLSupport.SetCurSel(0);
	}
	//
	// Throw exception if error occurs
	if(dwRet != D3D_OK) {		
//		THROWERR("Cannot get device caps due " , dwRet);		
	}	
}

void	CDisplaySetup::DestroyModesList()
{
	D3DDISPLAYMODE* pDisplayMode;
	for(int i = 0; i < m_ctrlDisplayModes.GetCount(); i++) {
		pDisplayMode = (D3DDISPLAYMODE*) m_ctrlDisplayModes.GetItemDataPtr(i);
		SAFE_DELETE(pDisplayMode);
	}
	m_ctrlDisplayModes.ResetContent();
}

void CDisplaySetup::OnDestroy()
{	
	//
	// Fill struct with dialog data
	UpdateDialog(TRUE);
	//
	// Destroy lists
	DestroyModesList();
	//
	// Release D3D object
	SAFE_RELEASE(m_lpD3DObject);
	//
	// Deletes tex filtering
	int i = 0;
	FilteringData *pFData;
	while((pFData = (FilteringData*) m_ctrlTextureFilter.GetItemDataPtr(i++)) != (FilteringData*)-1) {
		SAFE_DELETE(pFData);
	}
	//
	// Destroy window
	CPropertyPage::OnDestroy();
}


/*
 *	Reads display data form setupco robis<.in file to
 *	DisplayData structure
 */

void	CDisplaySetup::LoadSettings()
{
	m_CurrentDisplaySettings.AdapterOrdinal = cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_ADAPTER);
	m_CurrentDisplaySettings.DeviceType = (D3DDEVTYPE)cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_DEVICE);

	m_CurrentDisplaySettings.Filtering.MagFilterType = (D3DTEXTUREFILTERTYPE)cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_MAGTEXTUREFILTER);
	m_CurrentDisplaySettings.Filtering.MinFilterType = (D3DTEXTUREFILTERTYPE)cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_MINTEXTUREFILTER);
	m_CurrentDisplaySettings.Filtering.MipFilterType = (D3DTEXTUREFILTERTYPE)cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_MIPTEXTUREFILTER);

	m_CurrentDisplaySettings.TextureFormat = (D3DFORMAT)cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_TEXDEPTH);
	m_CurrentDisplaySettings.MultisampleType = (D3DMULTISAMPLE_TYPE)cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_FSAA);

	m_CurrentDisplaySettings.DisplayMode.Width = (D3DFORMAT)cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_WIDTH);
	m_CurrentDisplaySettings.DisplayMode.Height = (D3DFORMAT)cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_HEIGHT);
	m_CurrentDisplaySettings.DisplayMode.RefreshRate = cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_REFRESHRATE);
	m_CurrentDisplaySettings.DisplayMode.Format = (D3DFORMAT)cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_FORMAT);

	m_CurrentDisplaySettings.dwBehavior = cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_BEHAVIOR);
	m_CurrentDisplaySettings.bWireframe = cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_WIREFRAME);
	m_CurrentDisplaySettings.bFPSCounter = cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_FPS);
	m_CurrentDisplaySettings.bWindowed = cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_WINDOWED);
	m_CurrentDisplaySettings.bVSync = cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_VSYNC);
	m_CurrentDisplaySettings.iFPSLimit = cmnReadSetupInt(_S_SECTION_DISPLAY, _S_KEY_DESIREDFPS);
}

/*
 *	Save setting to setup.ini file
 */
void	CDisplaySetup::SaveSettings()
{	
	//
	// Save device informations
	cmnWriteSetupInt(_S_SECTION_DISPLAY, _S_KEY_ADAPTER, m_CurrentDisplaySettings.AdapterOrdinal);
	cmnWriteSetupInt(_S_SECTION_DISPLAY, _S_KEY_DEVICE, m_CurrentDisplaySettings.DeviceType);
	//
	// Save display informations
	cmnWriteSetupInt(_S_SECTION_DISPLAY, _S_KEY_WIDTH, m_CurrentDisplaySettings.DisplayMode.Width);
	cmnWriteSetupInt(_S_SECTION_DISPLAY, _S_KEY_HEIGHT, m_CurrentDisplaySettings.DisplayMode.Height);
	cmnWriteSetupInt(_S_SECTION_DISPLAY, _S_KEY_FORMAT, m_CurrentDisplaySettings.DisplayMode.Format);
	cmnWriteSetupInt(_S_SECTION_DISPLAY, _S_KEY_REFRESHRATE, m_CurrentDisplaySettings.DisplayMode.RefreshRate);
	//
	// Save other settings
	cmnWriteSetupInt(_S_SECTION_DISPLAY, _S_KEY_WIREFRAME, m_CurrentDisplaySettings.bWireframe);
	cmnWriteSetupInt(_S_SECTION_DISPLAY, _S_KEY_FPS, m_CurrentDisplaySettings.bFPSCounter);
	cmnWriteSetupInt(_S_SECTION_DISPLAY, _S_KEY_WINDOWED, m_CurrentDisplaySettings.bWindowed);
	cmnWriteSetupInt(_S_SECTION_DISPLAY, _S_KEY_VSYNC, m_CurrentDisplaySettings.bVSync);
	cmnWriteSetupInt(_S_SECTION_DISPLAY, _S_KEY_DESIREDFPS, m_CurrentDisplaySettings.iFPSLimit);
	cmnWriteSetupInt(_S_SECTION_DISPLAY, _S_KEY_BEHAVIOR, m_CurrentDisplaySettings.dwBehavior);
	cmnWriteSetupInt(_S_SECTION_DISPLAY, _S_KEY_FSAA, m_CurrentDisplaySettings.MultisampleType);
	cmnWriteSetupInt(_S_SECTION_DISPLAY, _S_KEY_TEXDEPTH, m_CurrentDisplaySettings.TextureFormat);

	cmnWriteSetupInt(_S_SECTION_DISPLAY, _S_KEY_MAGTEXTUREFILTER, m_CurrentDisplaySettings.Filtering.MagFilterType);
	cmnWriteSetupInt(_S_SECTION_DISPLAY, _S_KEY_MINTEXTUREFILTER, m_CurrentDisplaySettings.Filtering.MinFilterType);
	cmnWriteSetupInt(_S_SECTION_DISPLAY, _S_KEY_MIPTEXTUREFILTER, m_CurrentDisplaySettings.Filtering.MipFilterType);	
}

/* 
 *	Get data from display struct to dialog or get data from dialog to struct
 */
void	CDisplaySetup::UpdateDialog(BOOL bSaveAndValidate)
{
	int iCurSel;
	
	if(bSaveAndValidate) {
		UpdateData(TRUE);

		m_CurrentDisplaySettings.bWindowed = m_bWindowedMode;
		m_CurrentDisplaySettings.bFPSCounter = m_bFPSCounter;
		m_CurrentDisplaySettings.bWireframe = m_bWireframe;
		m_CurrentDisplaySettings.iFPSLimit = m_iDesiredFPS;
		m_CurrentDisplaySettings.bVSync = m_bVSync;

		iCurSel = m_ctrlAdapterList.GetCurSel();
		m_CurrentDisplaySettings.AdapterOrdinal = (int)m_ctrlAdapterList.GetItemData(iCurSel);
	
		m_CurrentDisplaySettings.dwBehavior = (DWORD)m_ctrlTLSupport.GetItemData(m_ctrlTLSupport.GetCurSel());

		iCurSel = m_ctrlFSAAList.GetCurSel();
		m_CurrentDisplaySettings.MultisampleType = (D3DMULTISAMPLE_TYPE)m_ctrlFSAAList.GetItemData(iCurSel);
 
		iCurSel = m_ctrlTextureFormat.GetCurSel();
		m_CurrentDisplaySettings.TextureFormat = (D3DFORMAT)m_ctrlTextureFormat.GetItemData(iCurSel);

		iCurSel = m_ctrlTextureFilter.GetCurSel();
		FilteringData * pFData = (FilteringData*)m_ctrlTextureFilter.GetItemDataPtr(iCurSel);
		if(pFData) {
			m_CurrentDisplaySettings.Filtering.MagFilterType = pFData->MagFilterType;
			m_CurrentDisplaySettings.Filtering.MinFilterType = pFData->MinFilterType;
			m_CurrentDisplaySettings.Filtering.MipFilterType = pFData->MipFilterType;
		}

		iCurSel = m_ctrlDisplayModes.GetCurSel();
		m_CurrentDisplaySettings.DisplayMode = *((D3DDISPLAYMODE*) m_ctrlDisplayModes.GetItemDataPtr(iCurSel));
	}
	else {
		m_bWindowedMode = m_CurrentDisplaySettings.bWindowed;
		m_bFPSCounter = m_CurrentDisplaySettings.bFPSCounter;
		m_bWireframe = m_CurrentDisplaySettings.bWireframe;
		m_iDesiredFPS = m_CurrentDisplaySettings.iFPSLimit;
		m_bVSync = m_CurrentDisplaySettings.bVSync;

		UpdateData(FALSE);
	}
}

/*
 *	Tests current display configuration
 *	-> Try to create Direct3D device
 */
void CDisplaySetup::OnBnClickedTest()
{
	LPDIRECT3DDEVICE8		lpDevice;
	D3DPRESENT_PARAMETERS	d3dDeviceParam;
	HWND	hWnd;
	DWORD   dwRet;

	// Download current settings from dialog
	UpdateDialog(TRUE);
	//
	// Create temporary window
	hWnd = CreateWindow("STATIC", "Testing window", WS_OVERLAPPEDWINDOW, 0, 0, 
		m_CurrentDisplaySettings.DisplayMode.Width, 
		m_CurrentDisplaySettings.DisplayMode.Height, 
		NULL, NULL, AfxGetApp()->m_hInstance , NULL);
	if(!hWnd) {
		ERR("Cannot create testing window due ", GetLastError());
		return;
	}		
	// 
	// Set properties of device	
	ZeroMemory(&d3dDeviceParam, sizeof(d3dDeviceParam));
	d3dDeviceParam.Windowed							= m_CurrentDisplaySettings.bWindowed;
	d3dDeviceParam.SwapEffect						= D3DSWAPEFFECT_DISCARD;
    d3dDeviceParam.BackBufferWidth					= m_CurrentDisplaySettings.DisplayMode.Width;
    d3dDeviceParam.BackBufferHeight					= m_CurrentDisplaySettings.DisplayMode.Height;
    d3dDeviceParam.BackBufferFormat					= m_CurrentDisplaySettings.DisplayMode.Format;
	d3dDeviceParam.BackBufferCount	 				= 1;	
	d3dDeviceParam.hDeviceWindow					= hWnd;
	d3dDeviceParam.MultiSampleType					= D3DMULTISAMPLE_NONE;// Default no antialiasing
	d3dDeviceParam.EnableAutoDepthStencil			= TRUE;
	d3dDeviceParam.AutoDepthStencilFormat			= D3DFMT_D16;

	//
	// Set size of back buffer from window position
	if(m_CurrentDisplaySettings.bWindowed) {	
		D3DDISPLAYMODE displaymode;
		// Get current display mode
		m_lpD3DObject->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &displaymode);
		// Set display mode to current
		d3dDeviceParam.BackBufferFormat	= displaymode.Format;        
	}
	else {
		//
		// -> PresentationInterval
		if(m_CurrentDisplaySettings.bVSync == 0) {
			d3dDeviceParam.FullScreen_PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;
		}
		else {
			d3dDeviceParam.FullScreen_PresentationInterval = D3DPRESENT_INTERVAL_ONE;
		}
		//
		// -> RefreshRate
		d3dDeviceParam.FullScreen_RefreshRateInHz = m_CurrentDisplaySettings.DisplayMode.RefreshRate;
		//
		// -> Fullscreen AA
		d3dDeviceParam.MultiSampleType = m_CurrentDisplaySettings.MultisampleType;
	}
	//
	// Try to create device
	dwRet = m_lpD3DObject->CreateDevice(m_CurrentDisplaySettings.AdapterOrdinal, 
										m_CurrentDisplaySettings.DeviceType, hWnd,
										 m_CurrentDisplaySettings.dwBehavior,	&d3dDeviceParam, &lpDevice);
	// Destry window
	::DestroyWindow(hWnd);
	// Release device
	SAFE_RELEASE(lpDevice);

	if(dwRet != D3D_OK) {
		cmnMessageBoxA("Settings are incorrect. Change some setings and try test again.", MSG_ERROR);		
	}
	else {
		cmnMessageBoxA("Settings are correct.", MSG_INFO);		
	}
}

// CONTROLS NOTIFACTIONS

void CDisplaySetup::OnCbnSelchangeAdapter()
{
	//
	// Get new adapter info
	GetDeviceCaps();
	//
	// Refill dialog because of another adapter
	GetDisplayModes();	
	GetTextureFormats();
	GetFSAA();
	GetTextureFilter();
	//
	// Save new adapter info
	UpdateDialog(TRUE);
}

// If user change resolution, get new texture fomrat for display format
void CDisplaySetup::OnLbnSelchangeResolutions()
{
	//
	// Refill texture formats
	GetTextureFormats();
	//
	// Save new mode
	UpdateDialog(TRUE);
}


void CDisplaySetup::OnCbnSelchangeTexformat()
{
	//
	// Save new texture format
	UpdateDialog(TRUE);
}


void CDisplaySetup::OnCbnSelchangeTexfilter()
{
	//
	// Save new texture filter
	UpdateDialog(TRUE);
}


void CDisplaySetup::OnCbnSelchangeTlsupport()
{	
	//
	// Save new tl support
	UpdateDialog(TRUE);
}

void CDisplaySetup::OnCbnSelchangeFsaa()
{
	//
	// Save new FSAA mode
	UpdateDialog(TRUE);
}


void CDisplaySetup::OnBnClickedWindowed()
{
	//
	// Save new FSAA mode
	UpdateDialog(TRUE);
	//
	// Refill FSAA combo box
	GetFSAA();
}
