// Tester.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "Tester.h"
#define MAX_LOADSTRING 100

#include "../common/common.h"
#include "../display/common.h"
#include "../input/common.h"

XDisplay		g_theDisplay;
XInput			g_theInput;
HWND			g_hWnd;

XMesh			g_Sphere;
XMesh			g_Box;
XMesh			g_Cylinder;
XMesh			g_Torus;
XMesh			g_Teapot;
XMesh			g_Tiger;
XMesh			g_Airplane;
X3DObject		g_Quad;

D3DLIGHT8		g_PointLight;
D3DLIGHT8		g_Sun;

//
// Address mode
DWORD	g_dwAddressMode;

#define		_S_DEBUG			"Common"
#define		_S_TRACETOFILE		"TraceToFile"

void	UpdateFrame();
void	RestoreUserObjects();
void	FillBuffers();

// Global Variables:
HINSTANCE hInst;								// current instance
TCHAR szTitle[MAX_LOADSTRING];					// The title bar text
TCHAR szWindowClass[MAX_LOADSTRING];			// the main window class name

// Forward declarations of functions included in this code module:
ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	About(HWND, UINT, WPARAM, LPARAM);

int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
 	// TODO: Place code here.
	MSG msg;
	HACCEL hAccelTable;

	// Initialize global strings
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	LoadString(hInstance, IDC_TESTER, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance);
	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_TESTER);

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow)) 
	{
		return FALSE;
	}

	// inicializace logovaciho souboru, pokud je zapnuta funkce Tracetofile
	if(cmnReadSetupInt(_S_DEBUG, _S_TRACETOFILE) == 1) {
		cmnInitLog("log.txt", FALSE);
	}

	// inicializace Direct3D
	g_theDisplay.Init(g_hWnd);
	g_theInput.Init(hInstance, g_hWnd, g_theDisplay.GetResolution(), g_theDisplay.GetDevice());

	g_theDisplay.GetCamera()->InitCamera(g_theDisplay.GetDevice(), -20, 20);

	// preddefinovane objekty
	g_Sphere.CreateSphere(2.0f, 48, 48, g_theDisplay.GetDevice());	
	g_Sphere.Translate(4.0f, 0.0f, -30.0f);
	g_Sphere.ScaleXYZ(0.1f);
	g_Box.CreateBox(1.0f, 1.0f, 6.0f, g_theDisplay.GetDevice());
	g_Cylinder.CreateCylinder(1.5f, 2.5f, 4.0f, 24, 8, g_theDisplay.GetDevice());
	g_Torus.CreateTorus(0.5f, 2.0f, 64, 64, g_theDisplay.GetDevice());
	g_Teapot.CreateTeapot(g_theDisplay.GetDevice());	
	g_Teapot.RotateX(D3DX_PI/2);
	g_Teapot.ScaleXYZ(0.5f);
	g_Teapot.Translate(4.5f, 0.0f, 0.0f);
	// model tygra
	g_Tiger.LoadMeshFromFile("tiger.x", g_theDisplay.GetDevice(), g_theDisplay.GetResourceManager());
	g_Tiger.RotateX(D3DX_PI/2);
	g_Tiger.ScaleY(0.5f);
	g_Tiger.Translate(0.0f, 0.0f, 4.0f);
	g_Airplane.LoadMeshFromFile("airplane 2.x", g_theDisplay.GetDevice(), g_theDisplay.GetResourceManager());
	g_Airplane.ScaleXYZ(0.1f);
	g_Airplane.Translate(6.0f, 0.0f, 0.0f);
	g_Airplane.RotateX(D3DX_PI/2);
	
	// zaregistrovani vsech zdroju
	g_theDisplay.GetResourceManager()->AddMesh(&g_Tiger);
	g_theDisplay.GetResourceManager()->AddMesh(&g_Sphere);
	g_theDisplay.GetResourceManager()->AddMesh(&g_Box);
	g_theDisplay.GetResourceManager()->AddMesh(&g_Cylinder);
	g_theDisplay.GetResourceManager()->AddMesh(&g_Teapot);
	g_theDisplay.GetResourceManager()->AddMesh(&g_Torus);
	g_theDisplay.GetResourceManager()->AddMesh(&g_Airplane);

	g_Quad.Init(g_theDisplay.GetDevice(), "smile.bmp", 5, g_theDisplay.GetTextureFormat(), 4 * sizeof(VERTEX), 6 * sizeof(WORD), IB_WRITEONLY|VB_WRITEONLY);
	g_Quad.Visible(FALSE);
	g_dwAddressMode = D3DTADDRESS_WRAP;
	FillBuffers();
	g_theDisplay.GetResourceManager()->AddIndexBuffer(g_Quad.GetIB());
	g_theDisplay.GetResourceManager()->AddVertexBuffer(g_Quad.GetVB());
	g_theDisplay.GetResourceManager()->AddTexture(g_Quad.GetTexture());
		
	// torus is visible by default
	g_Torus.Visible();

	// zde vytvorime slunicko
	ZeroMemory( &g_Sun, sizeof(D3DLIGHT8) );
	g_Sun.Type = D3DLIGHT_DIRECTIONAL;
	g_Sun.Direction.x = -1.0f;
	g_Sun.Direction.y = 0.0f;
	g_Sun.Direction.z = -1.0f;
	g_Sun.Diffuse.r  = 0.1f;
	g_Sun.Diffuse.g  = 0.0f;
	g_Sun.Diffuse.b  = 0.6f;
	g_Sun.Ambient.r  = 0.4f;
	g_Sun.Ambient.g  = 0.4f;
	g_Sun.Ambient.b  = 0.4f;
	g_Sun.Range = 1000.0f;	
	g_theDisplay.SetLight(0, &g_Sun);
	g_theDisplay.EnableLight(0, TRUE);

	// dalsi svetlo
	ZeroMemory( &g_PointLight, sizeof(D3DLIGHT8) );
	g_PointLight.Type = D3DLIGHT_POINT;
	g_PointLight.Position.x = 0.0f;
	g_PointLight.Position.y = 0.0f;
	g_PointLight.Position.z = 0.0f;
	g_PointLight.Diffuse.r  = 1.0f;
	g_PointLight.Diffuse.g  = 0.3f;
	g_PointLight.Diffuse.b  = 0.3f;
	g_PointLight.Range = 1000.0f;
	g_PointLight.Attenuation1 = 1.0f;	

	// Main message loop:
	while( TRUE )
    {
        // Look for messages, if none are found then 
        // update the state and display it
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            if( 0 == GetMessage(&msg, NULL, 0, 0 ) )
            {
                // WM_QUIT was posted, so exit
                return (int)msg.wParam;
            }
			
            // Translate and dispatch the message
            TranslateMessage( &msg ); 
            DispatchMessage( &msg );
        }
        else
        {
			// Obnova snimku
			UpdateFrame();			
		}
    }	
	return (int) msg.wParam;
}


void	FillBuffers()
{
	VERTEX *pVertices;
	g_Quad.GetVB()->GetBuffer()->Lock(0, 0, (BYTE**) &pVertices, 0);

	pVertices[0].vecPos = D3DXVECTOR3(-4.0f, -4.0f, 2.0f);
	pVertices[1].vecPos = D3DXVECTOR3(4.0f, -4.0f, 2.0f);
	pVertices[2].vecPos = D3DXVECTOR3(-4.0f, 4.0f, 2.0f);
	pVertices[3].vecPos = D3DXVECTOR3(4.0f, 4.0f, 2.0f);

	pVertices[0].vecNormal = D3DXVECTOR3(0.0f, 0.0f, 1.0f);
	pVertices[1].vecNormal = D3DXVECTOR3(0.0f, 0.0f, 1.0f);
	pVertices[2].vecNormal = D3DXVECTOR3(0.0f, 0.0f, 1.0f);
	pVertices[3].vecNormal = D3DXVECTOR3(0.0f, 0.0f, 1.0f);

	pVertices[0].tu1 = 0.0f;
	pVertices[0].tv1 = 0.0f;

	pVertices[1].tu1 = 3.0f;
	pVertices[1].tv1 = 0.0f;

	pVertices[2].tu1 = 0.0f;
	pVertices[2].tv1 = 3.0f;

	pVertices[3].tu1 = 3.0f;
	pVertices[3].tv1 = 3.0f;

	pVertices[0].dwDiffuse = 0xFFFFFFFF;
	pVertices[1].dwDiffuse = 0xFFFFFFFF;
	pVertices[2].dwDiffuse = 0xFFFFFFFF;
	pVertices[3].dwDiffuse = 0xFFFFFFFF;

	g_Quad.GetVB()->GetBuffer()->Unlock();


	WORD *pIndices;
	g_Quad.GetIB()->GetBuffer()->Lock(0, 0, (BYTE**) &pIndices, 0);

	pIndices[0] = 0;
	pIndices[1] = 1;
	pIndices[2] = 2;

	pIndices[3] = 1;
	pIndices[4] = 3;
	pIndices[5] = 2;

	g_Quad.GetIB()->GetBuffer()->Unlock();
}
//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    This function and its usage are only necessary if you want this code
//    to be compatible with Win32 systems prior to the 'RegisterClassEx'
//    function that was added to Windows 95. It is important to call this function
//    so that the application will get 'well formed' small icons associated
//    with it.
//
ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_TESTER);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_SMALL);

	return RegisterClassEx(&wcex);
}

//
//   FUNCTION: InitInstance(HANDLE, int)
//
//   PURPOSE: Saves instance handle and creates main window
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{  
   hInst = hInstance; // Store instance handle in our global variable

   g_hWnd = CreateWindow(szWindowClass, szTitle, WS_OVERLAPPEDWINDOW,
      CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);

   if (!g_hWnd)
   {
      return FALSE;
   }

   ShowWindow(g_hWnd, nCmdShow);
   UpdateWindow(g_hWnd);

   return TRUE;
}

//
//  FUNCTION: WndProc(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for the main window.
//
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) 
	{	
	case WM_KEYUP:    
        switch( wParam )
        {
            case VK_F9:
				g_theDisplay.RestoreDisplay();
				RestoreUserObjects();
				FillBuffers();
				break;
			case VK_F1:
				g_Sphere.Visible(!g_Sphere.IsVisible());				
				g_Quad.Visible(FALSE);				
				break;
			case VK_F2:
				g_Torus.Visible(!g_Torus.IsVisible());				
				g_Quad.Visible(FALSE);
				break;
			case VK_F3:
				g_Box.Visible(!g_Box.IsVisible());
				g_Quad.Visible(FALSE);
				break;
			case VK_F4:
				g_Cylinder.Visible(!g_Cylinder.IsVisible());
				g_Quad.Visible(FALSE);
				break;
			case VK_F5:
				g_Teapot.Visible(!g_Teapot.IsVisible());
				g_Quad.Visible(FALSE);
				break;
			case VK_F6:
				g_Tiger.Visible(!g_Tiger.IsVisible());
				g_Quad.Visible(FALSE);
				break;
			case VK_F7:
				g_Airplane.Visible(!g_Airplane.IsVisible());
				g_Quad.Visible(FALSE);
				break;
			case VK_F8:
				g_Torus.Visible(FALSE);
				g_Sphere.Visible(FALSE);
				g_Box.Visible(FALSE);
				g_Tiger.Visible(FALSE);
				g_Airplane.Visible(FALSE);
				g_Teapot.Visible(FALSE);
				g_Cylinder.Visible(FALSE);
				g_Quad.Visible(TRUE);
				break;
			case VK_SPACE:
				if(g_Quad.IsVisible())
				{
					g_Quad.EnableTransformation(!g_Quad.IsTransformationEnabled());
				}
				break;
			case VK_ESCAPE:
				PostQuitMessage(0);
				break;
		}
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}


void	UpdateFrame()
{
	DWORD dwOldAdrMode;
	float fFactor = cmnGetTime(TIMER_GETELAPSEDTIME1);

	g_theInput.ProcessInput();
	g_theDisplay.GetCamera()->ProcessCamera(fFactor);
	// svetlo se bude pohybovat po kruznici
	g_PointLight.Position.x = 0.0f;
	g_PointLight.Position.y = float(4.0f * sin(double(GetTickCount())/300.0f));
	g_PointLight.Position.z = float(5.0f * cos(double(GetTickCount())/300.0f));
	g_theDisplay.SetLight(1, &g_PointLight);
	g_theDisplay.EnableLight(1, TRUE);
	// draw scene
	g_theDisplay.UpdateBackground();
	g_theDisplay.GetDevice()->BeginScene();				
	g_Sphere.Draw();
	g_Torus.Draw();
	g_Box.Draw();
	g_Cylinder.Draw();
	g_Teapot.Draw();
	// tygr nema definovane normalove vektory, takze je treba vypnout osvetleni
	g_Airplane.Draw();
	g_theDisplay.GetDevice()->SetRenderState(D3DRS_LIGHTING, FALSE);
	g_Tiger.Draw();

	// Get old addr mode
	g_theDisplay.GetDevice()->GetTextureStageState(0, D3DTSS_ADDRESSU, &dwOldAdrMode);
	//
	// Change address mode
	if(g_theInput.IsKeyDown(DIK_Q)) 
	{
		g_dwAddressMode = D3DTADDRESS_WRAP;
	}
	if(g_theInput.IsKeyDown(DIK_W)) 
	{
		g_dwAddressMode = D3DTADDRESS_MIRROR;
	}
	if(g_theInput.IsKeyDown(DIK_E)) 
	{
		g_dwAddressMode = D3DTADDRESS_CLAMP;
	}
	if(g_theInput.IsKeyDown(DIK_R)) 
	{
		g_dwAddressMode = D3DTADDRESS_BORDER;
		g_theDisplay.GetDevice()->SetTextureStageState(0, D3DTSS_BORDERCOLOR, 0x00FF0000); // red color
	}
	if(g_theInput.IsKeyDown(DIK_ADD , FALSE)) {
		g_theDisplay.GetCamera()->ZoomIn();
	}
	if(g_theInput.IsKeyDown(DIK_SUBTRACT , FALSE)) {
		g_theDisplay.GetCamera()->ZoomOut();
	}
	if(g_theInput.IsKeyDown(DIK_NUMPAD1  , FALSE)) {
		g_theDisplay.GetCamera()->RotateLeft();
	}
	if(g_theInput.IsKeyDown(DIK_NUMPAD3  , FALSE)) {
		g_theDisplay.GetCamera()->RotateRight();
	}
	if(g_theInput.IsKeyDown(DIK_NUMPAD8  , FALSE)) {
		g_theDisplay.GetCamera()->IncreaseZAngle();
	}
	if(g_theInput.IsKeyDown(DIK_NUMPAD2  , FALSE)) {
		g_theDisplay.GetCamera()->DecreaseZAngle();
	}

	int iMouseZone = int(g_theDisplay.GetResolution()->x * 5 / 1024); 
	if(g_theInput.IsKeyDown(DIK_UP, FALSE)) {
		g_theDisplay.GetCamera()->MoveUp();
	} else {		
	if(g_theInput.GetCursor()->y < iMouseZone) {
		g_theDisplay.GetCamera()->MoveUp();			
	}}
	if(g_theInput.IsKeyDown(DIK_LEFT  , FALSE)) {
		g_theDisplay.GetCamera()->MoveLeft();
	} else {		
	if(g_theInput.GetCursor()->x < iMouseZone) {
		g_theDisplay.GetCamera()->MoveLeft();		
	}}
	iMouseZone += int( g_theDisplay.GetResolution()->x * 32 / 1024); 
	if(g_theInput.IsKeyDown(DIK_DOWN  , FALSE)) {
		g_theDisplay.GetCamera()->MoveDown();		
	} else {
	if(g_theInput.GetCursor()->y >=  g_theDisplay.GetResolution()->y-iMouseZone) {
		g_theDisplay.GetCamera()->MoveDown();		
	}}
	if(g_theInput.IsKeyDown(DIK_RIGHT  , FALSE)) {
		g_theDisplay.GetCamera()->MoveRight();	
	} else {
	if(g_theInput.GetCursor()->x >= g_theDisplay.GetResolution()->x-iMouseZone) {
		g_theDisplay.GetCamera()->MoveRight();		
	}}


	// Change address mode
	g_theDisplay.GetDevice()->SetTextureStageState(0, D3DTSS_ADDRESSU, g_dwAddressMode);
	g_theDisplay.GetDevice()->SetTextureStageState(0, D3DTSS_ADDRESSV, g_dwAddressMode);
	g_Quad.Draw(D3DPT_TRIANGLELIST, 0, 4, 0, 2);
	g_theDisplay.GetDevice()->SetRenderState(D3DRS_LIGHTING, TRUE);	
	// Restore addr modes
	g_theDisplay.GetDevice()->SetTextureStageState(0, D3DTSS_ADDRESSU, dwOldAdrMode);
	g_theDisplay.GetDevice()->SetTextureStageState(0, D3DTSS_ADDRESSV, dwOldAdrMode);
	//
	// Compute transformations	
	if(fFactor != 0) {			
		g_Torus.RotateY(0.8f * fFactor);
		g_Cylinder.RotateX(1.0f * fFactor);
		g_Box.RotateZ(3.0f * fFactor);	
		g_Teapot.RotateZ(-0.1f * fFactor);
		g_Airplane.RotateZ(1.8f * fFactor);
		g_Tiger.RotateZ(5.8f * fFactor);
		g_Sphere.RotateY(6.0f * fFactor);
		g_Quad.RotateZ(0.9f * fFactor);		
	}	
	g_theInput.UpdateCursor();
	g_theDisplay.GetDevice()->EndScene();
	
	if(D3DERR_DEVICENOTRESET == g_theDisplay.Present())
	{
		RestoreUserObjects();
		FillBuffers();
	}
}

void	RestoreUserObjects()
{
	g_theDisplay.SetLight(0, &g_Sun);
	g_theDisplay.EnableLight(0, TRUE);
}